@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: BaseClient.cpp,v 1.44 2007-11-28 20:22:42 alriddoch Exp $
+
 #include "BaseClient.h"
 
 #include "CreatorClient.h"
@@ -31,6 +33,7 @@
 using Atlas::Message::MapType;
 using Atlas::Objects::Root;
 using Atlas::Objects::Operation::Login;
+using Atlas::Objects::Operation::Logout;
 using Atlas::Objects::Operation::Create;
 using Atlas::Objects::Operation::RootOperation;
 using Atlas::Objects::Entity::RootEntity;
@@ -48,6 +51,10 @@ BaseClient::~BaseClient()
 {
 }
 
+/// \brief Create a new account on the server
+///
+/// @param name User name of the new account
+/// @param password Password of the new account
 MapType BaseClient::createPlayer(const std::string & name,
                                  const std::string & password)
 {
@@ -90,6 +97,11 @@ MapType BaseClient::createPlayer(const std::string & name,
     return ent->asMessage();
 }
 
+/// \brief Create a new Character or avatar on the server
+///
+/// Requires that an account is already logged in.
+/// @param type The type of avatar to be created
+/// @return The CreatorClient object used to directly interact with the avatar
 CreatorClient * BaseClient::createCharacter(const std::string & type)
 {
     Anonymous character;
@@ -121,10 +133,10 @@ CreatorClient * BaseClient::createCharacter(const std::string & type)
     long intId = integerId(id);
 
     if (intId == -1) {
-        log(ERROR, String::compose("Invalid character ID \"%1\" from server.", id).c_str());
+        log(ERROR, String::compose("Invalid character ID \"%1\" from server.", id));
     }
 
-    CreatorClient * obj = new CreatorClient(id, intId, type, m_connection);
+    CreatorClient * obj = new CreatorClient(id, intId, m_connection);
     obj->merge(ent->asMessage());
     // FIXME We are making no attempt to set LOC, as we have no entity to
     // set it to.
@@ -137,11 +149,22 @@ CreatorClient * BaseClient::createCharacter(const std::string & type)
     return obj;
 }
 
+void BaseClient::logout()
+{
+    Logout logout;
+    send(logout);
+
+    if (m_connection.wait() != 0) {
+        std::cerr << "ERROR: Failed to logout" << std::endl << std::flush;
+    }
+}
+
 // I'm making this pure virtual, to see if that is desired.
 //void BaseClient::idle() {
 //    time.sleep(0.1);
 //}
 
+/// \brief Handle any operations that have arrived from the server
 void BaseClient::handleNet()
 {
     RootOperation input;
