@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: CharacterClient.cpp,v 1.30 2007-11-28 20:22:42 alriddoch Exp $
+
 #include "CharacterClient.h"
 
 #include "ClientConnection.h"
@@ -25,29 +27,33 @@
 
 static const bool debug_flag = false;
 
+/// \brief CharacterClient constructor
+///
+/// @param id String identifier
+/// @param intId Integer identifier
+/// @param name The name of the avatar used by this remote agent
+/// @param c The network connection to the server used for communication
 CharacterClient::CharacterClient(const std::string & id, long intId,
-                                 const std::string & name,
                                  ClientConnection & c) :
-                                 BaseMind(id, intId, name), m_connection(c)
-{
-}
-
-void CharacterClient::sightImaginaryOperation(const Operation &,
-                                              Operation &, OpVector &)
-{
-}
-
-void CharacterClient::soundTalkOperation(const Operation & ,
-                                         Operation &, OpVector &)
+                                 BaseMind(id, intId), m_connection(c)
 {
 }
 
+/// \brief Send an operation to the server from this avatar
+///
+/// @param op Operation to be sent
 void CharacterClient::send(const Operation & op)
 {
     op->setFrom(getId());
     m_connection.send(op);
 }
 
+/// \brief Send an operation to the server, and wait for a reply
+///
+/// Reply is identified as it should have its refno attribute set to
+/// the serialno of the operation sent.
+/// @param op Operation to be sent
+/// @param res Result with correct refno is returned here
 int CharacterClient::sendAndWaitReply(const Operation & op, OpVector & res)
 {
     send(op);
@@ -56,7 +62,7 @@ int CharacterClient::sendAndWaitReply(const Operation & op, OpVector & res)
     long no = op->getSerialno();
     while (true) {
         if (m_connection.pending()) {
-            Operation input = CharacterClient::m_connection.pop();
+            Operation input = m_connection.pop();
             assert(input.isValid());
             if (input.isValid()) {
                 // FIXME What the hell is this!
