@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: CharacterClient.h,v 1.18 2007-11-28 20:22:42 alriddoch Exp $
+
 #ifndef CLIENT_CHARACTER_CLIENT_H
 #define CLIENT_CHARACTER_CLIENT_H
 
@@ -29,14 +31,7 @@ class CharacterClient : public BaseMind {
 
     int sendAndWaitReply(const Operation &, OpVector &);
   public:
-    CharacterClient(const std::string &, long, const std::string&, ClientConnection&);
-
-    virtual void sightImaginaryOperation(const Operation &,
-                                         Operation &,
-                                         OpVector &);
-    virtual void soundTalkOperation(const Operation &,
-                                    Operation &,
-                                    OpVector &);
+    CharacterClient(const std::string &, long, ClientConnection&);
 
     void send(const Operation & op);
 };
