@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: BaseEntity.cpp,v 1.101 2007-07-29 12:22:58 alriddoch Exp $
+
 #include "BaseEntity.h"
 
 #include "log.h"
@@ -22,12 +24,10 @@
 #include "op_switch.h"
 #include "serialno.h"
 #include "inheritance.h"
+#include "compose.hpp"
 
 #include "Add.h"
 #include "Attack.h"
-#include "Burn.h"
-#include "Chop.h"
-#include "Cut.h"
 #include "Eat.h"
 #include "Nourish.h"
 #include "Setup.h"
@@ -47,8 +47,12 @@ using Atlas::Objects::Entity::RootEntity;
 
 static const bool debug_flag = false;
 
+/// \brief BaseEntity constructor
+///
+/// @param id String identifier
+/// @param intId Integer identifier
 BaseEntity::BaseEntity(const std::string & id, long intId) : m_id(id),
-                                                            m_intId(intId)
+                                                             m_intId(intId)
 {
 }
 
@@ -56,14 +60,16 @@ BaseEntity::~BaseEntity()
 {
 }
 
-void BaseEntity::addToMessage(MapType & omap) const
+/// \brief Copy the attribute values of this object to an Atlas Message
+void BaseEntity::addToMessage(Atlas::Message::MapType & omap) const
 {
     debug( std::cout << "BaseEntity::addToMessage" << std::endl << std::flush;);
     omap["objtype"] = "obj";
     omap["id"] = getId();
 }
 
-void BaseEntity::addToEntity(const RootEntity & ent) const
+/// \brief Copy the attribute values of this object to an Atlas Entity
+void BaseEntity::addToEntity(const Atlas::Objects::Entity::RootEntity & ent) const
 {
     debug( std::cout << "BaseEntity::addToEntity" << std::endl << std::flush;);
     ent->setObjtype("obj");
@@ -96,14 +102,6 @@ void BaseEntity::LogoutOperation(const Operation & op, OpVector & res)
 {
 }
 
-/// \brief Process a Action operation.
-///
-/// @param op The operation to be processed.
-/// @param res The result of the operation is returned here.
-void BaseEntity::ActionOperation(const Operation & op, OpVector & res)
-{
-}
-
 /// \brief Process a Add operation.
 ///
 /// @param op The operation to be processed.
@@ -112,14 +110,6 @@ void BaseEntity::AddOperation(const Operation & op, OpVector & res)
 {
 }
 
-/// \brief Process a Chop operation.
-///
-/// @param op The operation to be processed.
-/// @param res The result of the operation is returned here.
-void BaseEntity::ChopOperation(const Operation & op, OpVector & res)
-{
-}
-
 /// \brief Process a Combine operation.
 ///
 /// @param op The operation to be processed.
@@ -136,14 +126,6 @@ void BaseEntity::CreateOperation(const Operation & op, OpVector & res)
 {
 }
 
-/// \brief Process a Cut operation.
-///
-/// @param op The operation to be processed.
-/// @param res The result of the operation is returned here.
-void BaseEntity::CutOperation(const Operation & op, OpVector & res)
-{
-}
-
 /// \brief Process a Delete operation.
 ///
 /// @param op The operation to be processed.
@@ -168,14 +150,6 @@ void BaseEntity::EatOperation(const Operation & op, OpVector & res)
 {
 }
 
-/// \brief Process a Burn operation.
-///
-/// @param op The operation to be processed.
-/// @param res The result of the operation is returned here.
-void BaseEntity::BurnOperation(const Operation & op, OpVector & res)
-{
-}
-
 /// \brief Process a Get operation.
 ///
 /// @param op The operation to be processed.
@@ -344,14 +318,13 @@ void BaseEntity::ErrorOperation(const Operation & op, OpVector & res)
 {
 }
 
-OpNo BaseEntity::opEnumerate(const Operation & op) const
-{
-    return op->getClassNo();
-}
-
+/// \brief Find and call the handler for an operation
+///
+/// @param op The operation to be processed.
+/// @param res The result of the operation is returned here.
 void BaseEntity::callOperation(const Operation & op, OpVector & res)
 {
-    const OpNo op_no = opEnumerate(op);
+    const OpNo op_no = op->getClassNo();
     OP_SWITCH(op, op_no, res,)
 }
 
@@ -363,13 +336,13 @@ void BaseEntity::callOperation(const Operation & op, OpVector & res)
 /// @param errstring A message describing the error.
 /// @param res The resulting error operation is returned here.
 /// @param to The error operation should be directed to this ID.
-void BaseEntity::error(const Operation& op, const char* errstring,
+void BaseEntity::error(const Operation& op, const std::string & errstring,
                        OpVector & res, const std::string & to) const
 {
     Error e;
 
-    std::string msg = "ERROR generated by " + getId() + " with message [" + errstring + "]";
-    log(NOTICE, msg.c_str());
+    log(NOTICE, String::compose("ERROR generated by %1 with message [%2]",
+                                getId(), errstring));
 
     std::vector<Root> & args = e->modifyArgs();
 
