@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: BaseEntity.h,v 1.90 2007-07-29 12:22:58 alriddoch Exp $
+
 #ifndef COMMON_BASE_ENTITY_H
 #define COMMON_BASE_ENTITY_H
 
@@ -40,22 +42,26 @@
 /// dictionary as they are called elsewhere in this code.
 class BaseEntity : virtual public sigc::trackable {
   private:
-    /// String id.
+    /// \brief String identifier
     const std::string m_id;
+    /// \brief Integer identifier
     const long m_intId;
 
-    // Private and un-implemented, to make sure slicing is impossible
+    /// \brief Copy constructor private and un-implemented to prevent slicing
     BaseEntity(const BaseEntity &);
+    /// \brief Assignment operator private and un-implemented to prevent slicing
     const BaseEntity & operator=(const BaseEntity &);
   protected:
     explicit BaseEntity(const std::string & id, long intId);
   public:
     virtual ~BaseEntity();
 
+    /// \brief Read only accessor for string identity
     const std::string & getId() const {
         return m_id;
     }
 
+    /// \brief Read only accessor for Integer identity
     long getIntId() const {
         return m_intId;
     }
@@ -65,15 +71,11 @@ class BaseEntity : virtual public sigc::trackable {
 
     virtual void operation(const Operation &, OpVector &);
 
-    virtual void ActionOperation(const Operation &, OpVector &);
     virtual void AddOperation(const Operation &, OpVector &);
     virtual void AppearanceOperation(const Operation &, OpVector &);
     virtual void AttackOperation(const Operation &, OpVector &);
-    virtual void BurnOperation(const Operation &, OpVector &);
-    virtual void ChopOperation(const Operation &, OpVector &);
     virtual void CombineOperation(const Operation &, OpVector &);
     virtual void CreateOperation(const Operation &, OpVector &);
-    virtual void CutOperation(const Operation &, OpVector &);
     virtual void DeleteOperation(const Operation &, OpVector &);
     virtual void DisappearanceOperation(const Operation &, OpVector &);
     virtual void DivideOperation(const Operation &, OpVector &);
@@ -100,13 +102,17 @@ class BaseEntity : virtual public sigc::trackable {
     virtual void OtherOperation(const Operation &, OpVector &);
     virtual void ErrorOperation(const Operation &, OpVector &);
 
-    OpNo opEnumerate(const Operation &) const;
     void callOperation(const Operation &, OpVector &);
-    void error(const Operation &, const char * errstring, OpVector &,
+    void error(const Operation &, const std::string & errstring, OpVector &,
                const std::string & to = "") const;
     void clientError(const Operation &, const std::string & errstring,
                      OpVector &, const std::string & to = "") const;
 
+    /// \brief Signal emitted when this entity is removed from the server
+    ///
+    /// Note that this is usually well before the object is actually deleted
+    /// and marks the conceptual destruction of the concept this entity
+    /// represents, not the destruction of this object.
     sigc::signal<void> destroyed;
 };
 
