@@ -1,5 +1,5 @@
 // Cyphesis Online RPG Server and AI Engine
-// Copyright (C) 2000-2004 Alistair Riddoch
+// Copyright (C) 2000-2007 Alistair Riddoch
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Database.cpp,v 1.99 2007-12-07 17:42:58 alriddoch Exp $
+
 #include "Database.h"
 
 #include "id.h"
@@ -36,6 +38,7 @@
 
 using Atlas::Message::Element;
 using Atlas::Message::MapType;
+using Atlas::Objects::Root;
 
 typedef Atlas::Codecs::XML Serialiser;
 
@@ -48,7 +51,7 @@ static void databaseNotice(void * arg, const char * message)
     std::string msg = std::string("DATABASE: ") + message;
     // Remove the trailing \n from the message.
     msg = msg.substr(0, msg.size() - 1);
-    log(NOTICE, msg.c_str());
+    log(NOTICE, msg);
 }
 
 Database::Database() : m_rule_db("rules"),
@@ -59,6 +62,8 @@ Database::Database() : m_rule_db("rules"),
 
 bool Database::tuplesOk()
 {
+    assert(m_connection != 0);
+
     bool status = false;
 
     PGresult * res;
@@ -73,6 +78,8 @@ bool Database::tuplesOk()
 
 bool Database::commandOk()
 {
+    assert(m_connection != 0);
+
     bool status = false;
 
     PGresult * res;
@@ -87,54 +94,100 @@ bool Database::commandOk()
     return status;
 }
 
-int Database::initConnection(bool createDatabase)
+int Database::createInstanceDatabase()
+{
+    assert(::instance != CYPHESIS);
+
+    std::string error_message;
+
+    if (connect(CYPHESIS, error_message) != 0) {
+        log(ERROR, String::compose("Connection to master database failed: \n%1",
+                                   error_message));
+        return -1;
+    }
+
+    std::string dbname;
+    if (::instance == CYPHESIS) {
+        dbname = CYPHESIS;
+    } else {
+        dbname = String::compose("%1_%2", CYPHESIS, ::instance);
+    }
+    readConfigItem(::instance, "dbname", dbname);
+
+    if (!runCommandQuery(String::compose("CREATE DATABASE %1", dbname))) {
+        shutdownConnection();
+        return -1;
+    }
+
+    shutdownConnection();
+
+    return 0;
+}
+
+int Database::connect(const std::string & context, std::string & error_msg)
 {
     std::stringstream conninfos;
-    if (global_conf->findItem("cyphesis", "dbserver")) {
-        std::string db_server(global_conf->getItem("cyphesis", "dbserver"));
+
+    std::string db_server;
+    if (readConfigItem(context, "dbserver", db_server) == 0) {
         if (db_server.empty()) {
-            log(WARNING, "Empty database hostname specified in config file. Using none.");
+            log(WARNING, "Empty database hostname specified in config file. "
+                         "Using none.");
         } else {
-            conninfos << "host=" << " ";
+            conninfos << "host=" << db_server << " ";
         }
     }
 
-    std::string dbname = "cyphesis";
-    if (global_conf->findItem("cyphesis", "dbname")) {
-        dbname = global_conf->getItem("cyphesis", "dbname").as_string();
+    std::string dbname;
+    if (context == CYPHESIS) {
+        dbname = CYPHESIS;
+    } else {
+        dbname = String::compose("%1_%2", CYPHESIS, ::instance);
     }
+    readConfigItem(context, "dbname", dbname);
     conninfos << "dbname=" << dbname << " ";
 
-    if (global_conf->findItem("cyphesis", "dbuser")) {
-        std::string db_user(global_conf->getItem("cyphesis", "dbuser"));
+    std::string db_user;
+    if (readConfigItem(context, "dbuser", db_user) == 0) {
         if (db_user.empty()) {
-            log(WARNING, "Empty username specified in config file. Using current user.");
+            log(WARNING, "Empty username specified in config file. "
+                         "Using current user.");
         } else {
             conninfos << "user=" << db_user << " ";
         }
     }
 
-    if (global_conf->findItem("cyphesis", "dbpasswd")) {
-        conninfos << "password=" << global_conf->getItem("cyphesis", "dbpasswd").as_string() << " ";
+    std::string db_passwd;
+    if (readConfigItem(context, "dbpasswd", db_passwd) == 0) {
+        conninfos << "password=" << db_passwd << " ";
     }
 
     const std::string cinfo = conninfos.str();
 
-    if (createDatabase) {
-        // Currently not able to create the database
-    }
-
-    
-
     m_connection = PQconnectdb(cinfo.c_str());
 
     if (m_connection == NULL) {
-        log(ERROR, "Database connection failed with unknown error");
+        error_msg = "Unknown error";
         return -1;
     }
 
     if (PQstatus(m_connection) != CONNECTION_OK) {
-        log(ERROR, String::compose("Connection to database failed: \"%1\"", PQerrorMessage(m_connection)).c_str());
+        error_msg = PQerrorMessage(m_connection);
+        PQfinish(m_connection);
+        m_connection = 0;
+        return -1;
+    }
+
+    return 0;
+}
+
+int Database::initConnection()
+{
+    std::string error_message;
+
+    if (connect(::instance, error_message) != 0) {
+        log(ERROR, String::compose("Connection to database failed: \n%1",
+                                   error_message));
         return -1;
     }
 
@@ -145,6 +198,8 @@ int Database::initConnection(bool createDatabase)
 
 bool Database::initRule(bool createTables)
 {
+    assert(m_connection != 0);
+
     int status = 0;
     clearPendingQuery();
     status = PQsendQuery(m_connection, "SELECT * FROM rules WHERE id = 'test' AND contents = 'test'");
@@ -179,8 +234,10 @@ bool Database::initRule(bool createTables)
 
 void Database::shutdownConnection()
 {
-    PQfinish(m_connection);
-    m_connection = 0;
+    if (m_connection != 0) {
+        PQfinish(m_connection);
+        m_connection = 0;
+    }
 }
 
 Database * Database::instance()
@@ -191,8 +248,43 @@ Database * Database::instance()
     return m_instance;
 }
 
+void Database::cleanup()
+{
+    if (m_instance != 0) {
+        delete m_instance;
+    }
+
+    m_instance = 0;
+}
+
 bool Database::decodeObject(const std::string & data,
-                            MapType &o)
+                            Root &o)
+{
+    if (data.empty()) {
+        return true;
+    }
+
+    std::stringstream str(data, std::ios::in);
+
+    Serialiser codec(str, m_od);
+    Atlas::Message::Encoder enc(codec);
+
+    // Clear the decoder
+    m_d.get();
+
+    codec.poll();
+
+    if (!m_od.check()) {
+        log(WARNING, "Database entry does not appear to be decodable");
+        return false;
+    }
+
+    o = m_od.get();
+    return true;
+}
+
+bool Database::decodeMessage(const std::string & data,
+                             MapType &o)
 {
     if (data.empty()) {
         return true;
@@ -236,6 +328,8 @@ bool Database::encodeObject(const MapType & o,
 bool Database::getObject(const std::string & table, const std::string & key,
                          MapType & o)
 {
+    assert(m_connection != 0);
+
     debug(std::cout << "Database::getObject() " << table << "." << key
                     << std::endl << std::flush;);
     std::string query = std::string("SELECT * FROM ") + table + " WHERE id = '" + key + "'";
@@ -253,7 +347,7 @@ bool Database::getObject(const std::string & table, const std::string & key,
                         << " table" << std::endl << std::flush;);
         return false;
     }
-    if ((PQntuples(res) < 1) || (PQnfields(res) < 2)) {
+    if (PQntuples(res) < 1 || PQnfields(res) < 2) {
         debug(std::cout << "No entry for " << key << " in " << table
                         << " table" << std::endl << std::flush;);
         PQclear(res);
@@ -266,7 +360,7 @@ bool Database::getObject(const std::string & table, const std::string & key,
     debug(std::cout << "Got record " << key << " from database, value " << data
                     << std::endl << std::flush;);
 
-    bool ret = decodeObject(data, o);
+    bool ret = decodeMessage(data, o);
     PQclear(res);
 
     while ((res = PQgetResult(m_connection)) != NULL) {
@@ -293,7 +387,7 @@ bool Database::putObject(const std::string & table,
     enc.streamMessageElement(o);
     codec.streamEnd();
 
-    debug(std::cout << "Encoded to: " << str.str().c_str() << " "
+    debug(std::cout << "Encoded to: " << str.str() << " "
                << str.str().size() << std::endl << std::flush;);
     std::string query = std::string("INSERT INTO ") + table + " VALUES ('" + key;
     StringVector::const_iterator Iend = c.end();
@@ -347,6 +441,8 @@ bool Database::delObject(const std::string & table, const std::string & key)
 
 bool Database::hasKey(const std::string & table, const std::string & key)
 {
+    assert(m_connection != 0);
+
     std::string query = std::string("SELECT id FROM ") + table +
                         " WHERE id='" + key + "'";
 
@@ -376,8 +472,14 @@ bool Database::hasKey(const std::string & table, const std::string & key)
     return ret;
 }
 
-bool Database::getTable(const std::string & table, MapType &o)
+bool Database::getTable(const std::string & table,
+                        std::map<std::string, Root> & contents)
 {
+    if (m_connection == 0) {
+        log(CRITICAL, "Database connection is down. This is okay during tests");
+        return false;
+    }
+
     std::string query = std::string("SELECT * FROM ") + table;
 
     clearPendingQuery();
@@ -395,7 +497,7 @@ bool Database::getTable(const std::string & table, MapType &o)
         return false;
     }
     int results = PQntuples(res);
-    if ((results < 1) || (PQnfields(res) < 2)) {
+    if (results < 1 || PQnfields(res) < 2) {
         debug(std::cout << "No entries in " << table
                         << " table" << std::endl << std::flush;);
         PQclear(res);
@@ -412,7 +514,7 @@ bool Database::getTable(const std::string & table, MapType &o)
         return false;
     }
 
-    MapType t;
+    Root t;
     for(int i = 0; i < results; ++i) {
         const char * key = PQgetvalue(res, i, id_column);
         const char * data = PQgetvalue(res, i, contents_column);
@@ -420,7 +522,7 @@ bool Database::getTable(const std::string & table, MapType &o)
                    << data << std::endl << std::flush;);
 
         if (decodeObject(data, t)) {
-            o[key] = t;
+            contents[key] = t;
         }
 
     }
@@ -442,6 +544,8 @@ bool Database::clearTable(const std::string & table)
 
 void Database::reportError()
 {
+    assert(m_connection != 0);
+
     char * message = PQerrorMessage(m_connection);
     assert(message != NULL);
 
@@ -450,11 +554,13 @@ void Database::reportError()
     }
     std::string msg = std::string("DATABASE: ") + message;
     msg = msg.substr(0, msg.size() - 1);
-    log(ERROR, msg.c_str());
+    log(ERROR, msg);
 }
 
 const DatabaseResult Database::runSimpleSelectQuery(const std::string & query)
 {
+    assert(m_connection != 0);
+
     debug(std::cout << "QUERY: " << query << std::endl << std::flush;);
     clearPendingQuery();
     int status = PQsendQuery(m_connection, query.c_str());
@@ -489,6 +595,8 @@ const DatabaseResult Database::runSimpleSelectQuery(const std::string & query)
 
 bool Database::runCommandQuery(const std::string & query)
 {
+    assert(m_connection != 0);
+
     clearPendingQuery();
     int status = PQsendQuery(m_connection, query.c_str());
     if (!status) {
@@ -498,7 +606,7 @@ bool Database::runCommandQuery(const std::string & query)
     }
     if (!commandOk()) {
         log(ERROR, "Error running command query row.");
-        log(NOTICE, query.c_str());
+        log(NOTICE, query);
         reportError();
         debug(std::cout << "Row query didn't work"
                         << std::endl << std::flush;);
@@ -514,6 +622,8 @@ bool Database::registerRelation(std::string & tablename,
                                 const std::string & targettable,
                                 RelationType kind)
 {
+    assert(m_connection != 0);
+
     tablename = sourcetable + "_" + targettable;
 
     std::string query = "SELECT * FROM ";
@@ -522,7 +632,7 @@ bool Database::registerRelation(std::string & tablename,
 
     std::string createquery = "CREATE TABLE ";
     createquery += tablename;
-    if ((kind == OneToOne) || (kind == ManyToOne)) {
+    if (kind == OneToOne || kind == ManyToOne) {
         createquery += " (source integer UNIQUE REFERENCES ";
     } else {
         createquery += " (source integer REFERENCES ";
@@ -530,7 +640,7 @@ bool Database::registerRelation(std::string & tablename,
     createquery += sourcetable;
 #if 0
     // FIXME Referential integrity not supported on inherited tables.
-    if ((kind == OneToOne) || (kind == OneToMany)) {
+    if (kind == OneToOne || kind == OneToMany) {
         createquery += " (id), target integer UNIQUE REFERENCES ";
     } else {
         createquery += " (id), target integer REFERENCES ";
@@ -538,7 +648,7 @@ bool Database::registerRelation(std::string & tablename,
     createquery += targettable;
     createquery += " (id))";
 #else
-    if ((kind == OneToOne) || (kind == OneToMany)) {
+    if (kind == OneToOne || kind == OneToMany) {
         createquery += " (id), target integer UNIQUE) WITHOUT OIDS";
     } else {
         createquery += " (id), target integer) WITHOUT OIDS";
@@ -570,7 +680,7 @@ bool Database::registerRelation(std::string & tablename,
     }
     allTables.insert(tablename);
 #if 0
-    if ((kind == ManyToOne) || (kind == OneToOne)) {
+    if (kind == ManyToOne || kind == OneToOne) {
         return true;
     } else {
         std::string indexQuery = "CREATE INDEX ";
@@ -639,6 +749,8 @@ bool Database::removeRelationRowByOther(const std::string & name,
 bool Database::registerSimpleTable(const std::string & name,
                                    const MapType & row)
 {
+    assert(m_connection != 0);
+
     if (row.empty()) {
         log(ERROR, "Attempt to create empty database table");
     }
@@ -778,6 +890,8 @@ bool Database::updateSimpleRow(const std::string & name,
 
 bool Database::registerEntityIdGenerator()
 {
+    assert(m_connection != 0);
+
     clearPendingQuery();
     int status = PQsendQuery(m_connection, "SELECT * FROM entity_ent_id_seq");
     if (!status) {
@@ -798,6 +912,8 @@ bool Database::registerEntityIdGenerator()
 
 long Database::newId(std::string & id)
 {
+    assert(m_connection != 0);
+
     clearPendingQuery();
     int status = PQsendQuery(m_connection,
                              "SELECT nextval('entity_ent_id_seq')");
@@ -833,10 +949,14 @@ bool Database::registerEntityTable(const std::string & classname,
 // row probably needs to be richer to provide a more detailed, and possibly
 // ordered description of each the columns required.
 {
+    if (m_connection == 0) {
+        log(CRITICAL, "Database connection is down. This is okay during tests");
+        return false;
+    }
+
     if (entityTables.find(classname) != entityTables.end()) {
-        log(ERROR, "Attempt to register entity table already registered.");
-        debug(std::cerr << "Table for class " << classname
-                        << " already registered." << std::endl << std::flush;);
+        log(ERROR, String::compose("Attempt to register entity table \"%1\" "
+                                   "which is already registered.", classname));
         return false;
     }
     if (!parent.empty()) {
@@ -1026,7 +1146,8 @@ const DatabaseResult Database::selectEntityRow(const std::string & id,
     std::string table = (classname == "" ? "entity" : classname);
     TableDict::const_iterator I = entityTables.find(classname);
     if (I == entityTables.end()) {
-        log(ERROR, "Attempt to select from entity table not registered.");
+        log(ERROR, String::compose("Attempt to select from entity table \"%1\" "
+                                   "which is not registered.", classname));
         return DatabaseResult(0);
     }
     std::string query = "SELECT * FROM ";
@@ -1045,7 +1166,8 @@ const DatabaseResult Database::selectOnlyByLoc(const std::string & loc,
 {
     TableDict::const_iterator I = entityTables.find(classname);
     if (I == entityTables.end()) {
-        log(ERROR, "Attempt to select from entity table not registered.");
+        log(ERROR, String::compose("Attempt to select from entity table \"%1\" "
+                                   "which is not registered.", classname));
         return DatabaseResult(0);
     }
 
@@ -1087,6 +1209,12 @@ bool Database::registerArrayTable(const std::string & name,
                                   unsigned int dimension,
                                   const MapType & row)
 {
+    if (m_connection == 0) {
+        log(CRITICAL, "Database connection is down. This is okay during tests");
+        return false;
+    }
+
+
     assert(dimension <= 5);
 
     if (row.empty()) {
@@ -1351,6 +1479,8 @@ void Database::queryComplete()
 
 bool Database::launchNewQuery()
 {
+    assert(m_connection != 0);
+
     if (m_queryInProgress) {
         log(ERROR, "Launching new query when query is in progress");
         return false;
@@ -1508,5 +1638,5 @@ void DatabaseResult::const_iterator::readColumn(const char * column,
         return;
     }
     const char * v = PQgetvalue(m_dr.m_res, m_row, col_num);
-    Database::instance()->decodeObject(v, val);
+    Database::instance()->decodeMessage(v, val);
 }
