@@ -15,8 +15,17 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Property.cpp,v 1.13 2007-01-14 21:55:22 alriddoch Exp $
+
 #include "Property_impl.h"
 
+/// \brief Classes that define properties on in world entities
+///
+/// Property classes handle the values of Atlas attributes on in
+/// game entitie, ensuring type safety, and encapsulating certain
+/// behavoirs related to the presence and value of the attribute.
+/// \defgroup PropertyClasses Entity Property Classes
+
 /// \brief Constructor called from classes which inherit from Property
 /// @param flags default value for the Property flags
 PropertyBase::PropertyBase(unsigned int flags) : m_flags(flags)
@@ -96,4 +105,8 @@ template class Property<float>;
 template class Property<double>;
 template class Property<std::string>;
 
+template class ImmutableProperty<int>;
+template class ImmutableProperty<long>;
+template class ImmutableProperty<float>;
+template class ImmutableProperty<double>;
 template class ImmutableProperty<std::string>;
