@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Property.h,v 1.17 2006-12-27 00:32:33 alriddoch Exp $
+
 #ifndef COMMON_PROPERTY_H
 #define COMMON_PROPERTY_H
 
@@ -24,7 +26,10 @@
 #include <sigc++/trackable.h>
 #include <sigc++/signal.h>
 
+
 /// \brief Interface for Entity properties
+///
+/// \ingroup PropertyClasses
 class PropertyBase {
   protected:
     /// \brief Flags indicating how this Property should be handled
@@ -37,13 +42,13 @@ class PropertyBase {
     unsigned int flags() const { return m_flags; }
 
     /// \brief Copy the value of the property into an Atlas Message
-    virtual bool get(Atlas::Message::Element &) const = 0;
+    virtual bool get(Atlas::Message::Element & val) const = 0;
     /// \brief Read the value of the property from an Atlas Message
-    virtual void set(const Atlas::Message::Element &) = 0;
+    virtual void set(const Atlas::Message::Element & val) = 0;
     /// \brief Add the value as an attribute to an Atlas map
-    virtual void add(const std::string &, Atlas::Message::MapType & map) const;
+    virtual void add(const std::string & key, Atlas::Message::MapType & map) const;
     /// \brief Add the value as an attribute to an Atlas entity
-    virtual void add(const std::string &, const Atlas::Objects::Entity::RootEntity &) const;
+    virtual void add(const std::string & key, const Atlas::Objects::Entity::RootEntity & ent) const;
 };
 
 /// \brief Entity property template for properties with single data values
@@ -51,6 +56,7 @@ class PropertyBase {
 ///
 /// Properties like CONTAINS, LOC and POS are accessed this way, as they
 /// are only ever modified as a result of a move operation.
+/// \ingroup PropertyClasses
 template <typename T>
 class ImmutableProperty : public PropertyBase {
   protected:
@@ -59,13 +65,14 @@ class ImmutableProperty : public PropertyBase {
   public:
     explicit ImmutableProperty(const T & data, unsigned int flags = 0);
 
-    virtual bool get(Atlas::Message::Element &) const;
-    virtual void set(const Atlas::Message::Element &);
-    virtual void add(const std::string &, Atlas::Message::MapType & map) const;
-    virtual void add(const std::string &, const Atlas::Objects::Entity::RootEntity &) const;
+    virtual bool get(Atlas::Message::Element & val) const;
+    virtual void set(const Atlas::Message::Element & val);
+    virtual void add(const std::string & key, Atlas::Message::MapType & map) const;
+    virtual void add(const std::string & key, const Atlas::Objects::Entity::RootEntity & ent) const;
 };
 
 /// \brief Entity property template for properties with single data values
+/// \ingroup PropertyClasses
 template <typename T>
 class Property : public ImmutableProperty<T> {
   protected:
@@ -78,13 +85,15 @@ class Property : public ImmutableProperty<T> {
 };
 
 /// \brief Entity property template for properties with single data values
+/// \ingroup PropertyClasses
 template <typename T>
 class SignalProperty : public Property<T>, virtual public sigc::trackable {
   public:
     explicit SignalProperty(T & data, unsigned int flags);
 
-    virtual void set(const Atlas::Message::Element &);
+    virtual void set(const Atlas::Message::Element & val);
 
+    /// \brief Signal that is emitted when this Property is modified.
     sigc::signal<void> modified;
 };
 
