@@ -15,24 +15,30 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: PropertyFactory.h,v 1.7 2007-12-07 01:19:16 alriddoch Exp $
+
 #ifndef COMMON_PROPERTY_FACTORY_H
 #define COMMON_PROPERTY_FACTORY_H
 
 #include <string>
 
+class Entity;
 class PropertyBase;
 
+/// \brief Factory interface for factories to create Property objects.
 class PropertyFactory {
   public:
     virtual ~PropertyFactory();
 
-    virtual PropertyBase * newProperty() = 0;
+    /// \brief Create a new Property instance
+    virtual PropertyBase * newProperty(Entity *) = 0;
 };
 
+/// \brief Factory template to create standard Property objects.
 template <class T>
 class PropertyBuilder : public PropertyFactory {
   public:
-    virtual PropertyBase * newProperty();
+    virtual PropertyBase * newProperty(Entity *);
 };
 
 #endif // COMMON_PROPERTY_FACTORY_H
