@@ -15,13 +15,15 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: PropertyFactory_impl.h,v 1.5 2006-10-26 00:48:03 alriddoch Exp $
+
 #ifndef COMMON_PROPERTY_FACTORY_IMPL_H
 #define COMMON_PROPERTY_FACTORY_IMPL_H
 
 #include "PropertyFactory.h"
 
 template <class T>
-PropertyBase * PropertyBuilder<T>::newProperty()
+PropertyBase * PropertyBuilder<T>::newProperty(Entity *)
 {
     return new T();
 }
