@@ -15,19 +15,16 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: custom.cpp,v 1.48 2007-07-04 22:00:15 alriddoch Exp $
+
 #include "inheritance.h"
 
 #include "Add.h"
 #include "Attack.h"
 #include "Burn.h"
-#include "Chop.h"
-#include "Cut.h"
-#include "Delve.h"
-#include "Dig.h"
 #include "Drop.h"
 #include "Eat.h"
 #include "Monitor.h"
-#include "Mow.h"
 #include "Nourish.h"
 #include "Pickup.h"
 #include "Setup.h"
@@ -37,22 +34,6 @@
 
 #include <Atlas/Objects/Entity.h>
 
-using Atlas::Objects::Root;
-using Atlas::Objects::Entity::AdminEntity;
-using Atlas::Objects::Operation::Add;
-using Atlas::Objects::Operation::Burn;
-using Atlas::Objects::Operation::Cut;
-using Atlas::Objects::Operation::Chop;
-using Atlas::Objects::Operation::Delve;
-using Atlas::Objects::Operation::Dig;
-using Atlas::Objects::Operation::Eat;
-using Atlas::Objects::Operation::Monitor;
-using Atlas::Objects::Operation::Mow;
-using Atlas::Objects::Operation::Nourish;
-using Atlas::Objects::Operation::Setup;
-using Atlas::Objects::Operation::Tick;
-using Atlas::Objects::Operation::Unseen;
-
 void installCustomOperations()
 {
     Inheritance & i = Inheritance::instance();
@@ -62,26 +43,10 @@ void installCustomOperations()
     Atlas::Objects::Operation::ADD_NO = atlas_factories->addFactory("add", &Atlas::Objects::generic_factory);
     i.opInstall("add", OP_ADD);
 
-    i.addChild(atlasOpDefinition("burn", "action"));
+    i.addChild(atlasOpDefinition("burn", "affect"));
     Atlas::Objects::Operation::BURN_NO = atlas_factories->addFactory("burn", &Atlas::Objects::generic_factory);
     i.opInstall("burn", OP_BURN);
 
-    i.addChild(atlasOpDefinition("chop", "affect"));
-    Atlas::Objects::Operation::CHOP_NO = atlas_factories->addFactory("chop", &Atlas::Objects::generic_factory);
-    i.opInstall("chop", OP_CHOP);
-
-    i.addChild(atlasOpDefinition("cut", "action"));
-    Atlas::Objects::Operation::CUT_NO = atlas_factories->addFactory("cut", &Atlas::Objects::generic_factory);
-    i.opInstall("cut", OP_CUT);
-
-    i.addChild(atlasOpDefinition("delve", "affect"));
-    Atlas::Objects::Operation::DELVE_NO = atlas_factories->addFactory("delve", &Atlas::Objects::generic_factory);
-    i.opInstall("delve", OP_DELVE);
-
-    i.addChild(atlasOpDefinition("dig", "affect"));
-    Atlas::Objects::Operation::DIG_NO = atlas_factories->addFactory("dig", &Atlas::Objects::generic_factory);
-    i.opInstall("dig", OP_DIG);
-
     i.addChild(atlasOpDefinition("eat", "action"));
     Atlas::Objects::Operation::EAT_NO = atlas_factories->addFactory("eat", &Atlas::Objects::generic_factory);
     i.opInstall("eat", OP_EAT);
@@ -90,10 +55,6 @@ void installCustomOperations()
     Atlas::Objects::Operation::MONITOR_NO = atlas_factories->addFactory("monitor", &Atlas::Objects::generic_factory);
     i.opInstall("monitor", OP_MONITOR);
 
-    i.addChild(atlasOpDefinition("mow", "affect"));
-    Atlas::Objects::Operation::MOW_NO = atlas_factories->addFactory("mow", &Atlas::Objects::generic_factory);
-    i.opInstall("mow", OP_MOW);
-
     i.addChild(atlasOpDefinition("nourish", "affect"));
     Atlas::Objects::Operation::NOURISH_NO = atlas_factories->addFactory("nourish", &Atlas::Objects::generic_factory);
     i.opInstall("nourish", OP_NOURISH);
