@@ -15,11 +15,19 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: debug.h,v 1.8 2006-10-26 00:48:05 alriddoch Exp $
+
 #ifndef COMMON_DEBUG_H
 #define COMMON_DEBUG_H
 
 #define debug(prg) { if (debug_flag) { prg } }
 
+#define debug_func() {\
+    if (debug_flag) { \
+        std::cerr << __PRETTY_FUNCTION__ << std::endl << std::flush;\
+    }\
+}
+
 template <typename T>
 void debug_dump(const T & t);
 
