@@ -15,16 +15,13 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: operations.cpp,v 1.28 2007-07-04 22:00:15 alriddoch Exp $
+
 #include "Add.h"
 #include "Attack.h"
-#include "Chop.h"
-#include "Cut.h"
 #include "Eat.h"
 #include "Burn.h"
-#include "Delve.h"
-#include "Dig.h"
 #include "Drop.h"
-#include "Mow.h"
 #include "Nourish.h"
 #include "Pickup.h"
 #include "Setup.h"
@@ -34,19 +31,21 @@
 #include "Monitor.h"
 #include "Connect.h"
 
+/// \brief Operation classes not defined in the Atlas spec
+///
+/// Operation classes used by Cyphesis as extensions of the
+/// original Atlas specification used to generate the classes in
+/// the Atlas-C++ library.
+/// \defgroup CustomOperations Custom Operation Classes
+
 namespace Atlas { namespace Objects { namespace Operation {
 
 int ADD_NO = -1;
 int ATTACK_NO = -1;
 int BURN_NO = -1;
-int CHOP_NO = -1;
 int CONNECT_NO = -1;
-int CUT_NO = -1;
-int DELVE_NO = -1;
-int DIG_NO = -1;
 int DROP_NO = -1;
 int MONITOR_NO = -1;
-int MOW_NO = -1;
 int EAT_NO = -1;
 int UNSEEN_NO = -1;
 int UPDATE_NO = -1;
