@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: system.cpp,v 1.31 2007-12-06 23:50:14 alriddoch Exp $
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -24,6 +26,7 @@
 #include "log.h"
 #include "debug.h"
 #include "globals.h"
+#include "compose.hpp"
 
 #include <wfmath/MersenneTwister.h>
 
@@ -34,9 +37,17 @@
 #include <iostream>
 
 extern "C" {
+#ifdef HAVE_SYS_UTSNAME_H
     #include <sys/utsname.h>
+#endif // HAVE_SYS_UTSNAME_H
     #include <sys/types.h>
+#ifdef HAVE_WINSOCK_H
+    #undef DATADIR
+    #include <winsock.h>
+#endif // HAVE_WINSOCK_H
+#ifdef HAVE_SYS_WAIT_H
     #include <sys/wait.h>
+#endif // HAVE_SYS_WAIT_H
     #include <signal.h>
     #include <fcntl.h>
     #include <unistd.h>
@@ -47,28 +58,40 @@ static const bool debug_flag = false;
 
 const std::string get_hostname()
 {
+#ifndef HAVE_UNAME
+    char hostname_buf[256];
+
+    if (gethostname(hostname_buf, 256) != 0) {
+        return "UNKNOWN";
+    }
+    return std::string(hostname_buf);
+#else // HAVE_UNAME
     struct utsname host_ident;
     if (uname(&host_ident) != 0) {
         return "UNKNOWN";
     }
     return std::string(host_ident.nodename);
+#endif // HAVE_UNAME
 }
 
 unsigned int security_check()
 {
+#ifdef HAVE_GETUID
     if (getuid() == 0 || geteuid() == 0) {
-        log(ERROR, "Running cyphesis as the superuser is dangerous.");
+        log(CYLOG_ERROR, "Running cyphesis as the superuser is dangerous.");
         return 0;
     }
+#endif // HAVE_GETUID
     return SECURITY_OKAY;
 }
 
 void reduce_priority(int p)
 {
-    // FIXME Make this optional
+#ifdef HAVE_NICE
     if (nice(p) < 0) {
         log(ERROR, "Unable to increase nice level to reduce priority");
     }
+#endif // HAVE_NICE
 }
 
 extern "C" void shutdown_on_signal(int signo)
@@ -106,6 +129,20 @@ extern "C" void report_abort(int signo)
 #endif
 }
 
+extern "C" void report_status(int signo)
+{
+    if (exit_flag) {
+        log(NOTICE, "Shutting down");
+    } else {
+        log(NOTICE, "Running");
+    }
+}
+
+extern "C" void rotate_logs(int signo)
+{
+    rotateLogger();
+}
+
 void interactive_signals()
 {
 #if defined(HAVE_SIGACTION)
@@ -145,15 +182,26 @@ void interactive_signals()
     action.sa_flags = SA_RESETHAND;
     action.sa_handler = report_abort;
     sigaction(SIGABRT, &action, NULL);
-#else
+
+#ifdef __APPLE__
+#warning Apple
+    sigemptyset(&action.sa_mask);
+    action.sa_flags = 0;
+    action.sa_handler = report_status;
+    sigaction(SIGINFO, &action, NULL);
+#endif // __APPLE__
+
+#else // defined(HAVE_SIGACTION)
     signal(SIGINT, shutdown_on_signal);
     signal(SIGTERM, shutdown_on_signal);
+#ifndef _WIN32
     signal(SIGQUIT, shutdown_on_signal);
     signal(SIGHUP, shutdown_on_signal);
     signal(SIGPIPE, SIG_IGN);
+#endif // _WIN32
     signal(SIGSEGV, report_segfault);
     signal(SIGABRT, report_abort);
-#endif
+#endif // defined(HAVE_SIGACTION)
 }
 
 void daemon_signals()
@@ -178,7 +226,7 @@ void daemon_signals()
 
     sigemptyset(&action.sa_mask);
     action.sa_flags = 0;
-    action.sa_handler = SIG_IGN;
+    action.sa_handler = rotate_logs;
     sigaction(SIGHUP, &action, NULL);
 
     sigemptyset(&action.sa_mask);
@@ -198,9 +246,11 @@ void daemon_signals()
 #else
     signal(SIGINT, SIG_IGN);
     signal(SIGTERM, shutdown_on_signal);
+#ifndef _WIN32
     signal(SIGQUIT, SIG_IGN);
-    signal(SIGHUP, SIG_IGN);
+    signal(SIGHUP, rotate_logs);
     signal(SIGPIPE, SIG_IGN);
+#endif // _WIN32
     signal(SIGSEGV, report_segfault);
     signal(SIGABRT, report_abort);
 #endif
@@ -208,6 +258,7 @@ void daemon_signals()
 
 int daemonise()
 {
+#ifdef HAVE_FORK
     int pid = fork();
     int new_stdio;
     int status = 0;
@@ -216,14 +267,18 @@ int daemonise()
     switch (pid) {
         case 0:
             // Child
+            // Get rid of controlling tty, and start new session
+            setsid();
             // Switch signal behavoir
             daemon_signals();
+            // Change current working directory to /
+            if (chdir("/") != 0) {
+                log(ERROR, "Unable to change current working directory to /");
+            }
             // Get rid if stdio
             close(STDIN_FILENO);
             close(STDOUT_FILENO);
             close(STDERR_FILENO);
-            // Get rid of controlling tty, and start new session
-            setsid();
             // Open /dev/null on the stdio file descriptors to avoid problems
             new_stdio = open("/dev/null", O_RDWR);
             dup2(new_stdio, STDIN_FILENO);
@@ -272,6 +327,13 @@ int daemonise()
                     log(ERROR, "Cyphesis was unable to connect to the database.");
                 } else if (estatus == EXIT_SOCKET_ERROR) {
                     log(ERROR, "Cyphesis was unable to open a listen socket.");
+                } else if (estatus == EXIT_PORT_ERROR) {
+                    log(ERROR, "Could not find free client listen socket. "
+                               "Init failed.");
+                    log(INFO, String::compose("To allocate 8 more ports please"
+                                              " run:\n\n    cyconfig "
+                                              "--cyphesis:dynamic_port_end=%1"
+                                              "\n\n", dynamic_port_end + 8));
                 } else {
                     log(ERROR, "Cyphesis exited unexpectedly at initialisation.");
                 }
@@ -281,13 +343,20 @@ int daemonise()
             break;
     }
     return pid;
+#else // HAVE_FORK
+    // On systems where we can't fork, fool the original process into thinking
+    // it is now the child.
+    return 0;
+#endif // HAVE_FORK
 }
 
 void running()
 {
+#ifdef HAVE_FORK
     if (daemon_flag) {
         kill(getppid(), SIGUSR1);
     }
+#endif // HAVE_FORK
 }
 
 static const char hex_table[16] = { '0', '1', '2', '3', '4', '5', '6', '7',
@@ -347,14 +416,14 @@ int check_password(const std::string & pwd, const std::string & hash)
     std::string new_hash;
     size_t hash_size = hash.size();
 
-    if ((hash_size < (digest_length * 2 + 5)) || (hash.substr(0, 3) != "$1$")) {
+    if (hash_size < (digest_length * 2 + 5) || hash.substr(0, 3) != "$1$") {
         // Get a hash of password with no salt
         hash_password(pwd, "", new_hash);
     } else {
         // Extract the salt from the hash string
         size_t dp = hash.find('$', 3);
         if (dp == std::string::npos) {
-            log(ERROR, "Password hash has no $ symbol after the salt.");
+            log(CYLOG_ERROR, "Password hash has no $ symbol after the salt.");
             return -1;
         }
         assert(dp > 3);
@@ -365,3 +434,30 @@ int check_password(const std::string & pwd, const std::string & hash)
     // Check if the generated hash matches the reference hash
     return hash == new_hash ? 0 : -1;
 }
+
+#ifndef HAVE_GETTIMEOFDAY
+
+int gettimeofday(struct timeval * tv, struct timezone * tz)
+{
+    assert(tz == 0);
+
+    SYSTEMTIME localtime;
+    struct tm unix_time;
+
+    GetLocalTime(&localtime);
+
+    unix_time.tm_sec = localtime.wSecond;
+    unix_time.tm_min = localtime.wMinute;
+    unix_time.tm_hour = localtime.wHour;
+    unix_time.tm_mday = localtime.wDay;
+    unix_time.tm_mon = localtime.wMonth - 1;
+    unix_time.tm_year = localtime.wYear - 1900;
+
+    tv->tv_usec = localtime.wMilliseconds * 1000;
+
+    tv->tv_sec = mktime(&unix_time);
+
+    return 0;
+}
+
+#endif
