@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Character.h,v 1.101 2007-12-07 01:19:16 alriddoch Exp $
+
 #ifndef RULESETS_CHARACTER_H
 #define RULESETS_CHARACTER_H
 
@@ -22,11 +24,13 @@
 
 #include "Statistics.h"
 
+#include "modules/EntityRef.h"
+
 #include <sigc++/connection.h>
 
+class BaseMind;
 class Movement;
 class Task;
-class BaseMind;
 
 typedef Thing Character_parent;
 
@@ -41,49 +45,67 @@ typedef Thing Character_parent;
 /// the mind or client to determine what the result on the world is, and
 /// operation verification for checking if an in-game operation should be
 /// passed to the mind.
+/// \ingroup EntityClasses
 class Character : public Character_parent {
   protected:
+    /// \brief Mathematical model that calculates characters stats and abilities
     Statistics m_statistics;
+    /// \brief Handler for simulating movement under direction from the mind
     Movement & m_movement;
+    /// \brief Handler for a medium term task being undertaken
     Task * m_task;
+    /// \brief Flag indicating if this entity is alive
     bool m_isAlive;
 
     // Properties
+    /// \brief Level of stamina character has left
     double m_stamina;
-    double m_drunkness;
-    std::string m_sex;
-    double m_food;
+    /// \brief Maximum mass of this type of creature
     double m_maxMass;
 
-    std::string m_rightHandWield;
+    /// \brief Identifier of entity in the characters right hand
+    EntityRef m_rightHandWield;
+    /// \brief Sigc connection used to ensure the entity is removed
+    /// when it changes containers.
     sigc::connection m_rightHandWieldConnection;
 
+    /// \brief Energy loss by metabolism per tick
     static const double energyConsumption;
+    /// \brief Food consumed by digestion per tick
     static const double foodConsumption;
+    /// \brief Proportion of weight converted to energy per tick when starving
     static const double weightConsumption;
-    static const double energyGain;
-    static const double energyLoss;
+    /// \brief Energy converted to weight by metabolism per tick
+    static const double energyLaidDown;
+    /// \brief Weight gained from excess energy by metabolism per tick
     static const double weightGain;
 
     void metabolise(OpVector &, double ammount = 1); 
     void wieldDropped();
+    LocatedEntity * findInContains(LocatedEntity * ent, const std::string & id);
+    LocatedEntity * findInInventory(const std::string & id);
 
     friend class Movement;
   public:
+    /// \brief Internal AI mind controlling this character
     BaseMind * m_mind;
+    /// \brief External network connected agent controlling this character
     BaseEntity * m_externalMind;
 
     explicit Character(const std::string & id, long intId);
     virtual ~Character();
 
+    /// \brief Accessor for mathematical stats model
     Statistics & statistics() { return m_statistics; }
+    /// \brief Accessor for medium term task
     Task * task() { return m_task; }
 
+    /// \brief Accessor for raw stamina property
     const double getStamina() const { return m_stamina; }
-    const double getDrunkness() const { return m_drunkness; }
-    const std::string & getSex() const { return m_sex; }
-    const double getFood() const { return m_food; }
-    const std::string & getRightHandWield() const { return m_rightHandWield; }
+    /// \brief Accessor for raw right hand wield property
+    const std::string & getRightHandWield() const {
+        return m_rightHandWield->getId();
+    }
 
     void setTask(Task *);
     void updateTask();
@@ -96,22 +118,15 @@ class Character : public Character_parent {
     virtual void SetupOperation(const Operation & op, OpVector &);
     virtual void TickOperation(const Operation & op, OpVector &);
     virtual void TalkOperation(const Operation & op, OpVector &);
-    virtual void EatOperation(const Operation & op, OpVector &);
     virtual void NourishOperation(const Operation & op, OpVector &);
-    virtual void UseOperation(const Operation &, OpVector &);
     virtual void WieldOperation(const Operation & op, OpVector &);
     virtual void AttackOperation(const Operation & op, OpVector &);
-    virtual void ChopOperation(const Operation & op, OpVector &);
 
-    virtual void mindActionOperation(const Operation &, OpVector &);
     virtual void mindAddOperation(const Operation &, OpVector &);
     virtual void mindAppearanceOperation(const Operation &, OpVector &);
     virtual void mindAttackOperation(const Operation &, OpVector &);
-    virtual void mindBurnOperation(const Operation &, OpVector &);
-    virtual void mindChopOperation(const Operation &, OpVector &);
     virtual void mindCombineOperation(const Operation &, OpVector &);
     virtual void mindCreateOperation(const Operation &, OpVector &);
-    virtual void mindCutOperation(const Operation &, OpVector &);
     virtual void mindDeleteOperation(const Operation &, OpVector &);
     virtual void mindDisappearanceOperation(const Operation &, OpVector &);
     virtual void mindDivideOperation(const Operation &, OpVector &);
@@ -138,37 +153,15 @@ class Character : public Character_parent {
     virtual void mindErrorOperation(const Operation &, OpVector &);
     virtual void mindOtherOperation(const Operation &, OpVector &);
 
-    bool w2mLoginOperation(const Operation &);
-    bool w2mLogoutOperation(const Operation &);
-    bool w2mActionOperation(const Operation &);
-    bool w2mAttackOperation(const Operation &);
-    bool w2mChopOperation(const Operation &);
-    bool w2mCombineOperation(const Operation &);
-    bool w2mCreateOperation(const Operation &);
-    bool w2mCutOperation(const Operation &);
-    bool w2mDeleteOperation(const Operation &);
-    bool w2mDivideOperation(const Operation &);
-    bool w2mEatOperation(const Operation &);
-    bool w2mBurnOperation(const Operation &);
-    bool w2mGetOperation(const Operation &);
-    bool w2mImaginaryOperation(const Operation &);
-    bool w2mInfoOperation(const Operation &);
-    bool w2mMoveOperation(const Operation &);
-    bool w2mNourishOperation(const Operation &);
-    bool w2mSetOperation(const Operation &);
     bool w2mSightOperation(const Operation &);
     bool w2mSoundOperation(const Operation &);
     bool w2mTouchOperation(const Operation &);
     bool w2mTickOperation(const Operation &);
-    bool w2mLookOperation(const Operation &);
+    bool w2mUnseenOperation(const Operation &);
     bool w2mSetupOperation(const Operation &);
-    bool w2mTalkOperation(const Operation &);
     bool w2mAppearanceOperation(const Operation &);
     bool w2mDisappearanceOperation(const Operation &);
-    bool w2mUseOperation(const Operation &);
-    bool w2mWieldOperation(const Operation &);
     bool w2mErrorOperation(const Operation &);
-    bool w2mOtherOperation(const Operation &);
 
     void sendMind(const Operation & op, OpVector &);
     void mind2body(const Operation & op, OpVector &);
