@@ -15,13 +15,14 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Creator.cpp,v 1.80 2007-11-26 15:06:33 alriddoch Exp $
+
 #include "Creator.h"
 
 #include "BaseMind.h"
 
 #include "common/log.h"
 #include "common/debug.h"
-#include "common/const.h"
 #include "common/serialno.h"
 #include "common/compose.hpp"
 
@@ -42,9 +43,6 @@ static const bool debug_flag = false;
 Creator::Creator(const std::string & id, long intId) : Creator_parent(id, intId)
 {
     debug( std::cout << "Creator::Creator" << std::endl << std::flush;);
-    if (consts::enable_omnipresence) {
-        m_attributes["omnipresent"] = 1;
-    }
 }
 
 void Creator::sendExternalMind(const Operation & op, OpVector & res)
@@ -83,7 +81,7 @@ void Creator::operation(const Operation & op, OpVector & res)
     // need to be implemented, in particular SetupOperation() would need
     // need to be implemented as below. Some might need to be blocked
     // to prevent anyone from messing with us, like SetOperation().
-    OpNo op_no = opEnumerate(op);
+    OpNo op_no = op->getClassNo();
     switch(op_no) {
         case OP_CREATE:
             CreateOperation(op, res);
@@ -102,7 +100,7 @@ void Creator::operation(const Operation & op, OpVector & res)
             break;
         default:
             if (op_no == OP_SETUP) {
-                m_world->addPerceptive(this);
+                BaseWorld::instance().addPerceptive(this);
             } else if (op_no == OP_TICK) {
                 TickOperation(op, res);
             }
@@ -119,11 +117,11 @@ void Creator::externalOperation(const Operation & op)
     debug( std::cout << "Creator::externalOperation("
                      << op->getParents().front() << ")" << std::endl
                      << std::flush;);
-    if (!op->hasAttrFlag(Atlas::Objects::Operation::TO_FLAG)) {
+    if (op->isDefaultTo()) {
         debug( std::cout << "Creator handling op normally" << std::endl
                          << std::flush;);
         Creator_parent::externalOperation(op);
-    } else if (op->getTo() == getId() && !op->hasAttrFlag(Atlas::Objects::Operation::FUTURE_SECONDS_FLAG)) {
+    } else if (op->getTo() == getId() && op->isDefaultFutureSeconds()) {
         debug( std::cout << "Creator handling op " << std::endl << std::flush;);
         OpVector lres;
         callOperation(op, lres);
@@ -137,12 +135,14 @@ void Creator::externalOperation(const Operation & op)
             // World will deal with it.
         }
     } else {
-        Entity * to = m_world->getEntity(op->getTo());
+        Entity * to = BaseWorld::instance().getEntity(op->getTo());
         if (to != 0) {
             // Make it appear like it came from target itself;
             to->sendWorld(op);
         } else {
-            log(ERROR, String::compose("Creator operation from client is to unknown ID %1", op->getTo()).c_str());
+            log(ERROR, String::compose("Creator operation from client "
+                                       "is to unknown ID \"%1\"",
+                                       op->getTo()));
 
             Unseen u;
 
@@ -170,14 +170,14 @@ void Creator::mindLookOperation(const Operation & op, OpVector & res)
     m_perceptive = true;
     const std::vector<Root> & args = op->getArgs();
     if (args.empty()) {
-        op->setTo(m_world->m_gameWorld.getId());
+        op->setTo(BaseWorld::instance().m_gameWorld.getId());
     } else {
         const Root & arg = args.front();
         if (arg->hasAttrFlag(Atlas::Objects::ID_FLAG)) {
             op->setTo(arg->getId());
         } else if (arg->hasAttrFlag(Atlas::Objects::NAME_FLAG)) {
             // Search by name
-            Entity * e = m_world->findByName(arg->getName());
+            Entity * e = BaseWorld::instance().findByName(arg->getName());
             if (e != NULL) {
                 op->setTo(e->getId());
             } else {
@@ -193,7 +193,7 @@ void Creator::mindLookOperation(const Operation & op, OpVector & res)
         } else if (arg->hasAttrFlag(Atlas::Objects::PARENTS_FLAG)) {
             // Search by name
             if (!arg->getParents().empty()) {
-                Entity * e = m_world->findByType(arg->getParents().front());
+                Entity * e = BaseWorld::instance().findByType(arg->getParents().front());
                 if (e != NULL) {
                     op->setTo(e->getId());
                 } else {
