@@ -15,11 +15,13 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: MemEntity.cpp,v 1.8 2007-12-02 23:49:06 alriddoch Exp $
+
 #include "MemEntity.h"
 
 static const bool debug_flag = false;
 
-MemEntity::MemEntity(const std::string & id, long intId) : Entity(id, intId),
+MemEntity::MemEntity(const std::string & id, long intId) : LocatedEntity(id, intId),
                                                            m_visible(false),
                                                            m_lastSeen(0.)
 {
