@@ -15,14 +15,18 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-#include "MemEntity.h"
+// $Id: MemMap.cpp,v 1.102 2007-12-02 23:49:06 alriddoch Exp $
+
 #include "MemMap.h"
+
+#include "MemEntity.h"
 #include "Script.h"
 
 #include "common/id.h"
 #include "common/log.h"
 #include "common/debug.h"
 #include "common/compose.hpp"
+#include "common/inheritance.h"
 
 #include <Atlas/Objects/Operation.h>
 #include <Atlas/Objects/Anonymous.h>
@@ -37,6 +41,8 @@ using Atlas::Objects::Operation::Look;
 using Atlas::Objects::Entity::RootEntity;
 using Atlas::Objects::Entity::Anonymous;
 
+const TypeNode * MemMap::m_entity_type = 0;
+
 MemEntity * MemMap::addEntity(MemEntity * entity)
 {
     assert(entity != 0);
@@ -66,9 +72,12 @@ void MemMap::readEntity(MemEntity * entity, const RootEntity & ent)
     if (ent->hasAttrFlag(Atlas::Objects::PARENTS_FLAG)) {
         const std::list<std::string> & parents = ent->getParents();
         if (!parents.empty()) {
-            if (entity->getType() == "entity") {
-                entity->setType(parents.front());
-            } else if (entity->getType() != parents.front()) {
+            if (entity->getType() == m_entity_type) {
+                const TypeNode * type = Inheritance::instance().getType(parents.front());
+                if (type != 0) {
+                    entity->setType(type);
+                }
+            } else if (entity->getType()->name() != parents.front()) {
                 debug(std::cout << "Attempting to mutate " << entity->getType()
                                 << " into " << parents.front()
                                 << std::endl << std::flush;);
@@ -77,7 +86,7 @@ void MemMap::readEntity(MemEntity * entity, const RootEntity & ent)
     }
     entity->merge(ent->asMessage());
     if (ent->hasAttrFlag(Atlas::Objects::Entity::LOC_FLAG)) {
-        Entity * old_loc = entity->m_location.m_loc;
+        LocatedEntity * old_loc = entity->m_location.m_loc;
         const std::string & new_loc_id = ent->getLoc();
         // Has LOC been changed?
         if (old_loc == 0 || new_loc_id != old_loc->getId()) {
@@ -109,13 +118,14 @@ void MemMap::updateEntity(MemEntity * entity, const RootEntity & ent)
     }
 }
 
-MemEntity * MemMap::newEntity(const std::string & id, long intId,
+MemEntity * MemMap::newEntity(const std::string & id, long int_id,
                               const RootEntity & ent)
 // Create a new entity from an Atlas message.
 {
-    assert(m_entities.find(intId) == m_entities.end());
+    assert(m_entities.find(int_id) == m_entities.end());
 
-    MemEntity * entity = new MemEntity(id, intId);
+    MemEntity * entity = new MemEntity(id, int_id);
+    entity->setType(m_entity_type);
 
     readEntity(entity, ent);
 
@@ -124,33 +134,37 @@ MemEntity * MemMap::newEntity(const std::string & id, long intId,
 
 MemMap::MemMap(Script *& s) : m_checkIterator(m_entities.begin()), m_script(s)
 {
-}
-
-Atlas::Objects::Operation::RootOperation MemMap::lookId()
-// Generate a look operation to look at an entity we are interested in
-{
-    debug( std::cout << "MemMap::lookId" << std::endl << std::flush;);
-    if (!m_additionsById.empty()) {
-        const std::string & id = m_additionsById.front();
-        Look l;
-        Anonymous m;
-        m->setId(id);
-        l->setArgs1(m);
-        m_additionsById.pop_front();
-        return l;
+    if (m_entity_type == 0) {
+        m_entity_type = Inheritance::instance().getType("game_entity");
+        assert(m_entity_type != 0);
     }
-    return NULL;
 }
 
-MemEntity * MemMap::addId(const std::string & id, long intId)
+void MemMap::sendLooks(OpVector & res)
+{
+    debug( std::cout << "MemMap::sendLooks" << std::endl << std::flush;);
+    std::list<std::string>::const_iterator I = m_additionsById.begin();
+    std::list<std::string>::const_iterator Iend = m_additionsById.end();
+    for (; I != Iend; ++I) {
+        Look l;
+        Anonymous look_arg;
+        look_arg->setId(*I);
+        l->setArgs1(look_arg);
+        res.push_back(l);
+    }
+    m_additionsById.clear();
+}
+
+MemEntity * MemMap::addId(const std::string & id, long int_id)
 // Queue the ID of an entity we are interested in
 {
     assert(!id.empty());
-    assert(m_entities.find(intId) == m_entities.end());
+    assert(m_entities.find(int_id) == m_entities.end());
 
     debug( std::cout << "MemMap::add_id" << std::endl << std::flush;);
     m_additionsById.push_back(id);
-    MemEntity * entity = new MemEntity(id, intId);
+    MemEntity * entity = new MemEntity(id, int_id);
+    entity->setType(m_entity_type);
     return addEntity(entity);
 }
 
@@ -159,13 +173,13 @@ void MemMap::del(const std::string & id)
 {
     debug( std::cout << "MemMap::del(" << id << ")" << std::endl << std::flush;);
 
-    long intId = integerId(id);
+    long int_id = integerId(id);
 
-    MemEntityDict::iterator I = m_entities.find(intId);
+    MemEntityDict::iterator I = m_entities.find(int_id);
     if (I != m_entities.end()) {
         MemEntity * ent = I->second;
         assert(ent != 0);
-        long next;
+        long next = -1;
         if (m_checkIterator != m_entities.end()) {
             next = m_checkIterator->first;
         }
@@ -173,7 +187,7 @@ void MemMap::del(const std::string & id)
 
         // Handling re-parenting is done very similarly to Entity::destroy,
         // but is slightly different as we tolerate LOC being null.
-        Entity * ent_loc = ent->m_location.m_loc;
+        LocatedEntity * ent_loc = ent->m_location.m_loc;
         if (ent_loc != 0) {
             // Remove deleted entity from its parents contains
             ent_loc->m_contains.erase(ent);
@@ -182,10 +196,10 @@ void MemMap::del(const std::string & id)
         ent->m_location.m_loc = 0;
 
         // Add deleted entities children into its parents contains
-        EntitySet::const_iterator K = ent->m_contains.begin();
-        EntitySet::const_iterator Kend = ent->m_contains.end();
+        LocatedEntitySet::const_iterator K = ent->m_contains.begin();
+        LocatedEntitySet::const_iterator Kend = ent->m_contains.end();
         for (; K != Kend; ++K) {
-            Entity * child_ent = *K;
+            LocatedEntity * child_ent = *K;
             child_ent->m_location.m_loc = ent_loc;
             // FIXME adjust pos and:
             // FIXME take account of orientation
@@ -194,7 +208,12 @@ void MemMap::del(const std::string & id)
             }
         }
 
-        m_checkIterator = m_entities.find(next);
+        if (next != -1) {
+            m_checkIterator = m_entities.find(next);
+        } else {
+            m_checkIterator = m_entities.begin();
+        }
+
         std::vector<std::string>::const_iterator J = m_deleteHooks.begin();
         std::vector<std::string>::const_iterator Jend = m_deleteHooks.end();
         for(; J != Jend; ++J) {
@@ -210,13 +229,13 @@ MemEntity * MemMap::get(const std::string & id) const
     debug( std::cout << "MemMap::get" << std::endl << std::flush;);
     if (id.empty()) {
         // This shouldn't really occur, and shouldn't be a problem
-        log(ERROR, "MemMap::get queried for empty id string");
+        log(ERROR, "MemMap::get queried for empty ID string.");
         return NULL;
     }
 
-    long intId = integerId(id);
+    long int_id = integerId(id);
 
-    MemEntityDict::const_iterator I = m_entities.find(intId);
+    MemEntityDict::const_iterator I = m_entities.find(int_id);
     if (I != m_entities.end()) {
         assert(I->second != 0);
         return I->second;
@@ -233,19 +252,19 @@ MemEntity * MemMap::getAdd(const std::string & id)
         return NULL;
     }
 
-    long intId = integerId(id);
+    long int_id = integerId(id);
 
-    if (intId == -1) {
-        log(ERROR, String::compose("MemMap::getAdd: Invalid ID \"%1\".", id).c_str());
+    if (int_id == -1) {
+        log(ERROR, String::compose("MemMap::getAdd: Invalid ID \"%1\".", id));
         return NULL;
     }
 
-    MemEntityDict::const_iterator I = m_entities.find(intId);
+    MemEntityDict::const_iterator I = m_entities.find(int_id);
     if (I != m_entities.end()) {
         assert(I->second != 0);
         return I->second;
     }
-    return addId(id, intId);
+    return addId(id, int_id);
 }
 
 void MemMap::addContents(const RootEntity & ent)
@@ -269,28 +288,28 @@ MemEntity * MemMap::updateAdd(const RootEntity & ent, const double & d)
 // entity is visible, as we may have received it because we can see the
 // creator.
 {
-    debug( std::cout << "MemMap::update" << std::endl << std::flush;);
+    debug( std::cout << "MemMap::updateAdd" << std::endl << std::flush;);
     if (!ent->hasAttrFlag(Atlas::Objects::ID_FLAG)) {
-        log(ERROR, "MemMap::update, Missing id in updated entity");
+        log(ERROR, "MemMap::updateAdd, Missing id in updated entity");
         return NULL;
     }
     const std::string & id = ent->getId();
     if (id.empty()) {
-        log(ERROR, "MemMap::update, Empty id in updated entity");
+        log(ERROR, "MemMap::updateAdd, Empty ID in updated entity.");
         return NULL;
     }
 
-    long intId = integerId(id);
+    long int_id = integerId(id);
 
-    if (intId == -1) {
-        log(ERROR, String::compose("MemMap::updateAdd: Invalid ID \"%1\".", id).c_str());
+    if (int_id == -1) {
+        log(ERROR, String::compose("MemMap::updateAdd: Invalid ID \"%1\".", id));
         return NULL;
     }
 
-    MemEntityDict::const_iterator I = m_entities.find(intId);
+    MemEntityDict::const_iterator I = m_entities.find(int_id);
     MemEntity * entity;
     if (I == m_entities.end()) {
-        entity = newEntity(id, intId, ent);
+        entity = newEntity(id, int_id, ent);
     } else {
         entity = I->second;
         updateEntity(entity, ent);
@@ -308,18 +327,20 @@ MemEntityVector MemMap::findByType(const std::string & what)
     for (MemEntityDict::const_iterator I = m_entities.begin(); I != Iend; ++I) {
         MemEntity * item = I->second;
         debug( std::cout << "F" << what << ":" << item->getType() << ":" << item->getId() << std::endl << std::flush;);
-        if (item->isVisible() && item->getType() == what) {
+        if (item->isVisible() && item->getType()->name() == what) {
             res.push_back(I->second);
         }
     }
     return res;
 }
 
-MemEntityVector MemMap::findByLocation(const Location & loc, double radius)
+MemEntityVector MemMap::findByLocation(const Location & loc, double radius,
+                                       const std::string & what)
 // Find an entity in our memory in a certain place
 // FIXME Don't return by value
 {
     MemEntityVector res;
+#if 0
     MemEntityDict::const_iterator Iend = m_entities.end();
     for (MemEntityDict::const_iterator I = m_entities.begin(); I != Iend; ++I) {
         MemEntity * item = I->second;
@@ -336,6 +357,32 @@ MemEntityVector MemMap::findByLocation(const Location & loc, double radius)
         }
     }
     return res;
+#else
+    LocatedEntity * place = loc.m_loc;
+    MemEntity * place_by_id = get(place->getId());
+    if (place != place_by_id) {
+        log(ERROR, "WTF!");
+        return res;
+    }
+    LocatedEntitySet::const_iterator I = place->m_contains.begin();
+    LocatedEntitySet::const_iterator Iend = place->m_contains.end();
+    float square_range = radius * radius;
+    for (; I != Iend; ++I) {
+        assert(*I != 0);
+        MemEntity * item = dynamic_cast<MemEntity *>(*I);
+        if (item == 0) {
+            log(ERROR, "Weird entity in memory");
+            continue;
+        }
+        if (!item->isVisible() || item->getType()->name() != what) {
+            continue;
+        }
+        if (squareDistance(loc.pos(), item->m_location.pos()) < square_range) {
+            res.push_back(item);
+        }
+    }
+    return res;
+#endif
 }
 
 const Element MemMap::asMessage()
@@ -351,17 +398,18 @@ const Element MemMap::asMessage()
 
 void MemMap::check(const double & time)
 {
-    if (m_checkIterator == m_entities.end()) {
+    MemEntityDict::const_iterator entities_end = m_entities.end();
+    if (m_checkIterator == entities_end) {
         m_checkIterator = m_entities.begin();
     } else {
         MemEntity * me = m_checkIterator->second;
         assert(me != 0);
         if (!me->isVisible() && ((time - me->lastSeen()) > 600) && (me->m_contains.empty())) {
-            debug(std::cout << me->getId() << "|" << me->getType()
+            debug(std::cout << me->getId() << "|" << me->getType()->name()
                       << " is a waste of space" << std::endl << std::flush;);
             MemEntityDict::const_iterator J = m_checkIterator;
-            long next;
-            if (++J != m_entities.end()) {
+            long next = -1;
+            if (++J != entities_end) {
                 next = J->first;
             }
             m_entities.erase(m_checkIterator);
@@ -373,11 +421,15 @@ void MemMap::check(const double & time)
             // FIXME This is required until MemMap uses parent refcounting
             me->m_location.m_loc = 0;
 
-            m_checkIterator = m_entities.find(next);
+            if (next != -1) {
+                m_checkIterator = m_entities.find(next);
+            } else {
+                m_checkIterator = m_entities.begin();
+            }
             // attribute of its its parent.
             me->decRef();
         } else {
-            debug(std::cout << me->getId() << "|" << me->getType() << "|"
+            debug(std::cout << me->getId() << "|" << me->getType()->name() << "|"
                             << me->lastSeen() << "|" << me->isVisible()
                             << " is fine" << std::endl << std::flush;);
             ++m_checkIterator;
