@@ -15,17 +15,23 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: MemMap.h,v 1.53 2007-12-07 01:19:16 alriddoch Exp $
+
 #ifndef RULESETS_MEM_MAP_H
 #define RULESETS_MEM_MAP_H
 
+#include "common/types.h"
+
+#include <Atlas/Message/Element.h>
+#include <Atlas/Objects/ObjectsFwd.h>
+
 #include <string>
 #include <list>
 
-#include <Atlas/Objects/ObjectsFwd.h>
-
+class Location;
 class MemEntity;
 class Script;
-class Location;
+class TypeNode;
 
 typedef std::vector<MemEntity *> MemEntityVector;
 typedef std::map<long, MemEntity *> MemEntityDict;
@@ -35,6 +41,8 @@ class MemMap {
   private:
     friend class BaseMind;
 
+    static const TypeNode * m_entity_type;
+
     MemEntityDict m_entities;
     MemEntityDict::iterator m_checkIterator;
     std::list<std::string> m_additionsById;
@@ -63,14 +71,15 @@ class MemMap {
         return m_entities;
     }
 
-    Atlas::Objects::Operation::RootOperation lookId();
+    void sendLooks(OpVector &);
     void del(const std::string & id);
     MemEntity * get(const std::string & id) const;
     MemEntity * getAdd(const std::string & id);
     MemEntity * updateAdd(const Atlas::Objects::Entity::RootEntity &, const double &);
 
     MemEntityVector findByType(const std::string & what);
-    MemEntityVector findByLocation(const Location & where, double radius);
+    MemEntityVector findByLocation(const Location & where, double radius,
+                                   const std::string & what);
 
     const Atlas::Message::Element asMessage();
 
