@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Plant.cpp,v 1.85 2007-12-03 23:18:52 alriddoch Exp $
+
 #include "Plant.h"
 
 #include "Script.h"
@@ -28,8 +30,6 @@
 
 #include "common/Tick.h"
 #include "common/Eat.h"
-#include "common/Chop.h"
-#include "common/Nourish.h"
 
 #include <wfmath/atlasconv.h>
 
@@ -96,66 +96,10 @@ int Plant::dropFruit(OpVector & res)
     return drop;
 }
 
-void Plant::ChopOperation(const Operation & op, OpVector & res)
-{
-    debug(std::cout << "Plant got chop op" << std::endl << std::flush;);
-    if (m_script->operation("chop", op, res)) {
-        return;
-    }
-    Element mode;
-    if (getAttr("mode", mode) && mode.isString() && mode.String() == "felled") {
-        debug(std::cout << "Plant is already down" << std::endl << std::flush;);
-        Set set_op;
-        Anonymous set_arg;
-        set_arg->setId(getId());
-        set_arg->setAttr("status", -1);
-        set_op->setArgs1(set_arg);
-        set_op->setTo(getId());
-        res.push_back(set_op);
-
-        if (m_location.bBox().isValid()) {
-            debug(std::cout << "Plant replaced by log" << std::endl << std::flush;);
-            Create create_op;
-            Anonymous create_arg;
-            create_arg->setParents(std::list<std::string>(1,"lumber"));
-            create_arg->setAttr("mass", getMass());
-            m_location.addToEntity(create_arg);
-            create_op->setArgs1(create_arg);
-            create_op->setTo(getId());
-            res.push_back(create_op);
-        }
-        return;
-    }
-    // FIXME In the future it will take more than one chop to chop down a tree.
-    Move move;
-    Anonymous move_arg;
-    if (m_location.m_loc != NULL) {
-        move_arg->setLoc(m_location.m_loc->getId());
-    } else {
-        log(ERROR, "Plant generating invalid Move op because LOC is NULL");
-    }
-    Vector3D axis(uniform(-1, 1), uniform(-1, 1), 0);
-    axis.normalize();
-    // FIXME Make tree fall away from axe, by using cross product of
-    // distance to axe, and vertical axis as axis of rotation
-    Quaternion orient(m_location.orientation());
-    orient.rotation(axis, M_PI/2);
-    move_arg->setAttr("orientation", orient.toAtlas());
-    move_arg->setAttr("mode", "felled");
-    ::addToEntity(m_location.pos(), move_arg->modifyPos());
-    move_arg->setId(getId());
-    move->setArgs1(move_arg);
-    move->setTo(getId());
-    res.push_back(move);
-}
-
 void Plant::NourishOperation(const Operation & op, OpVector & res)
 {
     debug(std::cout << "Plant::Nourish(" << getId() << "," << m_type << ")"
                     << std::endl << std::flush;);
-    if (m_script->operation("nourish", op, res)) {
-        return;
-    }
     if (op->getArgs().empty()) {
         error(op, "Nourish has no argument", res, getId());
         return;
@@ -170,13 +114,18 @@ void Plant::NourishOperation(const Operation & op, OpVector & res)
                     << std::endl << std::flush;);
 }
 
+void Plant::SetupOperation(const Operation & op, OpVector & res)
+{
+    Tick tick;
+    tick->setTo(getId());
+
+    res.push_back(tick);
+}
+
 void Plant::TickOperation(const Operation & op, OpVector & res)
 {
     debug(std::cout << "Plant::Tick(" << getId() << "," << m_type << ")"
                     << std::endl << std::flush;);
-    if (m_script->operation("tick", op, res)) {
-        return;
-    }
     Tick tick_op;
     tick_op->setTo(getId());
     tick_op->setFutureSeconds(consts::basic_tick * m_speed);
@@ -185,6 +134,7 @@ void Plant::TickOperation(const Operation & op, OpVector & res)
     // FIXME I don't like having to do this test, as its only required
     // during the unit tests.
     // Log an error perhaps?
+    // FIXME This causes a character holding an uprooted plant to die.
     if (m_location.m_loc != 0) {
         Eat eat_op;
         eat_op->setTo(m_location.m_loc->getId());
@@ -198,15 +148,27 @@ void Plant::TickOperation(const Operation & op, OpVector & res)
     Anonymous set_arg;
     set_arg->setId(getId());
 
-    double status = m_status;
+    Element new_status(1.);
+    PropertyBase * status = getProperty("status");
+    if (status != 0) {
+        status->get(new_status);
+    }
+    assert(new_status.isFloat());
     if (m_nourishment <= 0) {
-        status -= 0.1;
+        new_status = new_status.Float() - 0.1;
     } else {
-        status += 0.1;
-        if (status > 1.) {
-            status = 1.;
+        new_status = new_status.Float() + 0.1;
+        if (new_status.Float() > 1.) {
+            new_status = 1.;
         }
-        double new_mass = m_mass + m_nourishment;
+        set_arg->setAttr("status", new_status);
+
+        Element mass_attr;
+        double mass = 0;
+        if (getAttr("mass", mass_attr) && mass_attr.isFloat()) {
+            mass = mass_attr.Float();
+        }
+        double new_mass = mass + m_nourishment;
         m_nourishment = 0;
         Element maxmass_attr;
         if (getAttr("maxmass", maxmass_attr)) {
@@ -215,20 +177,26 @@ void Plant::TickOperation(const Operation & op, OpVector & res)
             }
         }
         set_arg->setAttr("mass", new_mass);
-        double scale = new_mass / m_mass;
-        double height_scale = pow(scale, 0.33333f);
-        debug(std::cout << "scale " << scale << ", " << height_scale
-                        << std::endl << std::flush;);
-        const BBox & ob = m_location.bBox();
-        BBox new_bbox(Point3D(ob.lowCorner().x() * height_scale,
-                              ob.lowCorner().y() * height_scale,
-                              ob.lowCorner().z() * height_scale),
-                      Point3D(ob.highCorner().x() * height_scale,
-                              ob.highCorner().y() * height_scale,
-                              ob.highCorner().z() * height_scale));
-        debug(std::cout << "Old " << ob
-                        << "New " << new_bbox << std::endl << std::flush;);
-        set_arg->setAttr("bbox", new_bbox.toAtlas());
+        if (hasAttr("biomass")) {
+            set_arg->setAttr("biomass", new_mass);
+        }
+
+        if (mass != 0) {
+            double scale = new_mass / mass;
+            double height_scale = pow(scale, 0.33333f);
+            debug(std::cout << "scale " << scale << ", " << height_scale
+                            << std::endl << std::flush;);
+            const BBox & ob = m_location.bBox();
+            BBox new_bbox(Point3D(ob.lowCorner().x() * height_scale,
+                                  ob.lowCorner().y() * height_scale,
+                                  ob.lowCorner().z() * height_scale),
+                          Point3D(ob.highCorner().x() * height_scale,
+                                  ob.highCorner().y() * height_scale,
+                                  ob.highCorner().z() * height_scale));
+            debug(std::cout << "Old " << ob
+                            << "New " << new_bbox << std::endl << std::flush;);
+            set_arg->setAttr("bbox", new_bbox.toAtlas());
+        }
     }
 
     int dropped = dropFruit(res);
@@ -239,7 +207,7 @@ void Plant::TickOperation(const Operation & op, OpVector & res)
             dropped--;
         }
     }
-    if ((dropped != 0) || (m_status < 1.)) {
+    if (dropped != 0 || new_status.Float() < 1.) {
         set_arg->setAttr("fruits", m_fruits);
     }
     set_op->setArgs1(set_arg);
@@ -251,9 +219,6 @@ void Plant::TouchOperation(const Operation & op, OpVector & res)
                     << std::endl << std::flush;);
     debug(std::cout << "Plant has " << m_fruits << " fruits right now"
                     << std::endl << std::flush;);
-    if (m_script->operation("touch", op, res)) {
-        return;
-    }
     debug(std::cout << "Checking for drop"
                     << std::endl << std::flush;);
 
