@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Plant.h,v 1.33 2007-06-19 22:17:43 alriddoch Exp $
+
 #ifndef RULESETS_PLANT_H
 #define RULESETS_PLANT_H
 
@@ -30,6 +32,7 @@ typedef Thing Plant_parent;
 /// for Acorn compatability. In the longer term this should provide for
 /// plants to grow, assuming we need plants to grow now that we are
 /// using Mercator for forests.
+/// \ingroup EntityClasses
 class Plant : public Plant_parent {
   protected:
     int m_fruits; // Number of fruits on the plant
@@ -55,8 +58,8 @@ class Plant : public Plant_parent {
     const int getFruitChance() const { return m_fruitChance; }
     const double getSizeAdult() const { return m_sizeAdult; }
 
-    virtual void ChopOperation(const Operation &, OpVector &);
     virtual void NourishOperation(const Operation &, OpVector &);
+    virtual void SetupOperation(const Operation &, OpVector &);
     virtual void TickOperation(const Operation &, OpVector &);
     virtual void TouchOperation(const Operation &, OpVector &);
 };
