@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Py_Location.cpp,v 1.46 2007-07-30 18:12:51 alriddoch Exp $
+
 #include "Py_Location.h"
 #include "Py_Thing.h"
 #include "Py_Vector3D.h"
@@ -44,10 +46,10 @@ static PyMethodDef Location_methods[] = {
 
 static void Location_dealloc(PyLocation *self)
 {
-    if ((self->owner == 0) && (self->location != NULL)) {
+    if (self->owner == 0 && self->location != NULL) {
         delete self->location;
     }
-    PyMem_DEL(self);
+    PyObject_Free(self);
 }
 
 static PyObject * Location_getattr(PyLocation *self, char *name)
@@ -111,12 +113,12 @@ static int Location_setattr(PyLocation *self, char *name, PyObject *v)
         self->location->m_loc = thing->m_entity;
         return 0;
     }
-    if ((strcmp(name, "bbox") == 0) && PyBBox_Check(v)) {
+    if (strcmp(name, "bbox") == 0 && PyBBox_Check(v)) {
         PyBBox * box = (PyBBox *)v;
         self->location->setBBox(box->box);
         return 0;
     }
-    if ((strcmp(name, "orientation") == 0) && PyQuaternion_Check(v)) {
+    if (strcmp(name, "orientation") == 0 && PyQuaternion_Check(v)) {
         PyQuaternion * quat = (PyQuaternion *)v;
         self->location->m_orientation = quat->rotation;
         return 0;
