@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Py_Mind.cpp,v 1.42 2007-12-02 23:49:07 alriddoch Exp $
+
 #include "Py_Mind.h"
 #include "Py_Thing.h"
 #include "Py_Map.h"
@@ -27,7 +29,7 @@
 #include "PythonWrapper.h"
 #include "BaseMind.h"
 
-#include "common/log.h"
+#include "common/inheritance.h"
 
 using Atlas::Message::Element;
 using Atlas::Message::MapType;
@@ -61,7 +63,7 @@ static PyMethodDef Mind_methods[] = {
 static void Mind_dealloc(PyMind *self)
 {
     Py_XDECREF(self->Mind_attr);
-    PyMem_DEL(self);
+    PyObject_Free(self);
 }
 
 static PyObject * Mind_getattr(PyMind *self, char *name)
@@ -77,12 +79,15 @@ static PyObject * Mind_getattr(PyMind *self, char *name)
         PyErr_SetString(PyExc_AttributeError, name);
         return NULL;
     }
+    if (strcmp(name, "id") == 0) {
+        return (PyObject *)PyString_FromString(self->m_mind->getId().c_str());
+    }
     if (strcmp(name, "type") == 0) {
         PyObject * list = PyList_New(0);
         if (list == NULL) {
             return NULL;
         }
-        PyObject * ent = PyString_FromString(self->m_mind->getType().c_str());
+        PyObject * ent = PyString_FromString(self->m_mind->getType()->name().c_str());
         PyList_Append(list, ent);
         Py_DECREF(ent);
         return list;
@@ -108,10 +113,10 @@ static PyObject * Mind_getattr(PyMind *self, char *name)
         if (list == NULL) {
             return NULL;
         }
-        EntitySet::const_iterator I = self->m_mind->m_contains.begin();
-        EntitySet::const_iterator Iend = self->m_mind->m_contains.end();
+        LocatedEntitySet::const_iterator I = self->m_mind->m_contains.begin();
+        LocatedEntitySet::const_iterator Iend = self->m_mind->m_contains.end();
         for (; I != Iend; ++I) {
-            Entity * child = *I;
+            LocatedEntity * child = *I;
             PyObject * wrapper = wrapEntity(child);
             if (wrapper == NULL) {
                 Py_DECREF(list);
@@ -129,7 +134,7 @@ static PyObject * Mind_getattr(PyMind *self, char *name)
             return v;
         }
     }
-    Entity * mind = self->m_mind;
+    LocatedEntity * mind = self->m_mind;
     Element attr;
     if (mind->getAttr(name, attr)) {
         return MessageElement_asPyObject(attr);
@@ -145,45 +150,29 @@ static int Mind_setattr(PyMind *self, char *name, PyObject *v)
         return -1;
     }
 #endif // NDEBUG
-    if (strcmp(name, "status") == 0) {
-        // This needs to be here until we can sort the difference
-        // between floats and ints in python.
-        if (PyInt_Check(v)) {
-            self->m_mind->setStatus((double)PyInt_AsLong(v));
-        } else if (PyFloat_Check(v)) {
-            self->m_mind->setStatus(PyFloat_AsDouble(v));
-        } else {
-            PyErr_SetString(PyExc_TypeError, "status must be numeric type");
-            return -1;
-        }
-        return 0;
-    }
     if (strcmp(name, "map") == 0) {
         return -1;
     }
-    Entity * thing = self->m_mind;
+    LocatedEntity * entity = self->m_mind;
+    // Should we support removal of attributes?
     //std::string attr(name);
     //if (v == NULL) {
-        //thing->attributes.erase(attr);
+        //entity->attributes.erase(attr);
         //return 0;
     //}
-    // FIXME It may now be possible to accept map and list attributes.
-    Element obj = PyObject_asMessageElement(v);
-    if (!obj.isNone() && !obj.isMap() && !obj.isList()) {
-        thing->setAttr(name, obj);
+    Element obj = PyObject_asMessageElement(v, true);
+    if (!obj.isNone()) {
+        assert(!obj.isMap() && !obj.isList());
+        // In the Python wrapper for Entity in Py_Thing.cpp notices are issued
+        // for some types.
+        entity->setAttr(name, obj);
         return 0;
     }
-#if 0
-    if (obj.isMap()) {
-        std::cout << name << " is m map" << std::endl << std::flush;
-    }
-    if (obj.isList()) {
-        std::cout << name << " is m list" << std::endl << std::flush;
-    }
-    if (obj.isNone()) {
-        std::cout << name << " is m none" << std::endl << std::flush;
-    }
-#endif
+    // Minds set a number of native Python members on themselves to store
+    // important state information, which seems to be lost if we munge them
+    // into Atlas data, probably because of the weird stuff that happens
+    // in the wrappers when scripts manipulate complex attributes. They
+    // are copied on getattr, rather than referenced.
     // If we get here, then the attribute is not Atlas compatable, so we
     // need to store it in a python dictionary
     if (self->Mind_attr == NULL) {
@@ -197,7 +186,7 @@ static int Mind_setattr(PyMind *self, char *name, PyObject *v)
 
 static int Mind_compare(PyMind *self, PyMind *other)
 {
-    if ((self->m_mind == NULL) || (other->m_mind == NULL)) {
+    if (self->m_mind == NULL || other->m_mind == NULL) {
         PyErr_SetString(PyExc_AssertionError, "NULL mind in Mind.compare");
         return -1;
     }
