@@ -15,19 +15,23 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Py_Point3D.h,v 1.9 2007-07-03 01:22:47 alriddoch Exp $
+
 #ifndef RULESETS_PY_POINT3D_H
 #define RULESETS_PY_POINT3D_H
 
 #include <Python.h>
 
-#include <physics/Vector3D.h>
+#include "physics/Vector3D.h"
 
 /// \brief Wrapper for Point3D in Python
 ///
 /// Uses in-place construction to embed full C++ object into what is
 /// basically a C structure.
+/// \ingroup PythonWrappers
 typedef struct {
     PyObject_HEAD
+    /// \brief Point3D object handled by this wrapper
     Point3D coords;
 } PyPoint3D;
 
