@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Py_Thing.cpp,v 1.64 2007-12-03 20:40:55 alriddoch Exp $
+
 #include "Py_Thing.h"
 #include "Py_Object.h"
 #include "Py_Vector3D.h"
@@ -30,11 +32,12 @@
 
 #include "common/log.h"
 #include "common/Property.h"
+#include "common/inheritance.h"
 
 using Atlas::Message::Element;
 using Atlas::Message::MapType;
 
-static PyObject * Entity_as_entity(PyEntity * self)
+static PyObject * Entity_as_entity(PyLocatedEntity * self)
 {
 #ifndef NDEBUG
     if (self->m_entity == NULL) {
@@ -51,6 +54,11 @@ static PyObject * Entity_as_entity(PyEntity * self)
     return (PyObject *)ret;
 }
 
+static PyMethodDef LocatedEntity_methods[] = {
+    {"as_entity",       (PyCFunction)Entity_as_entity,  METH_NOARGS},
+    {NULL,              NULL}           /* sentinel */
+};
+
 static PyObject * Entity_send_world(PyEntity * self, PyOperation * op)
 {
 #ifndef NDEBUG
@@ -165,7 +173,7 @@ static PyMethodDef Character_methods[] = {
 static void Entity_dealloc(PyEntity *self)
 {
     Py_XDECREF(self->Entity_attr);
-    PyMem_DEL(self);
+    PyObject_Free(self);
 }
 
 static PyObject * Entity_getattr(PyEntity *self, char *name)
@@ -181,12 +189,15 @@ static PyObject * Entity_getattr(PyEntity *self, char *name)
         PyErr_SetString(PyExc_AttributeError, name);
         return NULL;
     }
+    if (strcmp(name, "id") == 0) {
+        return (PyObject *)PyString_FromString(self->m_entity->getId().c_str());
+    }
     if (strcmp(name, "type") == 0) {
         PyObject * list = PyList_New(0);
         if (list == NULL) {
             return NULL;
         }
-        PyObject * ent = PyString_FromString(self->m_entity->getType().c_str());
+        PyObject * ent = PyString_FromString(self->m_entity->getType()->name().c_str());
         PyList_Append(list, ent);
         Py_DECREF(ent);
         return list;
@@ -199,7 +210,7 @@ static PyObject * Entity_getattr(PyEntity *self, char *name)
     }
     if (strcmp(name, "world") == 0) {
         PyWorld * world = newPyWorld();
-        world->world = self->m_entity->m_world;
+        world->world = &BaseWorld::instance();
         return (PyObject *)world;
     }
     if (strcmp(name, "contains") == 0) {
@@ -207,10 +218,10 @@ static PyObject * Entity_getattr(PyEntity *self, char *name)
         if (list == NULL) {
             return NULL;
         }
-        EntitySet::const_iterator I = self->m_entity->m_contains.begin();
-        EntitySet::const_iterator Iend = self->m_entity->m_contains.end();
+        LocatedEntitySet::const_iterator I = self->m_entity->m_contains.begin();
+        LocatedEntitySet::const_iterator Iend = self->m_entity->m_contains.end();
         for (; I != Iend; ++I) {
-            Entity * child = *I;
+            LocatedEntity * child = *I;
             PyObject * wrapper = wrapEntity(child);
             if (wrapper == NULL) {
                 Py_DECREF(list);
@@ -260,19 +271,6 @@ static int Entity_setattr(PyEntity *self, char *name, PyObject *v)
         return -1;
     }
 #endif // NDEBUG
-    if (strcmp(name, "status") == 0) {
-        // This needs to be here until we can sort the difference
-        // between floats and ints in python.
-        if (PyInt_Check(v)) {
-            self->m_entity->setStatus((double)PyInt_AsLong(v));
-        } else if (PyFloat_Check(v)) {
-            self->m_entity->setStatus(PyFloat_AsDouble(v));
-        } else {
-            PyErr_SetString(PyExc_TypeError, "status must be numeric type");
-            return -1;
-        }
-        return 0;
-    }
     if (strcmp(name, "map") == 0) {
         PyErr_SetString(PyExc_AttributeError, "map attribute forbidden");
         return -1;
@@ -309,7 +307,7 @@ static int Entity_setattr(PyEntity *self, char *name, PyObject *v)
 
 static int Entity_compare(PyEntity *self, PyEntity *other)
 {
-    if ((self->m_entity == NULL) || (other->m_entity == NULL)) {
+    if (self->m_entity == NULL || other->m_entity == NULL) {
         PyErr_SetString(PyExc_AssertionError, "NULL Entity in Entity.compare");
         return -1;
     }
@@ -335,30 +333,40 @@ PyTypeObject PyEntity_Type = {
         0,                              /*tp_hash*/
 };
 
-PyObject * wrapEntity(Entity * entity)
+PyObject * wrapEntity(LocatedEntity * le)
 {
     PyObject * wrapper;
-    PythonWrapper * pw = dynamic_cast<PythonWrapper *>(entity->script());
+    PythonWrapper * pw = dynamic_cast<PythonWrapper *>(le->script());
     if (pw == 0) {
-        Character * ch_entity = dynamic_cast<Character *>(entity);
-        if (ch_entity != 0) {
-            PyCharacter * pc = newPyCharacter();
-            if (pc == NULL) {
-                return NULL;
-            }
-            pc->m_entity = ch_entity;
-            wrapper = (PyObject *)pc;
+        Entity * entity = dynamic_cast<Entity *>(le);
+        if (entity != 0) {
+          Character * ch_entity = dynamic_cast<Character *>(entity);
+          if (ch_entity != 0) {
+              PyCharacter * pc = newPyCharacter();
+              if (pc == NULL) {
+                  return NULL;
+              }
+              pc->m_entity = ch_entity;
+              wrapper = (PyObject *)pc;
+          } else {
+              PyEntity * pe = newPyEntity();
+              if (pe == NULL) {
+                  return NULL;
+              }
+              pe->m_entity = entity;
+              wrapper = (PyObject *)pe;
+          }
         } else {
-            PyEntity * pe = newPyEntity();
-            if (pe == NULL) {
-                return NULL;
-            }
-            pe->m_entity = entity;
-            wrapper = (PyObject *)pe;
+          PyLocatedEntity * pe = newPyLocatedEntity();
+          if (pe == NULL) {
+              return NULL;
+          }
+          pe->m_entity = le;
+          wrapper = (PyObject *)pe;
         }
-        if (entity->script() == &noScript) {
+        if (le->script() == &noScript) {
             pw = new PythonWrapper(wrapper);
-            entity->setScript(pw);
+            le->setScript(pw);
         } else {
             log(WARNING, "Entity has script of unknown type");
         }
@@ -370,6 +378,18 @@ PyObject * wrapEntity(Entity * entity)
     return wrapper;
 }
 
+PyLocatedEntity * newPyLocatedEntity()
+{
+    PyLocatedEntity * self;
+    self = PyObject_NEW(PyLocatedEntity, &PyEntity_Type);
+    if (self == NULL) {
+        return NULL;
+    }
+    self->Entity_attr = NULL;
+    self->m_methods = LocatedEntity_methods;
+    return self;
+}
+
 PyEntity * newPyEntity()
 {
     PyEntity * self;
