@@ -15,21 +15,28 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: PythonThingScript.cpp,v 1.39 2007-12-02 23:49:07 alriddoch Exp $
+
 #include "PythonThingScript.h"
 
 #include "Py_Operation.h"
 #include "Py_Oplist.h"
+#include "Py_Thing.h"
 
-#include "Entity.h"
+#include "common/Tick.h"
 
 #include "common/log.h"
 #include "common/debug.h"
+#include "common/types.h"
 #include "common/compose.hpp"
 
 #include <Atlas/Objects/RootOperation.h>
 
+#include <iostream>
+
 static const bool debug_flag = false;
 
+/// \brief PythonEntityScript constructor
 PythonEntityScript::PythonEntityScript(PyObject * o, PyObject * wrapper) :
                     PythonScript(o, wrapper)
 {
@@ -41,8 +48,7 @@ PythonEntityScript::~PythonEntityScript()
 
 bool PythonEntityScript::operation(const std::string & op_type,
                                    const Operation & op,
-                                   OpVector & ret_list,
-                                   const Operation * sub_op)
+                                   OpVector & res)
 {
     assert(scriptObject != NULL);
     std::string op_name = op_type + "_operation";
@@ -69,6 +75,13 @@ bool PythonEntityScript::operation(const std::string & op_type,
         } else {
             log(ERROR, "Reporting python error");
             PyErr_Print();
+            if (op->getClassNo() == OP_TICK) {
+                log(ERROR,
+                    String::compose("Script for \"%1\" has reported an error "
+                                    "processing a tick operation. "
+                                    "This entity is probably now inactive.",
+                                    op->getTo()));
+            }
         }
         return false;
     }
@@ -79,23 +92,44 @@ bool PythonEntityScript::operation(const std::string & op_type,
     } else if (PyOperation_Check(ret)) {
         PyOperation * op = (PyOperation*)ret;
         assert(op->operation.isValid());
-        ret_list.push_back(op->operation);
+        res.push_back(op->operation);
     } else if (PyOplist_Check(ret)) {
         PyOplist * op = (PyOplist*)ret;
         assert(op->ops != NULL);
         const OpVector & o = *op->ops;
         OpVector::const_iterator Iend = o.end();
         for (OpVector::const_iterator I = o.begin(); I != Iend; ++I) {
-            ret_list.push_back(*I);
+            res.push_back(*I);
         }
     } else {
-       log(ERROR, String::compose("Python script \"%1\" returned an invalid result", op_name).c_str());
+        log(ERROR, String::compose("Python script \"%1\" returned an invalid "
+                                   "result.", op_name));
     }
     
     Py_DECREF(ret);
     return true;
 }
 
-void PythonEntityScript::hook(const std::string &, Entity *)
+void PythonEntityScript::hook(const std::string & function,
+                              LocatedEntity * entity)
 {
+    PyObject * wrapper = wrapEntity(entity);
+    if (wrapper == NULL) {
+        return;
+    }
+
+    PyObject * ret = PyObject_CallMethod(scriptObject,
+                                         (char *)(function.c_str()),
+                                         "(O)",
+                                         wrapper);
+    Py_DECREF(wrapper);
+    if (ret == NULL) {
+        if (PyErr_Occurred() == NULL) {
+            log(NOTICE, "No hook");
+        } else {
+            log(ERROR, "Reporting python error");
+            PyErr_Print();
+        }
+    }
+    Py_DECREF(ret);
 }
