@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Python_API.cpp,v 1.172 2007-12-04 00:07:34 alriddoch Exp $
+
 #include "Python.h"
 
 #include "Python_API.h"
@@ -35,14 +37,14 @@
 #include "Py_RootEntity.h"
 #include "Py_Oplist.h"
 #include "Py_Statistics.h"
+#include "Py_Property.h"
 
 #include "PythonThingScript.h"
-#include "PythonMindScript.h"
 #include "World.h"
-#include "Entity.h"
 #include "BaseMind.h"
 
 #include "common/inheritance.h"
+#include "common/compose.hpp"
 #include "common/globals.h"
 #include "common/const.h"
 #include "common/debug.h"
@@ -58,6 +60,11 @@ using Atlas::Objects::Entity::Anonymous;
 
 static const bool debug_flag = false;
 
+/// \defgroup PythonWrappers Python Wrapper Types
+///
+/// Structure types based on the PyObject header used to wrap C++ objects
+/// in Python.
+
 /// \brief Python wrapper for C++ functions to be exposed to Python
 typedef struct {
     PyObject_HEAD
@@ -65,11 +72,41 @@ typedef struct {
 
 static void Function_dealloc(FunctionObject * self)
 {
-    PyMem_DEL(self);
+    PyObject_Free(self);
 }
 
 static PyObject * log_debug(PyObject * self, PyObject * args, PyObject * kwds)
 {
+    if (consts::debug_level != 0) {
+        int level;
+        char *message;
+        PyObject * op;
+
+        if (!PyArg_ParseTuple(args, "is|O", &level, &message, &op)) {
+            return NULL;
+        }
+
+        if (consts::debug_level >= level) {
+            log(SCRIPT, message);
+        }
+    }
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject * log_think(PyObject * self, PyObject * args, PyObject * kwds)
+{
+    if (consts::debug_thinking != 0) {
+        char *message;
+
+        if (!PyArg_ParseTuple(args, "s", &message)) {
+            return NULL;
+        }
+
+        log(SCRIPT, message);
+    }
+
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -94,6 +131,26 @@ PyTypeObject log_debug_type = {
         log_debug,      /* tp_call */
 };
 
+PyTypeObject log_think_type = {
+        PyObject_HEAD_INIT(&PyType_Type)
+        0,
+        "Function",
+        sizeof(FunctionObject),
+        0,
+        /* methods */
+        (destructor)Function_dealloc,
+        0,              /* tp_print */
+        0,              /* tp_getattr */
+        0,              /* tp_setattr */
+        0,              /* tp_compare */
+        0,              /* tp_repr */
+        0,              /* tp_as_number */
+        0,              /* tp_as_sequence */
+        0,              /* tp_as_mapping */
+        0,              /* tp_hash */
+        log_think,      /* tp_call */
+};
+
 //////////////////////////////////////////////////////////////////////////
 // Logger replaces sys.stdout and sys.stderr so the nothing goes to output
 //////////////////////////////////////////////////////////////////////////
@@ -117,7 +174,7 @@ static void python_log(LogLevel lvl, const char * msg)
     for (p = message.find_first_of('\n');
          p != std::string::npos;
          p = message.find_first_of('\n', n)) {
-        log(lvl, message.substr(n, p - n).c_str());
+        log(lvl, message.substr(n, p - n));
         n = p + 1;
     }
     if (message.size() > n) {
@@ -167,7 +224,7 @@ static PyMethodDef PyErrLogger_methods[] = {
 
 static void PyLogger_dealloc(PyObject * self)
 {
-    PyMem_DEL(self);
+    PyObject_Free(self);
 }
 
 static PyObject * PyOutLogger_getattr(PyObject * self, char *name)
@@ -227,43 +284,40 @@ static PyObject * Get_PyClass(const std::string & package,
     PyObject * module = PyImport_Import(package_name);
     Py_DECREF(package_name);
     if (module == NULL) {
-        std::string msg = std::string("Missing python module ") + package;
-        log(ERROR, msg.c_str());
+        log(ERROR, String::compose("Missing python module \"%1\"", package));
         PyErr_Print();
         return NULL;
     }
-    PyObject * pyClass = PyObject_GetAttrString(module, (char *)classname.c_str());
+    PyObject * py_class = PyObject_GetAttrString(module, (char *)classname.c_str());
     Py_DECREF(module);
-    if (pyClass == NULL) {
-        std::string msg = std::string("Could not find python class ")
-                        + package + "." + classname;
-        log(ERROR, msg.c_str());
+    if (py_class == NULL) {
+        log(ERROR, String::compose("Could not find python class \"%1.%2\"",
+                                   package, classname));
         PyErr_Print();
         return NULL;
     }
-    if (PyCallable_Check(pyClass) == 0) {
-        std::string msg = std::string("Could not instance python class ")
-                        + package + "." + classname;
-        log(ERROR, msg.c_str());
-        Py_DECREF(pyClass);
+    if (PyCallable_Check(py_class) == 0) {
+        log(ERROR, String::compose("Could not instance python class \"%1.%2\"",
+                                   package, classname));
+        Py_DECREF(py_class);
         return NULL;
     }
 #if 0
     // In later versions of python using PyType_* will become the right thing
     // to do. This might become true when things have been done right with
     // installing types.
-    if (PyType_Check(pyClass) == 0) {
+    if (PyType_Check(py_class) == 0) {
         std::cerr << "PyCallable_Check returned true, but PyType_Check returned false " << package << "." << type << std::endl << std::flush;
     } else {
         std::cerr << "PyType_Check returned true" << std::endl << std::flush;
     }
 #endif
-    return pyClass;
+    return py_class;
 }
 
-PyObject * Create_PyScript(PyObject * wrapper, PyObject * pyClass)
+PyObject * Create_PyScript(PyObject * wrapper, PyObject * py_class)
 {
-    PyObject * pyob = PyEval_CallFunction(pyClass,"(O)", wrapper);
+    PyObject * pyob = PyEval_CallFunction(py_class,"(O)", wrapper);
     
     if (pyob == NULL) {
         if (PyErr_Occurred() == NULL) {
@@ -277,62 +331,18 @@ PyObject * Create_PyScript(PyObject * wrapper, PyObject * pyClass)
     return pyob;
 }
 
-void Subscribe_Script(Entity * entity, PyObject * pyclass,
-                      const std::string& package)
-{
-#if 0
-    PyObject * dmap = PyObject_GetAttrString(pyclass, "__dict__");
-    if (dmap == NULL) {
-        std::string msg = std::string( "Python class for ") + package
-                        + " has no __dict__";
-        log(ERROR, msg.c_str());
-        return;
-    }
-    if (!PyDict_Check(dmap)) {
-        std::string msg = std::string( "Python class for ") + package
-                        + " is malformed";
-        log(ERROR, msg.c_str());
-        return;
-    }
-    PyObject * keys = PyDict_Keys(dmap);
-#else
-    PyObject * keys = PyObject_Dir(pyclass);
-#endif
-    if (keys == NULL) {
-        std::string msg = std::string("Error getting attribute list of Python class for ") + package;
-        log(ERROR, msg.c_str());
-        return;
-    }
-    for(int i = 0; i < PyList_Size(keys); i++) {
-        std::string method(PyString_AsString(PyList_GetItem(keys, i)));
-        std::string::size_type l = method.find("_operation", 0, 10);
-        if (l == std::string::npos) {
-            debug(std::cout << method << " is not a method" << std::endl
-                            << std::flush;);
-        } else {
-            std::string op_name = method.substr(0,l);
-            debug(std::cout << method << " is a method, it contains _op.. at "
-                            << l << " so we can register for "
-                            << method.substr(0,l) << std::endl << std::flush;);
-            entity->scriptSubscribe(op_name);
-        }
-    }
-    Py_DECREF(keys);
-}
-
 void Create_PyMind(BaseMind * mind, const std::string & package,
                                     const std::string & type)
 {
-    PyObject * pyClass = Get_PyClass(package, type);
-    if (pyClass == NULL) { return; }
+    PyObject * py_class = Get_PyClass(package, type);
+    if (py_class == NULL) { return; }
     PyMind * wrapper = newPyMind();
     wrapper->m_mind = mind;
-    Subscribe_Script(mind, pyClass, package);
-    PyObject * o = Create_PyScript((PyObject *)wrapper, pyClass);
-    Py_DECREF(pyClass);
+    PyObject * o = Create_PyScript((PyObject *)wrapper, py_class);
+    Py_DECREF(py_class);
 
     if (o != NULL) {
-        mind->setScript(new PythonMindScript(o, (PyObject *)wrapper, *mind));
+        mind->setScript(new PythonEntityScript(o, (PyObject *)wrapper));
     }
 }
 
@@ -351,13 +361,13 @@ static PyObject * location_new(PyObject * self, PyObject * args)
     PyLocation *o;
     // We need to deal with actual args here
     PyObject * refO = NULL, * coordsO = NULL;
-    Entity * ref_ent = NULL;
+    LocatedEntity * ref_ent = NULL;
     bool decrefO = false;
     if (!PyArg_ParseTuple(args, "|OO", &refO, &coordsO)) {
         return NULL;
     }
     if (refO != NULL) {
-        if ((!PyEntity_Check(refO)) && (!PyWorld_Check(refO)) && (!PyMind_Check(refO))) {
+        if (!PyEntity_Check(refO) && !PyWorld_Check(refO) && !PyMind_Check(refO)) {
             if (PyObject_HasAttrString(refO, "cppthing")) {
                 refO = PyObject_GetAttrString(refO, "cppthing");
                 decrefO = true;
@@ -368,7 +378,7 @@ static PyObject * location_new(PyObject * self, PyObject * args)
                 return NULL;
             }
         }
-        if ((coordsO != NULL) && (!PyPoint3D_Check(coordsO))) {
+        if (coordsO != NULL && !PyPoint3D_Check(coordsO)) {
             PyErr_SetString(PyExc_TypeError, "Arg coords required");
             if (decrefO) { Py_DECREF(refO); }
             return NULL;
@@ -572,7 +582,7 @@ static PyObject * quaternion_new(PyObject * self, PyObject * args)
                 break;
             case 1:
                 clist = PyTuple_GetItem(args, 0);
-                if ((!PyList_Check(clist)) || (PyList_Size(clist) != 4)) {
+                if (!PyList_Check(clist) || PyList_Size(clist) != 4) {
                     PyErr_SetString(PyExc_TypeError, "Quaternion() from single value must a list 4 long");
                     return NULL;
                 }
@@ -679,6 +689,42 @@ static PyObject * oplist_new(PyObject * self, PyObject * args)
         return (PyObject *)o;
 }
 
+static int PySequence_asVector(PyObject * o, std::vector<double> & ret)
+{
+    int len;
+    PyObject * item;
+    if (PyList_Check(o)) {
+        len = PyList_Size(o);
+        ret.resize(len);
+        for(int i = 0; i < len; i++) {
+            item = PyList_GetItem(o, i);
+            if (PyFloat_Check(item)) {
+                ret[i] = PyFloat_AsDouble(item);
+            } else if (PyInt_Check(item)) {
+                ret[i] = PyInt_AsLong(item);
+            } else {
+                return -1;
+            }
+        }
+    } else if (PyTuple_Check(o)) {
+        len = PyTuple_Size(o);
+        ret.resize(len);
+        for(int i = 0; i < len; i++) {
+            item = PyTuple_GetItem(o, i);
+            if (PyFloat_Check(item)) {
+                ret[i] = PyFloat_AsDouble(item);
+            } else if (PyInt_Check(item)) {
+                ret[i] = PyInt_AsLong(item);
+            } else {
+                return -1;
+            }
+        }
+    } else {
+        return -1;
+    }
+    return 0;
+}
+
 static PyObject * entity_new(PyObject * self, PyObject * args, PyObject * kwds)
 {
     char * id = NULL;
@@ -690,10 +736,10 @@ static PyObject * entity_new(PyObject * self, PyObject * args, PyObject * kwds)
     if (id != NULL) {
         ent->setId(id);
     }
-    if ((kwds != NULL) && (PyDict_Check(kwds))) {
+    if (kwds != NULL && PyDict_Check(kwds)) {
         PyObject * keys = PyDict_Keys(kwds);
         PyObject * vals = PyDict_Values(kwds);
-        if ((keys == NULL) || (vals == NULL)) {
+        if (keys == NULL || vals == NULL) {
             PyErr_SetString(PyExc_RuntimeError, "Error in keywords");
             return NULL;
         }
@@ -701,14 +747,31 @@ static PyObject * entity_new(PyObject * self, PyObject * args, PyObject * kwds)
         for(i = 0; i < size; i++) {
             char * key = PyString_AsString(PyList_GetItem(keys, i));
             PyObject * val = PyList_GetItem(vals, i);
-            if ((strcmp(key, "location") == 0) && (PyLocation_Check(val))) {
+            if (strcmp(key, "location") == 0) {
+                if (!PyLocation_Check(val)) {
+                    PyErr_SetString(PyExc_TypeError, "location must be a Location object");
+                    return NULL;
+                }
                 PyLocation * loc = (PyLocation*)val;
                 loc->location->addToEntity(ent);
-            } else if (strcmp(key, "xyz") == 0) {
-                ent->setAttr("pos", PyObject_asMessageElement(val));
-            } else if ((strcmp(key, "parent") == 0) && (PyString_Check(val))) {
+            } else if (strcmp(key, "pos") == 0) {
+                std::vector<double> vector_val;
+                if (PySequence_asVector(val, vector_val) != 0) {
+                    PyErr_SetString(PyExc_TypeError, "pos must be a number sequence.");
+                    return NULL;
+                }
+                ent->setPos(vector_val);
+            } else if (strcmp(key, "parent") == 0) {
+                if (!PyString_Check(val)) {
+                    PyErr_SetString(PyExc_TypeError, "parent must be a string.");
+                    return NULL;
+                }
                 ent->setLoc(PyString_AsString(val));
-            } else if ((strcmp(key, "type") == 0) && (PyString_Check(val))) {
+            } else if (strcmp(key, "type") == 0) {
+                if (!PyString_Check(val)) {
+                    PyErr_SetString(PyExc_TypeError, "type must be a string.");
+                    return NULL;
+                }
                 ent->setParents(std::list<std::string>(1, PyString_AsString(val)));
                 ent->setObjtype("obj");
             } else {
@@ -734,40 +797,46 @@ static PyObject * entity_new(PyObject * self, PyObject * args, PyObject * kwds)
     return (PyObject *)o;
 }
 
-static inline void addToArgs(std::vector<Root> & args, PyObject * arg)
+static int addToArgs(std::vector<Root> & args, PyObject * arg)
 {
-    if (arg == NULL) {
-        return;
-    }
     if (PyMessageElement_Check(arg)) {
         PyMessageElement * obj = (PyMessageElement*)arg;
+#ifndef NDEBUG
         if (obj->m_obj == NULL) {
-            log(ERROR, "Operation() Null element object added to new operation arguments.");
-            return;
+            PyErr_SetString(PyExc_AssertionError,"NULL MessageElement in Operation constructor argument");
+            return -1;
         }
+#endif // NDEBUG
         const Element & o = *obj->m_obj;
         if (o.isMap()) {
             args.push_back(Atlas::Objects::Factories::instance()->createObject(o.asMap()));
         } else {
-            log(ERROR, "Operation() Non-map element object added to new operation arguments."); // FIXME perhaps this should raise a python exception?
+            PyErr_SetString(PyExc_TypeError, "Operation arg is not a map");
+            return -1;
         }
     } else if (PyOperation_Check(arg)) {
         PyOperation * op = (PyOperation*)arg;
+#ifndef NDEBUG
         if (!op->operation.isValid()) {
-            log(ERROR, "Operation() Null operation object added to new operation arguments.");
-            return;
+            PyErr_SetString(PyExc_AssertionError,"Invalid operation in Operation constructor argument");
+            return -1;
         }
+#endif // NDEBUG
         args.push_back(op->operation);
     } else if (PyRootEntity_Check(arg)) {
         PyRootEntity * ent = (PyRootEntity*)arg;
+#ifndef NDEBUG
         if (!ent->entity.isValid()) {
-            log(ERROR, "Operation() Null operation object added to new operation arguments.");
-            return;
+            PyErr_SetString(PyExc_AssertionError,"Invalid rootentity in Operation constructor argument");
+            return -1;
         }
+#endif // NDEBUG
         args.push_back(ent->entity);
     } else {
-        log(ERROR, "Operation() Unknown object added to operation arguments.");
+        PyErr_SetString(PyExc_TypeError, "Operation arg is of unknown type");
+        return -1;
     }
+    return 0;
 }
 
 static PyObject * operation_new(PyObject * self, PyObject * args, PyObject * kwds)
@@ -838,9 +907,19 @@ static PyObject * operation_new(PyObject * self, PyObject * args, PyObject * kwd
         }
     }
     std::vector<Root> & args_list = op->operation->modifyArgs();
-    addToArgs(args_list, arg1);
-    addToArgs(args_list, arg2);
-    addToArgs(args_list, arg3);
+    assert(args_list.empty());
+    if (arg1 != 0 && addToArgs(args_list, arg1) != 0) {
+        Py_DECREF(op);
+        op = NULL;
+    }
+    if (arg2 != 0 && addToArgs(args_list, arg2) != 0) {
+        Py_DECREF(op);
+        op = NULL;
+    }
+    if (arg3 != 0 && addToArgs(args_list, arg3) != 0) {
+        Py_DECREF(op);
+        op = NULL;
+    }
     return (PyObject *)op;
 }
 
@@ -884,17 +963,6 @@ static PyMethodDef common_methods[] = {
 
 void init_python_api()
 {
-    std::string importCmd("ruleset_import_hooks.install([");
-    std::vector<std::string>::const_iterator Ibeg = rulesets.begin();
-    std::vector<std::string>::const_iterator Iend = rulesets.end();
-    for (std::vector<std::string>::const_iterator I = Ibeg; I != Iend; ++I) {
-        if (I != Ibeg) {
-            importCmd = importCmd + ",";
-        }
-        importCmd = importCmd + "\"" + *I + "\"";
-    }
-    importCmd = importCmd + "])\n";
-
     Py_Initialize();
 
     PyObject * sys_name = PyString_FromString("sys");
@@ -906,24 +974,33 @@ void init_python_api()
         return;
     }
 
-    PyObject * outLogger = (PyObject*)PyObject_NEW(PyLogger, &PyOutLogger_Type);
-    PyObject_SetAttrString(sys_module, "stdout", outLogger);
-    Py_DECREF(outLogger);
+    PyObject * out_logger = (PyObject*)PyObject_NEW(PyLogger, &PyOutLogger_Type);
+    PyObject_SetAttrString(sys_module, "stdout", out_logger);
+    Py_DECREF(out_logger);
 
-    PyObject * errLogger = (PyObject*)PyObject_NEW(PyLogger, &PyErrLogger_Type);
-    PyObject_SetAttrString(sys_module, "stderr", errLogger);
-    Py_DECREF(errLogger);
+    PyObject * err_logger = (PyObject*)PyObject_NEW(PyLogger, &PyErrLogger_Type);
+    PyObject_SetAttrString(sys_module, "stderr", err_logger);
+    Py_DECREF(err_logger);
 
     PyObject * sys_path = PyObject_GetAttrString(sys_module, "path");
     if (sys_path != 0) {
         if (PyList_Check(sys_path)) {
-            std::vector<std::string>::const_iterator I = Ibeg;
-            for (; I != Iend; ++I) {
-                std::string p = share_directory + "/cyphesis/rulesets/" + *I;
-                PyObject * path = PyString_FromString(p.c_str());
-                PyList_Append(sys_path, path);
-                Py_DECREF(path);
-            }
+            // Add the path to the non-ruleset specific code.
+            std::string p = share_directory + "/cyphesis/scripts";
+            PyObject * path = PyString_FromString(p.c_str());
+            PyList_Append(sys_path, path);
+            Py_DECREF(path);
+
+            p = share_directory + "/cyphesis/rulesets/basic";
+            path = PyString_FromString(p.c_str());
+            PyList_Append(sys_path, path);
+            Py_DECREF(path);
+
+            // Add the path to the ruleset specific code.
+            p = share_directory + "/cyphesis/rulesets/" + ruleset;
+            path = PyString_FromString(p.c_str());
+            PyList_Append(sys_path, path);
+            Py_DECREF(path);
         } else {
             log(CRITICAL, "Python sys.path is not a list");
         }
@@ -967,6 +1044,11 @@ void init_python_api()
     PyObject * debug = (PyObject*)PyObject_NEW(FunctionObject, &log_debug_type);
     PyObject_SetAttrString(log_mod, "debug", debug);
     Py_DECREF(debug);
+
+    PyObject * think = (PyObject*)PyObject_NEW(FunctionObject, &log_think_type);
+    PyObject_SetAttrString(log_mod, "thinking", think);
+    Py_DECREF(think);
+
     Py_DECREF(log_mod);
 
     PyObject * o;
@@ -978,9 +1060,11 @@ void init_python_api()
     o = PyInt_FromLong(0);
     PyObject_SetAttrString(_const, "server_python", o);
     Py_DECREF(o);
+
     o = PyInt_FromLong(consts::debug_level);
     PyObject_SetAttrString(_const, "debug_level", o);
     Py_DECREF(o);
+
     o = PyInt_FromLong(consts::debug_thinking);
     PyObject_SetAttrString(_const, "debug_thinking", o);
     Py_DECREF(o);
@@ -988,9 +1072,11 @@ void init_python_api()
     o = PyFloat_FromDouble(consts::time_multiplier);
     PyObject_SetAttrString(_const, "time_multiplier", o);
     Py_DECREF(o);
+
     o = PyFloat_FromDouble(consts::base_velocity_coefficient);
     PyObject_SetAttrString(_const, "base_velocity_coefficient", o);
     Py_DECREF(o);
+
     o = PyFloat_FromDouble(consts::base_velocity);
     PyObject_SetAttrString(_const, "base_velocity", o);
     Py_DECREF(o);
@@ -999,15 +1085,10 @@ void init_python_api()
     PyObject_SetAttrString(_const, "basic_tick", o);
     Py_DECREF(o);
 
-    o = PyFloat_FromDouble(consts::sight_range);
-    PyObject_SetAttrString(_const, "sight_range", o);
-    Py_DECREF(o);
-    o = PyFloat_FromDouble(consts::hearing_range);
-    PyObject_SetAttrString(_const, "hearing_range", o);
-    Py_DECREF(o);
-    o = PyInt_FromLong(consts::enable_ranges);
-    PyObject_SetAttrString(_const, "enable_ranges", o);
+    o = PyFloat_FromDouble(WFMATH_EPSILON);
+    PyObject_SetAttrString(_const, "epsilon", o);
     Py_DECREF(o);
+
     Py_DECREF(_const);
 
     /// Create the common.globals module
@@ -1067,8 +1148,11 @@ void init_python_api()
     }
     PyModule_AddObject(vector3d, "Vector3D", (PyObject *)&PyVector3D_Type);
 
-    PyRun_SimpleString("from hooks import ruleset_import_hooks\n");
-    PyRun_SimpleString((char *)importCmd.c_str());
+    if (PyType_Ready(&PyTerrainProperty_Type) < 0) {
+        log(CRITICAL, "Python init failed to ready TerrainProperty wrapper type");
+        return;
+    }
+
 
     debug(std::cout << Py_GetPath() << std::endl << std::flush;);
 }
