@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Stackable.cpp,v 1.53 2007-11-28 10:57:00 alriddoch Exp $
+
 // A stackable object, ie one which can represent multiple object of the
 // same type. Used for things like coins.
 
@@ -23,6 +25,8 @@
 #include "Script.h"
 
 #include "common/Property.h"
+#include "common/log.h"
+#include "common/inheritance.h"
 
 #include <Atlas/Objects/Operation.h>
 #include <Atlas/Objects/Anonymous.h>
@@ -31,8 +35,14 @@ using Atlas::Message::Element;
 using Atlas::Objects::Root;
 using Atlas::Objects::Operation::Create;
 using Atlas::Objects::Operation::Delete;
+using Atlas::Objects::Operation::Set;
+using Atlas::Objects::Operation::Sight;
 using Atlas::Objects::Entity::Anonymous;
 
+/// \brief Stackable constructor
+///
+/// @param id identifier of this Entity
+/// @param intId identifier of this Entity
 Stackable::Stackable(const std::string & id, long intId) :
                                              Stackable_parent(id, intId),
                                              m_num(1)
@@ -46,9 +56,8 @@ Stackable::~Stackable()
 
 void Stackable::CombineOperation(const Operation & op, OpVector & res)
 {
-    if (m_script->operation("combine", op, res) != 0) {
-        return;
-    }
+    std::cout << "CombineOperation" << std::endl << std::flush;
+    int old_num = m_num;
     const std::vector<Root> & args = op->getArgs();
     std::vector<Root>::const_iterator Iend = args.end();
     for (std::vector<Root>::const_iterator I = args.begin(); I != Iend; ++I) {
@@ -62,7 +71,7 @@ void Stackable::CombineOperation(const Operation & op, OpVector & res)
             // This is normal
             continue;
         }
-        Entity * ent = m_world->getEntity(id);
+        Entity * ent = BaseWorld::instance().getEntity(id);
         if (ent == NULL) {
             // FIXME Send an Unseen op?
             continue;
@@ -71,7 +80,9 @@ void Stackable::CombineOperation(const Operation & op, OpVector & res)
         if (obj == NULL) { continue; }
         if (obj->m_type != m_type) { continue; }
         m_num = m_num + obj->m_num;
-        // Set op for num change?
+        // This ensures an attempt to stack this entitie multiple times will
+        // not result in a dupped item
+        obj->m_num = 0;
 
         Delete d;
         Anonymous del_arg;
@@ -80,29 +91,54 @@ void Stackable::CombineOperation(const Operation & op, OpVector & res)
         d->setArgs1(del_arg);
         res.push_back(d);
     }
-    // Currently does not send sight ops, as the Sight ops for this type of
-    // thing have not been discussed
+
+    if (old_num == m_num) {
+        return;
+    }
+
+    Set set;
+    Anonymous set_arg;
+    set_arg->setId(getId());
+    set_arg->setAttr("num", m_num);
+    set->setArgs1(set_arg);
+    set->setTo(getId());
+
+    Sight sight;
+    sight->setArgs1(set_arg);
+    res.push_back(sight);
 }
 
 void Stackable::DivideOperation(const Operation & op, OpVector & res)
 {
-    if (m_script->operation("divide", op, res) != 0) {
-        return;
-    }
+    int old_num = m_num;
     const std::vector<Root> & args = op->getArgs();
     std::vector<Root>::const_iterator Iend = args.end();
     for (std::vector<Root>::const_iterator I = args.begin(); I != Iend; ++I) {
         const Root & arg = *I;
+        if (arg->hasAttrFlag(Atlas::Objects::ID_FLAG)) {
+            if (arg->getId() != getId()) {
+                log(ERROR, "Divide operation has wrong ID in args");
+            }
+            continue;
+        }
         int new_num = 1;
         Element num_attr;
         if (arg->copyAttr("num", num_attr) != 0 && num_attr.isInt()) {
             new_num = num_attr.asInt();
         }
-        if (m_num <= new_num) { continue; }
+        if (m_num <= new_num) {
+            log(ERROR, "Attempt to divide entity into a chunk larger than the original");
+            continue;
+        }
+
+        m_num -= new_num;
         
         Anonymous create_arg;
-        create_arg->setParents(std::list<std::string>(1, m_type));
-        create_arg->setAttr("num", new_num);
+        create_arg->setParents(std::list<std::string>(1, m_type->name()));
+        if (new_num > 1) {
+            create_arg->setAttr("num", new_num);
+        }
+
         Create c;
         c->setArgs1(create_arg);
         c->setTo(getId());
@@ -110,4 +146,19 @@ void Stackable::DivideOperation(const Operation & op, OpVector & res)
     }
     // Currently does not send sight ops, as the Sight ops for this type of
     // thing have not been discussed
+
+    if (old_num == m_num) {
+        return;
+    }
+
+    Set set;
+    Anonymous set_arg;
+    set_arg->setId(getId());
+    set_arg->setAttr("num", m_num);
+    set->setArgs1(set_arg);
+    set->setTo(getId());
+
+    Sight sight;
+    sight->setArgs1(set_arg);
+    res.push_back(sight);
 }
