@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Thing.cpp,v 1.224 2007-12-02 23:49:07 alriddoch Exp $
+
 #include "Thing.h"
 
 #include "Script.h"
@@ -24,11 +26,10 @@
 #include "common/const.h"
 #include "common/debug.h"
 #include "common/compose.hpp"
+#include "common/Property.h"
 
 #include "common/Burn.h"
 #include "common/Nourish.h"
-#include "common/Setup.h"
-#include "common/Tick.h"
 #include "common/Update.h"
 #include "common/Pickup.h"
 #include "common/Drop.h"
@@ -44,7 +45,6 @@ using Atlas::Message::Element;
 using Atlas::Message::MapType;
 using Atlas::Objects::Root;
 using Atlas::Objects::Operation::Set;
-using Atlas::Objects::Operation::Tick;
 using Atlas::Objects::Operation::Sight;
 using Atlas::Objects::Operation::Delete;
 using Atlas::Objects::Operation::Drop;
@@ -72,40 +72,12 @@ Thing::~Thing()
     delete m_motion;
 }
 
-void Thing::SetupOperation(const Operation & op, OpVector & res)
-{
-    Appearance app;
-    Anonymous arg;
-    arg->setId(getId());
-    arg->setStamp(m_seq);
-    app->setArgs1(arg);
-
-    res.push_back(app);
-
-    if (m_script->operation("setup", op, res) != 0) {
-        return;
-    }
-
-    Tick tick;
-    tick->setTo(getId());
-
-    res.push_back(tick);
-}
-
-void Thing::ActionOperation(const Operation & op, OpVector & res)
-{
-    log(ERROR, String::compose("Action::Operation called on %1(%2), but it should be abstract", getId(), getType()).c_str());
-    if (m_script->operation("action", op, res) != 0) {
-        return;
-    }
-    Sight s;
-    s->setArgs1(op);
-    res.push_back(s);
-}
-
 void Thing::DeleteOperation(const Operation & op, OpVector & res)
 {
-    if (m_script->operation("delete", op, res) != 0) {
+    if (m_location.m_loc == 0) {
+        log(ERROR, String::compose("Deleting %1(%2) when it is not "
+                                   "in the world.", getType(), getId()));
+        assert(m_location.m_loc != 0);
         return;
     }
     // The actual destruction and removal of this entity will be handled
@@ -115,56 +87,14 @@ void Thing::DeleteOperation(const Operation & op, OpVector & res)
     res.push_back(s);
 }
 
-void Thing::BurnOperation(const Operation & op, OpVector & res)
-{
-    if (m_script->operation("burn", op, res) != 0) {
-        return;
-    }
-    if (op->getArgs().empty()) {
-        error(op, "Fire op has no argument", res, getId());
-        return;
-    }
-    MapType::const_iterator I = m_attributes.find("burn_speed");
-    if ((I == m_attributes.end()) || !I->second.isNum()) {
-        return;
-    }
-    double bspeed = I->second.asNum();
-    const Root & fire_ent = op->getArgs().front();
-    double consumed = bspeed * fire_ent->getAttr("status").asNum();
-
-    const std::string & to = fire_ent->getId();
-    Anonymous nour_ent;
-    nour_ent->setId(to);
-    nour_ent->setAttr("mass", consumed);
-
-    Set s;
-    s->setTo(getId());
-
-    Anonymous self_ent;
-    self_ent->setId(getId());
-    self_ent->setAttr("status", m_status - (consumed / m_mass));
-    s->setArgs1(self_ent);
-
-    res.push_back(s);
-
-    Nourish n;
-    n->setTo(to);
-    n->setArgs1(nour_ent);
-
-    res.push_back(n);
-}
-
 void Thing::MoveOperation(const Operation & op, OpVector & res)
 {
     debug( std::cout << "Thing::move_operation" << std::endl << std::flush;);
 
-    // FIXME This is probably here in case the op is handled by a script.
-    // Should it reall be here, or would it be better to move in after
-    // the checks, and make the script responsible for doing this if it
-    // needs to?
-    m_seq++;
-
-    if (m_script->operation("move", op, res) != 0) {
+    if (m_location.m_loc == 0) {
+        log(ERROR, String::compose("Moving %1(%2) when it is not in the world.",
+                                   getType(), getId()));
+        assert(m_location.m_loc != 0);
         return;
     }
 
@@ -193,13 +123,13 @@ void Thing::MoveOperation(const Operation & op, OpVector & res)
     if (new_loc_id != m_location.m_loc->getId()) {
         // If the LOC has not changed, we don't need to look it up, or do
         // any of the following checks.
-        new_loc = m_world->getEntity(new_loc_id);
+        new_loc = BaseWorld::instance().getEntity(new_loc_id);
         if (new_loc == 0) {
             error(op, "Move op loc does not exist", res, getId());
             return;
         }
         debug(std::cout << "LOC: " << new_loc_id << std::endl << std::flush;);
-        Entity * test_loc = new_loc;
+        LocatedEntity * test_loc = new_loc;
         for (; test_loc != 0; test_loc = test_loc->m_location.m_loc) {
             if (test_loc == this) {
                 error(op, "Attempt to move into itself", res, getId());
@@ -267,7 +197,9 @@ void Thing::MoveOperation(const Operation & op, OpVector & res)
         if (mode_attr.isString()) {
             mode = mode_attr.String();
         } else {
-            log(ERROR, String::compose("Mode on entity is a %1 in Thing::MoveOperation", Element::typeName(mode_attr.getType())).c_str());
+            log(ERROR, String::compose("Mode on entity is a \"%1\" in "
+                                       "Thing::MoveOperation",
+                                       Element::typeName(mode_attr.getType())));
         }
     }
 
@@ -276,7 +208,6 @@ void Thing::MoveOperation(const Operation & op, OpVector & res)
     // an additional Set op would be required.
     Element attr_mode;
     if (ent->copyAttr("mode", attr_mode) == 0) {
-        // FIXME
         if (!attr_mode.isString()) {
             log(ERROR, "Non string mode set in Thing::MoveOperation");
         } else {
@@ -287,14 +218,14 @@ void Thing::MoveOperation(const Operation & op, OpVector & res)
         }
     }
 
-    const double & current_time = m_world->getTime();
+    const double & current_time = BaseWorld::instance().getTime();
 
-    Point3D oldpos = m_location.pos();
+    Point3D old_pos = m_location.pos();
 
     // Update pos
     fromStdVector(m_location.m_pos, ent->getPos());
     // FIXME Quick height hack
-    m_location.m_pos.z() = m_world->constrainHeight(m_location.m_loc,
+    m_location.m_pos.z() = BaseWorld::instance().constrainHeight(m_location.m_loc,
                                                     m_location.pos(),
                                                     mode);
     m_location.update(current_time);
@@ -354,6 +285,7 @@ void Thing::MoveOperation(const Operation & op, OpVector & res)
     // Serial number must be changed regardless of whether we will use it
     ++m_motion->serialno();
 
+    // If we are moving, schedule an update to track the movement
     if (moving) {
         debug(std::cout << "Move Update in " << update_time << std::endl << std::flush;);
 
@@ -369,13 +301,24 @@ void Thing::MoveOperation(const Operation & op, OpVector & res)
     // This code handles sending Appearance and Disappearance operations
     // to this entity and others to indicate if one has gained or lost
     // sight of the other because of this movement
-    if (consts::enable_ranges && isPerceptive()) {
-        checkVisibility(oldpos, res);
+
+    // FIXME Why only for a perceptive moving entity? Surely other entities
+    // must gain/lose sight of this entity if it's moving?
+    if (isPerceptive()) {
+        checkVisibility(old_pos, res);
     }
+    m_seq++;
     updated.emit();
 }
 
-void Thing::checkVisibility(const Point3D & oldpos, OpVector & res)
+/// \brief Check changes in visibility of this entity
+///
+/// Check how this entity's position has changed since the last update
+/// and how this has affected which entities it can see, and which can see
+/// it. Return Appearance and Disappearance operations as required.
+/// @param old_pos The coordinates of this entity before the update
+/// @param res Resulting operations are returned here
+void Thing::checkVisibility(const Point3D & old_pos, OpVector & res)
 {
     debug(std::cout << "testing range" << std::endl;);
     float fromSquSize = m_location.squareBoxSize();
@@ -385,28 +328,31 @@ void Thing::checkVisibility(const Point3D & oldpos, OpVector & res)
     this_ent->setId(getId());
     this_ent->setStamp(m_seq);
 
-    EntitySet::const_iterator I = m_location.m_loc->m_contains.begin();
-    EntitySet::const_iterator Iend = m_location.m_loc->m_contains.end();
+    LocatedEntitySet::const_iterator I = m_location.m_loc->m_contains.begin();
+    LocatedEntitySet::const_iterator Iend = m_location.m_loc->m_contains.end();
     for(; I != Iend; ++I) {
-        float oldDist = squareDistance((*I)->m_location.pos(), oldpos),
-              newDist = squareDistance((*I)->m_location.pos(), m_location.pos()),
-              oSquSize = (*I)->m_location.squareBoxSize();
+        float old_dist = squareDistance((*I)->m_location.pos(), old_pos),
+              new_dist = squareDistance((*I)->m_location.pos(), m_location.pos()),
+              squ_size = (*I)->m_location.squareBoxSize();
 
         // Build appear and disappear lists, and send operations
         // Also so operations to (dis)appearing perceptive
         // entities saying that we are (dis)appearing
-        if ((*I)->isPerceptive()) {
-            bool wasInRange = ((fromSquSize / oldDist) > consts::square_sight_factor),
-                 isInRange = ((fromSquSize / newDist) > consts::square_sight_factor);
-            if (wasInRange ^ isInRange) {
-                if (wasInRange) {
+        // FIXME Should this be Entity *
+        Entity * viewer = dynamic_cast<Entity *>(*I);
+        assert(viewer != 0);
+        if (viewer->isPerceptive()) {
+            bool was_in_range = ((fromSquSize / old_dist) > consts::square_sight_factor),
+                 is_in_range = ((fromSquSize / new_dist) > consts::square_sight_factor);
+            if (was_in_range != is_in_range) {
+                if (was_in_range) {
                     // Send operation to the entity in question so it
                     // knows it is losing sight of us.
                     Disappearance d;
                     d->setArgs1(this_ent);
                     d->setTo((*I)->getId());
                     res.push_back(d);
-                } else /*if (isInRange)*/ {
+                } else /*if (is_in_range)*/ {
                     // Send operation to the entity in question so it
                     // knows it is gaining sight of us.
                     // FIXME We don't need to do this, cos its about
@@ -419,18 +365,18 @@ void Thing::checkVisibility(const Point3D & oldpos, OpVector & res)
             }
         }
         
-        bool couldSee = ((oSquSize / oldDist) > consts::square_sight_factor),
-             canSee = ((oSquSize / newDist) > consts::square_sight_factor);
-        if (couldSee ^ canSee) {
+        bool could_see = ((squ_size / old_dist) > consts::square_sight_factor),
+             can_see = ((squ_size / new_dist) > consts::square_sight_factor);
+        if (could_see ^ can_see) {
             Anonymous that_ent;
             that_ent->setId((*I)->getId());
             that_ent->setStamp((*I)->getSeq());
-            if (couldSee) {
+            if (could_see) {
                 // We are losing sight of that object
                 disappear.push_back(that_ent);
                 debug(std::cout << getId() << ": losing site of "
                                 << (*I)->getId() << std::endl;);
-            } else /*if (canSee)*/ {
+            } else /*if (can_see)*/ {
                 // We are gaining sight of that object
                 appear.push_back(that_ent);
                 debug(std::cout << getId() << ": gaining site of "
@@ -458,10 +404,6 @@ void Thing::checkVisibility(const Point3D & oldpos, OpVector & res)
 
 void Thing::SetOperation(const Operation & op, OpVector & res)
 {
-    m_seq++;
-    if (m_script->operation("set", op, res) != 0) {
-        return;
-    }
     const std::vector<Root> & args = op->getArgs();
     if (args.empty()) {
         error(op, "Set has no argument", res, getId());
@@ -472,25 +414,125 @@ void Thing::SetOperation(const Operation & op, OpVector & res)
     Sight s;
     s->setArgs1(op);
     res.push_back(s);
-    if (m_status < 0) {
-        Delete d;
-        Anonymous darg;
-        darg->setId(getId());
-        d->setArgs1(darg);
-        d->setTo(getId());
-        res.push_back(d);
-    }
+    m_seq++;
     if (m_update_flags != 0) {
         updated.emit();
     }
 }
 
+/// \brief Generate a Sight(Set) operation giving an update on named attributes
+///
+/// When another operation causes the properties of an entity to be changed,
+/// it can trigger propagation of this change by sending an Update operation
+/// nameing the attributes or properties that need to be updated. This
+/// member function handles the Operation, sending a Sight(Set) for
+/// any perceptible changes, and will in future handle persisting those
+/// changes. Should this also handle side effects?
+/// The main reason for this up is that if other ops need to generate a
+/// Set op to update attributes, there are race conditions all over the
+/// place.
+/// @param op Update operation that notifies of the changes.
+/// @param res The result of the operation is returned here.
+void Thing::updateProperties(const Operation & op, OpVector & res) const
+{
+    const std::vector<Root> & args = op->getArgs();
+    if (args.empty()) {
+       return;
+    }
+
+    RootEntity arg = smart_dynamic_cast<RootEntity>(args.front());
+    if (!arg.isValid()) {
+        error(op, "Update op has malformed args", res, getId());
+        return;
+    }
+
+    debug(std::cout << "Generating property update" << std::endl << std::flush;);
+
+    Anonymous set_arg;
+    set_arg->setId(getId());
+
+    // Check if any of the hard attributes on RootEntity of changed.
+    // All other hard attributes cannot be changed this way. Location related
+    // attrs can only be changed by a Move or move relate update.
+    // ID and PARENTS are immutable.
+    if (!arg->isDefaultName()) {
+        // Update the name
+        // FIXME How?
+    }
+    if (!arg->isDefaultContains()) {
+        // Update the contains
+        // FIXME Really?
+        // Surely better to go through the Property?
+    }
+
+    // FIXME SLOW!
+    MapType attrs = arg->asMessage();
+    MapType::const_iterator I = attrs.begin();
+    MapType::const_iterator Iend = attrs.end();
+
+    PropertyDict::const_iterator J;
+    PropertyDict::const_iterator Jend = m_properties.end();
+
+    MapType::const_iterator K;
+    MapType::const_iterator Kend = m_attributes.end();
+
+    for (; I != Iend; ++I) {
+        const std::string & attr = I->first;
+        debug(std::cout << "  " << attr << std::endl << std::flush;);
+
+        J = m_properties.find(attr);
+        if (J != Jend) {
+            // Dump the property value into the Sight(Set()) arg
+            J->second->add(attr, set_arg);
+        } else {
+            K = m_attributes.find(attr);
+            if (K != Kend) {
+                // Copy the attribute into the Sight(Set()) arg
+                set_arg->setAttr(attr, K->second);
+            } else {
+                error(op, String::compose("Got update for non-existant attribute or property \"%1\"",
+                                          attr), res, getId());
+            }
+        }
+    }
+
+    Set set;
+    set->setTo(getId());
+    set->setFrom(getId());
+    set->setSeconds(op->getSeconds());
+    set->setArgs1(set_arg);
+
+    Sight sight;
+    sight->setArgs1(set);
+    res.push_back(sight);
+}
+
 void Thing::UpdateOperation(const Operation & op, OpVector & res)
 {
+    // If it has no refno, then it is a generic request to broadcast
+    // an update of some properties which have changed.
+    if (op->isDefaultRefno()) {
+        updateProperties(op, res);
+        return;
+    }
+
+    // If LOC is null, this cannot be part of the world, or must be the
+    // world itself, so should not be involved in any movement.
+    if (m_location.m_loc == 0) {
+        log(ERROR, String::compose("Updating %1(%2) when it is not in the world.",
+                                   getType(), getId()));
+        return;
+    }
+
+    // If it has a refno, then it is a movement update. If it does not
+    // match the current movement serialno, then its obsolete, and can
+    // be discarded.
     if (op->getRefno() != m_motion->serialno()) {
         return;
     }
 
+    // If somehow a movement update arrives with the correct refno, but
+    // we are not moving, then something has gone wrong.
     if (!m_location.velocity().isValid() ||
         m_location.velocity().sqrMag() < WFMATH_EPSILON) {
         log(ERROR, "Update got for entity not moving");
@@ -501,9 +543,7 @@ void Thing::UpdateOperation(const Operation & op, OpVector & res)
     // than in the mind interface. The details will be sorted by a new type
     // of object which will handle the specifics.
 
-    // FIXME Check whether this is still the next update.
-
-    const double & current_time = m_world->getTime();
+    const double & current_time = BaseWorld::instance().getTime();
     double time_diff = current_time - m_location.timeStamp();
 
     std::string mode;
@@ -514,11 +554,13 @@ void Thing::UpdateOperation(const Operation & op, OpVector & res)
         if (mode_attr.isString()) {
             mode = mode_attr.String();
         } else {
-            log(ERROR, String::compose("Mode on entity is a %1 in Thing::MoveOperation", Element::typeName(mode_attr.getType())).c_str());
+            log(ERROR, String::compose("Mode on entity is a \"%1\" "
+                                       "in Thing::UpdateOperation",
+                                       Element::typeName(mode_attr.getType())));
         }
     }
 
-    Point3D oldpos = m_location.pos();
+    Point3D old_pos = m_location.pos();
 
     bool moving = true;
 
@@ -543,7 +585,7 @@ void Thing::UpdateOperation(const Operation & op, OpVector & res)
 
     // Adjust the position to world constraints - essentially fit
     // to the terrain height at this stage.
-    m_location.m_pos.z() = m_world->constrainHeight(m_location.m_loc,
+    m_location.m_pos.z() = BaseWorld::instance().constrainHeight(m_location.m_loc,
                                                     m_location.pos(),
                                                     mode);
     m_location.update(current_time);
@@ -601,8 +643,75 @@ void Thing::UpdateOperation(const Operation & op, OpVector & res)
     // This code handles sending Appearance and Disappearance operations
     // to this entity and others to indicate if one has gained or lost
     // sight of the other because of this movement
-    if (consts::enable_ranges && isPerceptive()) {
-        checkVisibility(oldpos, res);
+
+    // FIXME Why only for a perceptive moving entity? Surely other entities
+    // must gain/lose sight of this entity if it's moving?
+    if (isPerceptive()) {
+        checkVisibility(old_pos, res);
     }
     updated.emit();
 }
+
+void Thing::LookOperation(const Operation & op, OpVector & res)
+{
+    Sight s;
+
+    Anonymous new_ent;
+    addToEntity(new_ent);
+    s->setArgs1(new_ent);
+
+    s->setTo(op->getFrom());
+
+    res.push_back(s);
+}
+
+void Thing::CreateOperation(const Operation & op, OpVector & res)
+{
+    const std::vector<Root> & args = op->getArgs();
+    if (args.empty()) {
+       return;
+    }
+    try {
+        RootEntity ent = smart_dynamic_cast<RootEntity>(args.front());
+        if (!ent.isValid()) {
+            error(op, "Entity to be created is malformed", res, getId());
+            return;
+        }
+        const std::list<std::string> & parents = ent->getParents();
+        if (parents.empty()) {
+            error(op, "Entity to be created has empty parents", res, getId());
+            return;
+        }
+        if (!ent->hasAttrFlag(Atlas::Objects::Entity::LOC_FLAG) &&
+            (m_location.m_loc != 0)) {
+            ent->setLoc(m_location.m_loc->getId());
+            if (!ent->hasAttrFlag(Atlas::Objects::Entity::POS_FLAG)) {
+                ::addToEntity(m_location.pos(), ent->modifyPos());
+            }
+        }
+        const std::string & type = parents.front();
+        debug( std::cout << getId() << " creating " << type;);
+
+        Entity * obj = BaseWorld::instance().addNewEntity(type,ent);
+
+        if (obj == 0) {
+            error(op, "Create op failed.", res, op->getFrom());
+            return;
+        }
+
+        Operation c(op.copy());
+
+        Anonymous new_ent;
+        obj->addToEntity(new_ent);
+        c->setArgs1(new_ent);
+
+        Sight s;
+        s->setArgs1(c);
+        res.push_back(s);
+    }
+    catch (Atlas::Message::WrongTypeException&) {
+        log(ERROR, "EXCEPTION: Malformed object to be created");
+        error(op, "Malformed object to be created", res, getId());
+        return;
+    }
+}
