@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: World.h,v 1.35 2007-07-04 21:20:41 alriddoch Exp $
+
 #ifndef RULESETS_WORLD_H
 #define RULESETS_WORLD_H
 
@@ -29,10 +31,11 @@ typedef Thing World_parent;
 
 typedef std::map<int, std::set<int> > PointSet;
 
-/// This is the in-game entity class used to represent the world.
+/// \brief This is the in-game entity class used to represent the world.
 ///
 /// I added this because I was not happy with the way the old object model
 /// used an out of game object of type WorldRouter to represent the world.
+/// \ingroup EntityClasses
 class World : public World_parent {
   protected:
     /// Terrain manager for the world.
@@ -72,17 +75,10 @@ class World : public World_parent {
     float getHeight(float x, float y);
     int getSurface(const Point3D &,  int &);
 
-    void delveOperation(const Operation &, OpVector &);
-    void digOperation(const Operation &, OpVector &);
-    void mowOperation(const Operation &, OpVector &);
-
     virtual void EatOperation(const Operation &, OpVector &);
     virtual void LookOperation(const Operation &, OpVector &);
-    virtual void BurnOperation(const Operation &, OpVector &);
     virtual void DeleteOperation(const Operation &, OpVector &);
     virtual void MoveOperation(const Operation &, OpVector &);
-    virtual void SetOperation(const Operation &, OpVector &);
-    virtual void OtherOperation(const Operation &, OpVector &);
 };
 
 #endif // RULESETS_WORLD_H
