@@ -12,7 +12,7 @@ if const.server_python:
     from mind.BaseMind import BaseMind
 else:
     from Vector3D import Vector3D
-    from world.objects.Thing import Thing
+    from cyphesis.Thing import Thing
     from server import WorldTime
     BaseMind = Thing
 
@@ -36,7 +36,24 @@ def get_dict_func(self, func_str, func_undefined):
     return func
 
 class NPCMind(BaseMind):
-    """base class for all NPCs"""
+    """Mind class for most mobile entities in the game.
+
+    An NPCMind object is associated with all NPC and similar entities on a
+    game server. It handles perception data from the world, tracks what
+    the NPC knows about, and handles its goals.
+
+    The data is organised into three key data structures:
+
+    self.map is handled by the underlying C++ code, and contains a copy of
+    all the entities in the world that this NPC is currently able to perceive.
+
+    self.knowledge contains data triples which define relations between
+    entities.
+
+    self.goals and self.trigger_goals contain trees of goals which represent
+    current and potential activities that NPC might engage in. self.goals are
+    goals which are checked each tick, self.trigger_goals are goals which
+    are activated by an event."""
     ########## Initialization
     def __init__(self, cppthing):
         self.cinit(cppthing)
@@ -72,10 +89,13 @@ class NPCMind(BaseMind):
         return event_name, sub_op
     ########## Map updates
     def add_map(self, obj):
+        """Hook called by underlying map code when an entity is added."""
         #print "Map add",obj
         pass
     def update_map(self, obj):
-        """fix ownership category for objects owned temporary under 'Foo' type"""
+        """Hook called by underlying map code when an entity is added.
+
+        Fix ownership category for objects owned temporary under 'Foo' type."""
         #print "Map update",obj
         foo_lst = self.things.get('Foo',[])
         for foo in foo_lst[:]: #us copy in loop, because it might get modified
@@ -84,6 +104,7 @@ class NPCMind(BaseMind):
                 self.remove_thing(foo)
                 self.add_thing(obj)
     def delete_map(self, obj):
+        """Hook called by underlying map code when an entity is added."""
         #print "Map delete",obj
         self.remove_thing(obj)
     ########## Operations
@@ -118,25 +139,26 @@ class NPCMind(BaseMind):
     def load_operation(self, op):
         print "LOAD"
     ########## Sight operations
-    def sight_create_operation(self, original_op, op):
+    def sight_create_operation(self, op):
+        """Note our ownership of entities we created."""
         #BaseMind version overridden!
         obj=self.map.add(op[0], op.getSeconds())
-        if original_op.from_==self.id:
+        if op.to==self.id:
             self.add_thing(obj)
-    def sight_move_operation(self, original_op, op):
+    def sight_move_operation(self, op):
         """change position in out local map"""
         obj=self.map.update(op[0], op.getSeconds())
         if obj.location.parent.id==self.id:
             self.add_thing(obj)
-            if original_op.from_ != self.id:
+            if op.to != self.id:
                 self.transfers.append((op.from_, obj.id))
-            if obj.type[0]=="coin":
+            if obj.type[0]=="coin" and op.from_ != self.id:
                 self.money_transfers.append([op.from_, 1])
                 return Operation("imaginary", Entity(description="accepts"))
     #replaced with dynamically added add_extinguish_fire -goal
     #ie: NPC is first teached that when it sees "sight_burn" it needs to
     #execute above goal, which then adds extinguish_fire goal and executes it
-##     def sight_burn_operation(self, original_op, op):
+##     def sight_burn_operation(self, op):
 ##         #CHEAT!:
 ##         fire=self.map.get(op[0].id)
 ##         if fire:
@@ -148,23 +170,30 @@ class NPCMind(BaseMind):
 ##         log.debug(2,"sight_burn_operation: add goal updating")
     ########## Talk operations
     def admin_sound(self, op):
-        return op[0].from_==self.id
+        assert(op.from_ == op.to)
+        return op.from_ == self.id
 
     def interlinguish_warning(self, op, say, msg):
         log.debug(1,str(self.id)+" interlinguish_warning: "+str(msg)+\
                   ": "+str(say[0].lexlink.id[1:]),op)
     def interlinguish_desire_verb3_buy_verb1_operation(self, op, say):
+        """Handle a sentence of the form 'I would iike to buy a ....'
+
+        Check if we have any of the type of thing the other character is
+        interested in, and whether we know what price to sell at. If so
+        set up the transaction goal, which offers to sell it."""
         object=say[1].word
         thing=self.things.get(object)
         if thing:
             price=self.get_knowledge("price", object)
             if not price:
                 return
-            goal=mind.goals.common.misc_goal.transaction(object, op.from_, price)
-            who=self.map.get(op.from_)
+            goal=mind.goals.common.misc_goal.transaction(object, op.to, price)
+            who=self.map.get(op.to)
             self.goals.insert(0,goal)
-            return Operation("talk", Entity(say=who.name+" one "+object+" will be "+str(price)+" coins")) + self.face(who)
+            return Operation("talk", Entity(say=self.thing_name(who)+" one "+object+" will be "+str(price)+" coins")) + self.face(who)
     def interlinguish_desire_verb3_operation(self, op, say):
+        """Handle a sentence of the form 'I would like to ...'"""
         object=say[2:]
         verb=interlinguish.get_verb(object)
         operation_method=self.find_op_method(verb,"interlinguish_desire_verb3_",
@@ -182,6 +211,10 @@ class NPCMind(BaseMind):
         else:
             return self.interlinguish_warning(op,say,"Unkown assertion")
     def interlinguish_know_verb1_operation(self, op, say):
+        """Handle a sentence of the form 'know subject predicate object'
+
+        Accept admin instruction about knowledge, and store the triple
+        in our knowledge base."""
         if not self.admin_sound(op):
             return self.interlinguish_warning(op,say,"You are not admin")
         subject=say[1].word
@@ -197,6 +230,11 @@ class NPCMind(BaseMind):
         else:
             self.add_knowledge(predicate,subject,object)
     def interlinguish_tell_verb1_operation(self, op, say):
+        """Handle a sentence of the form 'Tell (me) ....'
+
+        Accept queries about what we know. Mostly this is for debugging
+        and for the time being it is useful to answer these queries no matter
+        who hasks."""
         # Currently no checking for trus here.
         # We are being liberal with interpretation of "subject" and "object"
         subject=say[1].word
@@ -219,15 +257,52 @@ class NPCMind(BaseMind):
             elif k_type!=StringType:
                 k='difficult to explain'
             elif predicate=='about':
-                return Operation('talk', Entity(say=k)) + self.face(self.map.get(op.from_))
-            return Operation('talk', Entity(say="The "+predicate+" of "+object+" is "+k)) + self.face(self.map.get(op.from_))
+                return Operation('talk', Entity(say=k)) + \
+                       self.face(self.map.get(op.to))
+            return Operation('talk', Entity(say="The " + predicate + " of " +
+                                                object + " is " + k)) + \
+                   self.face(self.map.get(op.to))
+    def interlinguish_list_verb1_operation(self, op, say):
+        """Handle a sentence of the form 'List (me) ....'
+
+        Accept queries about what we know. Mostly this is for debugging
+        and for the time being it is useful to answer these queries no matter
+        who hasks."""
+        # Currently no checking for trus here.
+        # We are being liberal with interpretation of "subject" and "object"
+        subject=say[1].word
+        predicate=say[2].word
+        if not hasattr(self.knowledge, predicate):
+            return None
+        d=getattr(self.knowledge, predicate)
+        res = Message()
+        res = res + self.face(self.map.get(op.to))
+        for key in d:
+            res = res + Operation('talk',
+                                  Entity(say="The " + predicate + " of " + key +
+                                             " is " + str(d[key])))
+        return res
     def interlinguish_learn_verb1_operation(self, op, say):
+        """Handle a sentence of the form 'learn ....'
+
+        The learn sentence contains two components, and can hardly
+        be concidered a real sentence at all. The first is the verb
+        to be associated with a goal activity, and the second is
+        the python fragment executed in order to create the goal instance.
+        Obviously accepting a code fragment from the client is very
+        dangerous, and we need to be carefull that the admin_sound() check
+        works."""
         if not self.admin_sound(op):
             return self.interlinguish_warning(op,say,"You are not admin")
         subject=say[1].word
         object=say[2].word
         self.add_goal(subject,object)
     def interlinguish_own_verb1_operation(self, op, say):
+        """Handle a sentence of the form ' own ...'
+
+        Sentences of this form from the admin inform us that we own an
+        entity. This is essential when an entity needs to be used as a
+        tool, or raw material."""
         if not self.admin_sound(op):
             return self.interlinguish_warning(op,say,"You are not admin")
 ##         print self,"own:",say[1].word,say[2].word
@@ -247,7 +322,15 @@ class NPCMind(BaseMind):
         #CHEAT!: any way to handle these?
         pass
     ########## Sound operations
-    def sound_talk_operation(self, original_op, op):
+    def sound_talk_operation(self, op):
+        """Handle the sound of a talk operation from another character.
+
+        The spoken sentence comes in as a sentence string, which
+        is converted into a structure representation by the interlinguish
+        code. Embedded in the structure is the interlinguish string which
+        is then used to call methods and activate triggers, such as
+        dynamic goals."""
+
         talk_entity=op[0]
         res = Message()
         if interlinguish.convert_english_to_interlinguish(self,talk_entity):
@@ -255,16 +338,24 @@ class NPCMind(BaseMind):
             verb=interlinguish.get_verb(say)
             operation_method=self.find_op_method(verb,"interlinguish_",
                                   self.interlinguish_undefined_operation)
-            res = res + self.call_interlinguish_triggers(verb, "interlinguish_", original_op,say)
+            res = res + self.call_interlinguish_triggers(verb, "interlinguish_", op, say)
         else:
             operation_method=self.talk_undefined_operation
             say=talk_entity
             if hasattr(say,"say"): say=say.say
         log.debug(3,"talk: "+str(operation_method))
-        res = res + operation_method(original_op,say)
+        res = res + operation_method(op,say)
         return res
     ########## Other operations
     def call_interlinguish_triggers(self, verb, prefix, op, say):
+        """Call trigger goals that have registered a trigger string that
+        matches the current interlinguish string.
+
+        Given an interlinguish verb string, and a prefix, find any trigger
+        goals that should be activated by the combined trigger string, and
+        activate them."""
+        # FIXME Don't need this call to get_op_name_and_sub, as we don't use
+        # the result.
         null_name, sub_op = self.get_op_name_and_sub(op)
         event_name = prefix+verb
         reply = Message()
@@ -341,20 +432,23 @@ class NPCMind(BaseMind):
                     return cmp=='>'
         return 1
     ########## things we own
-    def add_thing(self,thing): 
+    def thing_name(self,thing):
+        if hasattr(thing, 'name'):
+            return thing.name
+        return thing.type[0]
+    ########## things we own
+    def add_thing(self,thing):
         """I own this thing"""
         #CHEAT!: this feature not yet supported
 ##         if not thing.location:
 ##             thing.location=self.get_knowledge("location",thing.place)
         log.debug(3,str(self)+" "+str(thing)+" before add_thing: "+str(self.things))
         #thought about owing thing
-        desc="I own %s." % thing.name
+        name = self.thing_name(thing)
+        desc="I own %s." % name
         what=thing.as_entity()
         ent = Entity(description=desc, what=what)
         self.send(Operation("thought",ent))
-        name=thing.name
-        if not name or name=='':
-            name=thing.type[0]
         dictlist.add_value(self.things,name,thing)
         log.debug(3,"\tafter: "+str(self.things))
     def find_thing(self, thing):
@@ -362,7 +456,7 @@ class NPCMind(BaseMind):
             #return found list or empty list
             return self.things.get(thing,[])
         found=[]
-        for t in self.things.get(thing.name,[]):
+        for t in self.things.get(self.thing_name(thing),[]):
             if t==thing: found.append(t)
         return found
     def remove_thing(self, thing):
