@@ -16,6 +16,8 @@ try:
 except ImportError:
   from whrandom import *
 
+import types
+
 ############################ MOVE ME ####################################
 
 class move_me(Goal):
@@ -113,6 +115,23 @@ class move_me_area(Goal):
         #print "Latching at location"
         self.arrived=1
 
+############################ MOVE ME PLACE ####################################
+
+class move_me_place(move_me):
+    def __init__(self, what):
+        Goal.__init__(self, "move me to a place where I can get something",
+                      self.am_I_at_loc,
+                      [self.move_to_loc])
+        self.what = what
+        self.vars = ["what"]
+    def get_location_instance(self, me):
+        location = me.get_knowledge("place", self.what)
+        if type(location) == StringType:
+            location = me.get_knowledge("location", location)
+        if not location:
+            return None
+        return location
+
 ############################ MOVE THING ####################################
 
 class move_it(Goal):
@@ -216,31 +235,35 @@ class move_me_to_focus(Goal):
         Goal.__init__(self,"move me to this thing",
                       self.am_i_at_it,
                       [self.move_me_to_it])
-        self.what=what
+        if type(what) == types.ListType:
+            self.what = what
+        else:
+            self.what = [ what ]
         self.vars=["what"]
     def am_i_at_it(self, me):
-        what = self.what
-        if type(what)==StringType:
-            id=me.get_knowledge('focus',what)
-            if id==None: return 0
-            what=me.map.get(id)
-            if what==None: return 0
-        if square_horizontal_distance(me.location, what.location) < 4: # 2 * 2
-            return 1
-        else:
-            return 0
+        for what in self.what:
+            id = me.get_knowledge('focus', what)
+            if id == None: continue
+            thing = me.map.get(id)
+            if thing == None:
+                me.remove_knowledge('focus', what)
+                continue
+            if square_horizontal_distance(me.location, thing.location) < 4:
+                return 1
+        return 0
 
     def move_me_to_it(self, me):
-        what = self.what
-        if type(what)==StringType:
-            id=me.get_knowledge('focus',what)
-            if id==None: return
-            what=me.map.get(id)
-            if what==None: return
-        target=what.location.copy()
-        if target.parent.id==me.location.parent.id:
-            target.velocity=me.location.coordinates.unit_vector_to(target.coordinates)
-            return Operation("move", Entity(me.id, location=target))
+        for what in self.what:
+            id = me.get_knowledge('focus', what)
+            if id == None: continue
+            thing = me.map.get(id)
+            if thing == None:
+                me.remove_knowledge('focus', what)
+                return
+            target=thing.location.copy()
+            if target.parent.id==me.location.parent.id:
+                target.velocity=me.location.coordinates.unit_vector_to(target.coordinates)
+                return Operation("move", Entity(me.id, location=target))
 
 ############################ MOVE THING TO ME ####################################
 
@@ -276,28 +299,37 @@ class pick_up_focus(Goal):
                       self.is_it_with_me,
                       [move_me_to_focus(what),
                        self.pick_it_up])
-        self.what=what
+        if type(what) == types.ListType:
+            self.what = what
+        else:
+            self.what = [ what ]
         self.vars=["what"]
     def is_it_with_me(self, me):
         #CHEAT!: cludge
-        what=self.what
-        if type(what)==StringType:
-            id=me.get_knowledge('focus',what)
-            if id==None: return 0
-            what=me.map.get(id)
-            if what==None: return 0
-        if what.location.parent.id!=me.id:
-            if what.location.parent.id!=me.location.parent.id:
-                me.remove_knowledge('focus',self.what)
-        return what.location.parent.id==me.id
+        for what in self.what:
+            id=me.get_knowledge('focus', what)
+            if id == None: continue
+            thing = me.map.get(id)
+            if thing == None:
+                me.remove_knowledge('focus', what)
+                continue
+            # If its not not near us on the ground, forget about it.
+            if thing.location.parent.id != me.location.parent.id:
+                me.remove_knowledge('focus', what)
+                continue
+            if thing.location.parent.id != me.id:
+                return 0
+        return 1
     def pick_it_up(self, me):
-        what=self.what
-        if type(what)==StringType:
-            id=me.get_knowledge('focus',what)
-            if id==None: return
-            what=me.map.get(id)
-            if what==None: return
-        return Operation("move", Entity(id, location=Location(me, Point3D(0,0,0))))
+        for what in self.what:
+            id=me.get_knowledge('focus', what)
+            if id==None: continue
+            thing = me.map.get(id)
+            if thing == None:
+                me.remove_knowledge('focus', what)
+                continue
+            if thing.location.parent.id != me.id:
+                return Operation("move", Entity(id, location=Location(me, Point3D(0,0,0))))
 
 ############################ WANDER ####################################
 
@@ -316,7 +348,7 @@ class wander(Goal):
 ############################ WANDER & SEARCH ############################
 
 class search(Goal):
-    def __init__(self, me, what):
+    def __init__(self, what):
         Goal.__init__(self, "search for a thing",
                       self.do_I_have,
                       [wander(false),
@@ -342,12 +374,11 @@ class pursuit(Goal):
         return not me.mem.recall_place(me.location,self.range,self.what)
     def run(self, me):
         lst_of_what = me.mem.recall_place(me.location,self.range,self.what)
-        if lst_of_what==[]: return
+        if not lst_of_what or len(lst_of_what)==0: return
         dist_vect=distance_to(me.location,lst_of_what[0].location).unit_vector()
         multiply = const.base_velocity * self.direction * const.basic_tick
         loc = Location(me.location.parent)
         loc.coordinates =  me.location.coordinates + (dist_vect * multiply)
-        #print me,multiply,dist,loc.coordinates.distance(other_xyz)
         ent=Entity(me.id,location=loc)
         return Operation("move",ent)
 
@@ -415,6 +446,9 @@ class accompany(Goal):
         self.vars=["who"]
     def am_i_with(self, me):
         who=me.map.get(self.who)
+        if who == None:
+            self.irrelevant = 1
+            return 1
         dist=distance_to(me.location, who.location)
         # Are we further than 3 metres away
         if dist.square_mag() > 25:
@@ -431,6 +465,9 @@ class accompany(Goal):
             return 1
     def follow(self, me):
         who=me.map.get(self.who)
+        if who == None:
+            self.irrelevant = 1
+            return
         dist=distance_to(me.location, who.location)
         target = Location(me.location.parent)
         square_dist=dist.square_mag()
