@@ -2,7 +2,7 @@
 #Copyright (C) 1999 Al Riddoch (See the file COPYING for details).
 
 from atlas import *
-from world.objects.Thing import Thing
+from cyphesis.Thing import Thing
 from common import log,const
 try:
   from random import *
@@ -10,15 +10,19 @@ except ImportError:
   from whrandom import *
 
 class Weather(Thing):
+    def setup_operation(self, op):
+        return Operation("tick",to=self)
     def tick_operation(self, op):
         res = Message()
         optick = Operation("tick", to=self)
         res = res + optick
-        if self.rain<0.5:
-            optick.setFutureSeconds(randint(120,300))
-            self.rain=1.0
+        if self.rain<0.1:
+            optick.setFutureSeconds(randint(60,300))
+            self.rain=uniform(0.1, 0.9)
+            self.visibility=10/(self.rain * self.rain)
         else:
-            optick.setFutureSeconds(randint(600,1200))
+            optick.setFutureSeconds(randint(600,2400))
             self.rain=0.0
-        res = res+Operation("set", Entity(self.id,rain=self.rain), to=self)
+            self.visibility=1000
+        res = res+Operation("set", Entity(self.id,rain=self.rain,visibility=self.visibility), to=self)
         return res
