@@ -3,7 +3,7 @@
 
 from atlas import *
 
-from world.objects.Thing import Thing
+from cyphesis.Thing import Thing
 
 class Garment(Thing):
     """This is base class for clothing. This mechanism is just a quick hacky
