@@ -1,6 +1,6 @@
 from atlas import *
 
-from world.objects.Thing import Thing
+from cyphesis.Thing import Thing
 from world.objects.plants.seeds.Seed import Seed
 
 import atlas
