@@ -3,7 +3,7 @@
 
 from atlas import *
 
-from world.objects.Thing import Thing
+from cyphesis.Thing import Thing
 
 class Cleaver(Thing):
     """This is base class for axes, this one just ordinary axe"""
