@@ -1,7 +1,7 @@
 #This file is distributed under the terms of the GNU General Public license.
 #Copyright (C) 2005 Al Riddoch (See the file COPYING for details).
 
-from world.objects.Thing import Thing
+from cyphesis.Thing import Thing
 
 class Rope(Thing):
     def attach_operation(self, op): pass
