@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Account.cpp,v 1.154 2007-11-28 20:22:43 alriddoch Exp $
+
 #include "Account.h"
 
 #include "Connection_methods.h"
@@ -57,6 +59,13 @@ using Atlas::Objects::smart_dynamic_cast;
 
 static const bool debug_flag = false;
 
+/// \brief Account constructor
+///
+/// @param conn Network Connection creating this Account
+/// @param uname Username for this account
+/// @param passwd Password for this account
+/// @param id String identifier for this account
+/// @param intId Integer identifier for this account
 Account::Account(Connection * conn, const std::string & uname,
                  const std::string& passwd, const std::string & id, long intId)
                  : OOGThing(id, intId), m_connection(conn),
@@ -68,6 +77,9 @@ Account::~Account()
 {
 }
 
+/// \brief Called when the Character has been removed from the world.
+///
+/// @param id Integer identifier of the Character destroyed.
 void Account::characterDestroyed(long id)
 {
     m_charactersDict.erase(id);
@@ -76,6 +88,9 @@ void Account::characterDestroyed(long id)
     }
 }
 
+/// \brief Add a Character to those that belong to this Account
+///
+/// @param chr Character object to be added
 void Account::addCharacter(Entity * chr)
 {
     Character * pchar = dynamic_cast<Character *>(chr);
@@ -86,6 +101,10 @@ void Account::addCharacter(Entity * chr)
     chr->destroyed.connect(sigc::bind(sigc::mem_fun(this, &Account::characterDestroyed), chr->getIntId()));
 }
 
+/// \brief Create a new Character and add it to this Account
+///
+/// @param typestr The type name of the Character to be created
+/// @param ent Atlas description of the Character to be created
 Entity * Account::addNewCharacter(const std::string & typestr,
                                   const RootEntity & ent)
 {
@@ -99,11 +118,9 @@ Entity * Account::addNewCharacter(const std::string & typestr,
     debug(std::cout << "Added" << std::endl << std::flush;);
     assert(chr->m_location.isValid());
     debug(std::cout << "Location set to: " << chr->m_location << std::endl << std::flush;);
-    Character * pchar = dynamic_cast<Character *>(chr);
-    if (pchar != 0) {
-        pchar->m_externalMind = new ExternalMind(*m_connection,
-                                                 pchar->getId(),
-                                                 pchar->getIntId());
+    Character * character = dynamic_cast<Character *>(chr);
+    if (character != 0) {
+        m_connection->connectAvatar(character);
         // Only genuinely playable characters should go in here. Otherwise
         // if a normal entity gets into the account, and connection, it
         // starts getting hard to tell whether or not they exist.
@@ -113,36 +130,81 @@ Entity * Account::addNewCharacter(const std::string & typestr,
         if (consts::enable_persistence) {
             Persistance::instance()->addCharacter(*this, *chr);
         }
-    }
 
-    // Hack in default objects
-    // This needs to be done in a generic way
-    Anonymous create_arg;
-    create_arg->setParents(std::list<std::string>(1,"coin"));
-    ::addToEntity(Point3D(0,0,0), create_arg->modifyPos());
-    create_arg->setLoc(chr->getId());
-    create_arg->setName("coin");
-    // FIXME We can probably send the same op 10 times, rather than create 10
-    // FIXME alternatively we can set 10 args on one op
-    for(int i = 0; i < 10; i++) {
+        // Hack in default objects
+        // This needs to be done in a generic way
+        Anonymous create_arg;
+        create_arg->setParents(std::list<std::string>(1,"coin"));
+        ::addToEntity(Point3D(0,0,0), create_arg->modifyPos());
+        create_arg->setLoc(chr->getId());
+        create_arg->setName("coin");
+        // FIXME We can probably send the same op 10 times, rather than create 10
+        // FIXME alternatively we can set 10 args on one op
+        for(int i = 0; i < 10; i++) {
+            Create c;
+            c->setTo(chr->getId());
+            c->setArgs1(create_arg);
+            world.message(c, *chr);
+        }
+
+        create_arg = create_arg.copy();
+        create_arg->setParents(std::list<std::string>(1, "shirt"));
+        create_arg->setName("shirt");
         Create c;
         c->setTo(chr->getId());
         c->setArgs1(create_arg);
         world.message(c, *chr);
+
+        create_arg = create_arg.copy();
+        create_arg->setParents(std::list<std::string>(1, "trousers"));
+        create_arg->setName("trousers");
+        c = Create();
+        c->setTo(chr->getId());
+        c->setArgs1(create_arg);
+        world.message(c, *chr);
+
+        create_arg = create_arg.copy();
+        create_arg->setParents(std::list<std::string>(1, "cloak"));
+        create_arg->setName("cloak");
+        c = Create();
+        c->setTo(chr->getId());
+        c->setArgs1(create_arg);
+        world.message(c, *chr);
+
+        create_arg = create_arg.copy();
+        create_arg->setParents(std::list<std::string>(1, "boots"));
+        create_arg->setName("boots");
+        c = Create();
+        c->setTo(chr->getId());
+        c->setArgs1(create_arg);
+        world.message(c, *chr);
+
+        create_arg = create_arg.copy();
+        create_arg->setParents(std::list<std::string>(1, "hat"));
+        create_arg->setName("hat");
+        c = Create();
+        c->setTo(chr->getId());
+        c->setArgs1(create_arg);
+        world.message(c, *chr);
+
     }
 
-    logEvent(TAKE_CHAR, String::compose("%1 %2 %3 Created character %4(%5) by account %6", m_connection->getId(), getId(), chr->getId(), chr->getName(), chr->getType(), m_username).c_str());
+    logEvent(TAKE_CHAR, String::compose("%1 %2 %3 Created character (%4) "
+                                        "by account %5",
+                                        m_connection->getId(),
+                                        getId(),
+                                        chr->getId(),
+                                        chr->getType(),
+                                        m_username));
 
     return chr;
 }
 
-void Account::LogoutOperation(const Operation & op, OpVector &)
+void Account::LogoutOperation(const Operation & op, OpVector & res)
 {
-    std::cout << "Account logout: " << getId() << std::endl;
-
-    // It is possible this account is being forcibly logged out at a time
-    // when it is not logged in.
     if (m_connection == 0) {
+        log(ERROR, "Account::LogoutOperation on account that doesn't seem to "
+                   "be connected.");
         return;
     }
 
@@ -153,7 +215,6 @@ void Account::LogoutOperation(const Operation & op, OpVector &)
     }
     info->setFrom(getId());
     info->setTo(getId());
-    // FIXME Direct send rather than reply - requires local refno handling
     m_connection->send(info);
     m_connection->disconnect();
 }
@@ -225,44 +286,40 @@ void Account::CreateOperation(const Operation & op, OpVector & res)
         return;
     }
     
-#if 0
     const std::string & typestr = parents.front();
 
     if (characterError(op, arg, res)) {
         return;
     }
-#else
-    // Interim fix because of weird client interface
-    std::string typestr = parents.front();
-    
-    if (characterError(op, arg, res)) {
-        if (typestr == "__bad__type__") {
-            return;
-        }
-        res.clear();
-        if (!Inheritance::instance().hasClass(typestr)) {
-            Inheritance::instance().addChild(atlasClass(typestr, "settler"));
-        }
-        typestr = "settler";
-    }
-#endif
 
     debug( std::cout << "Account creating a " << typestr << " object"
                      << std::endl << std::flush; );
 
-    BaseEntity * obj = addNewCharacter(typestr, arg);
+    Anonymous new_character;
+    new_character->setParents(std::list<std::string>(1, typestr));
+    new_character->setAttr("status", 0.024);
+    if (!arg->isDefaultName()) {
+        new_character->setName(arg->getName());
+    }
 
-    if (obj == 0) {
+    Entity * entity = addNewCharacter(typestr, new_character);
+
+    if (entity == 0) {
         error(op, "Character creation failed", res, getId());
         return;
     }
 
-    Info info;
-    Anonymous info_arg;
-    obj->addToEntity(info_arg);
-    info->setArgs1(info_arg);
-    res.push_back(info);
+    Character * character = dynamic_cast<Character *>(entity);
+    if (character != 0) {
+        // Inform the client that it has successfully subscribed
+        Info info;
+        Anonymous info_arg;
+        entity->addToEntity(info_arg);
+        info->setArgs1(info_arg);
+        res.push_back(info);
+    }
 
+    // Inform the client of the newly created character
     Sight sight;
     sight->setTo(getId());
     Anonymous sight_arg;
@@ -373,6 +430,14 @@ void Account::TalkOperation(const Operation & op, OpVector & res)
 {
     const std::vector<Root> & args = op->getArgs();
     if (args.empty()) {
+        error(op, "Talk has no args", res, getId());
+        return;
+    }
+
+    RootEntity arg = smart_dynamic_cast<RootEntity>(args.front());
+
+    if (!arg.isValid()) {
+        error(op, "Talk arg is malformed", res, getId());
         return;
     }
 
@@ -383,12 +448,6 @@ void Account::TalkOperation(const Operation & op, OpVector & res)
     if (!op->isDefaultSerialno()) {
         s->setRefno(op->getSerialno());
     }
-    RootEntity arg = smart_dynamic_cast<RootEntity>(args.front());
-
-    if (!arg.isValid()) {
-        error(op, "Talk arg is malformed", res, getId());
-        return;
-    }
 
     if (arg->hasAttrFlag(Atlas::Objects::Entity::LOC_FLAG)) {
         s->setTo(arg->getLoc());
