@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Admin.cpp,v 1.117 2007-11-15 02:07:05 alriddoch Exp $
+
 #include "Admin.h"
 
 #include "ServerRouting.h"
@@ -27,7 +29,6 @@
 
 #include "common/id.h"
 #include "common/log.h"
-#include "common/const.h"
 #include "common/debug.h"
 #include "common/serialno.h"
 #include "common/inheritance.h"
@@ -53,6 +54,7 @@ using Atlas::Objects::Entity::RootEntity;
 
 static const bool debug_flag = false;
 
+/// \brief Admin constructor
 Admin::Admin(Connection * conn, const std::string& username,
              const std::string& passwd, const std::string & id, long intId) :
              Account(conn, username, passwd, id, intId)
@@ -79,7 +81,9 @@ static void addTypeToList(const Root & type, ListType & typeList)
         return;
     }
     if (!children.isList()) {
-        log(ERROR, String::compose("Type %1 children attribute has type %2 instead of string.", type->getId(), Element::typeName(children.getType())).c_str());
+        log(ERROR, String::compose("Type %1 children attribute has type "
+                                   "%2 instead of string.", type->getId(),
+                                   Element::typeName(children.getType())));
         return;
     }
     ListType::const_iterator I = children.List().begin();
@@ -87,7 +91,8 @@ static void addTypeToList(const Root & type, ListType & typeList)
     for (; I != Iend; ++I) {
         Root child = Inheritance::instance().getClass(I->asString());
         if (!child.isValid()) {
-            log(ERROR, String::compose("Unable to find %1 in inheritance table", I->asString()).c_str());
+            log(ERROR, String::compose("Unable to find %1 in inheritance table",
+                                       I->asString()));
             continue;
         }
         addTypeToList(child, typeList);
@@ -115,6 +120,11 @@ void Admin::addToEntity(const Atlas::Objects::Entity::RootEntity & ent) const
     ent->setAttr("character_types", typeList);
 }
 
+/// \brief Function to monitor server operations
+///
+/// This function is connected to the WorldRouter operation dispatch
+/// signal when monitoring is enabled, and relays all in-game operations
+/// to the client.
 void Admin::opDispatched(Operation op)
 {
     if (m_connection != 0) {
@@ -196,7 +206,7 @@ void Admin::GetOperation(const Operation & op, OpVector & res)
         return;
     }
     Info info;
-    if ((objtype == "object") || (objtype == "obj")) {
+    if (objtype == "object" || objtype == "obj") {
         long intId = integerId(id);
 
         const BaseDict & OOGDict = m_connection->m_server.getObjects();
@@ -213,20 +223,26 @@ void Admin::GetOperation(const Operation & op, OpVector & res)
             K->second->addToEntity(info_arg);
             info->setArgs1(info_arg);
         } else {
-            error(op, String::compose("Unknown object id \"%1\" requested", id).c_str(), res, getId());
+            error(op, String::compose("Unknown object id \"%1\" requested", id),
+                  res, getId());
             return;
         }
-    } else if ((objtype == "class") ||
-               (objtype == "meta") ||
-               (objtype == "op_definition")) {
+    } else if (objtype == "class" ||
+               objtype == "meta" ||
+               objtype == "op_definition") {
         const Root & o = Inheritance::instance().getClass(id);
         if (!o.isValid()) {
-            error(op, String::compose("Unknown type definition for \"%1\" requested", id).c_str(), res);
+            error(op, String::compose("Unknown type definition for \"%1\" "
+                                      "requested", id),
+                  res);
             return;
         }
         info->setArgs1(o);
     } else {
-        error(op, String::compose("Unknown object type \"%1\" requested for \"%2\"", objtype, id).c_str(), res, getId());
+        error(op,
+              String::compose("Unknown object type \"%1\" requested for \"%2\"",
+                              objtype, id),
+              res, getId());
         return;
     }
     res.push_back(info);
@@ -250,9 +266,8 @@ void Admin::SetOperation(const Operation & op, OpVector & res)
         return;
     }
     const std::string & id = arg->getId();
-    // FIXME Use this id to install a type from the client
 
-    if ((objtype == "object") || (objtype == "obj")) {
+    if (objtype == "object" || objtype == "obj") {
 
         long intId = integerId(id);
 
@@ -274,7 +289,8 @@ void Admin::SetOperation(const Operation & op, OpVector & res)
             }
             return;
         }
-        error(op, "Client attempting to use obsolete Set to install new type", res, getId());
+        error(op, "Client attempting to use obsolete Set to install new type",
+              res, getId());
         return;
     } else {
         error(op, "Unknow object type set", res, getId());
@@ -311,22 +327,23 @@ void Admin::CreateOperation(const Operation & op, OpVector & res)
         const std::string & id = arg->getId();
 
         if (parent.empty()) {
-            error(op, "Attempt to install type with empty parent", res, getId());
+            error(op, "Attempt to install type with empty parent", res,
+                  getId());
             return;
         }
         if (Inheritance::instance().hasClass(id)) {
-            error(op, "Attempt to install type that already exists", res, getId());
+            error(op, "Attempt to install type that already exists", res,
+                  getId());
             return;
         }
         const Root & o = Inheritance::instance().getClass(parent);
         if (!o.isValid()) {
-            std::string msg("Attempt to install type with non-existant parent \"");
-            msg += parent;
-            msg += "\"";
-            error(op, msg.c_str(), res, getId());
+            error(op, String::compose("Attempt to install type with "
+                                      "non-existant parent \"%1\"", parent),
+                  res, getId());
             return;
         }
-        if (EntityFactory::instance()->installRule(id, arg->asMessage()) == 0) {
+        if (EntityFactory::instance()->installRule(id, arg) == 0) {
             Info info;
             info->setTo(getId());
             info->setArgs1(arg);
@@ -349,6 +366,10 @@ void Admin::OtherOperation(const Operation & op, OpVector & res)
     }
 }
 
+/// \brief Process a Connect operation
+///
+/// @param op The operation to be processed.
+/// @param res The result of the operation is returned here.
 void Admin::customConnectOperation(const Operation & op, OpVector & res)
 {
     const std::vector<Root> & args = op->getArgs();
@@ -391,6 +412,10 @@ void Admin::customConnectOperation(const Operation & op, OpVector & res)
     // Fix it up
 }
 
+/// \brief Process a Monitor operation
+///
+/// @param op The operation to be processed.
+/// @param res The result of the operation is returned here.
 void Admin::customMonitorOperation(const Operation & op, OpVector & res)
 {
     if (!op->getArgs().empty()) {
