@@ -15,12 +15,15 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: CommClient.cpp,v 1.60 2007-07-29 03:33:35 alriddoch Exp $
+
 #include "CommClient.h"
 #include "CommServer.h"
 #include "ServerRouting.h"
 
 #include "common/log.h"
 #include "common/debug.h"
+#include "common/compose.hpp"
 
 #include <Atlas/Objects/Operation.h>
 #include <Atlas/Objects/Encoder.h>
@@ -150,8 +153,17 @@ void CommClient::objectArrived(const Atlas::Objects::Root & obj)
 {
     Atlas::Objects::Operation::RootOperation op = Atlas::Objects::smart_dynamic_cast<Atlas::Objects::Operation::RootOperation>(obj);
     if (!op.isValid()) {
-        // FIXME report the parents and objtype
-        log(ERROR, "Non op object received from client");
+        const std::list<std::string> & parents = obj->getParents();
+        if (parents.empty()) {
+            log(ERROR, String::compose("Object of type \"%1\" with no parent "
+                                       "arrived from client",
+                                       obj->getObjtype()));
+        } else {
+            log(ERROR, String::compose("Object of type \"%1\" with parent "
+                                       "\"%2\" arrived from client",
+                                       obj->getObjtype(),
+                                       obj->getParents().front()));
+        }
         return;
     }
     debug(std::cout << "A " << op->getParents().front() << " op from client!" << std::endl << std::flush;);
