@@ -15,6 +15,12 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: CommListener.cpp,v 1.40 2007-07-29 03:33:35 alriddoch Exp $
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "CommListener.h"
 
 #include "CommRemoteClient.h"
@@ -23,11 +29,18 @@
 #include "common/id.h"
 #include "common/log.h"
 #include "common/debug.h"
+#include "common/compose.hpp"
+#include "common/system.h"
+
+#include <skstream/skstream.h>
 
 #include <iostream>
 
-#include <sys/socket.h>
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif // HAVE_ARPA_INET_H
+#include <unistd.h>
+#include <errno.h>
 
 #include <cassert>
 
@@ -53,15 +66,17 @@ bool CommListener::eof()
 {
     return false;
 }
+
 bool CommListener::isOpen() const
 {
-    return true;
+    return m_listener.is_open();
 }
 
 int CommListener::read()
 {
     accept();
-    // FIXME Perhaps we should return the error if accept() doesn't work?
+    // Accept errors are not returned, as the listen socket should not
+    // be removed.
     return 0;
 }
 
@@ -83,11 +98,11 @@ int CommListener::setup(int port)
         int   l_onoff;
         int   l_linger;
     } listenLinger = { 1, 0 };
-    ::setsockopt(socket, SOL_SOCKET, SO_LINGER, &listenLinger,
+    ::setsockopt(socket, SOL_SOCKET, SO_LINGER, (char *)&listenLinger,
                                                 sizeof(listenLinger));
     // Ensure the address can be reused once we are done with it.
     int flag = 1;
-    ::setsockopt(socket, SOL_SOCKET, SO_REUSEADDR, &flag, sizeof(flag));
+    ::setsockopt(socket, SOL_SOCKET, SO_REUSEADDR, (char *)&flag, sizeof(flag));
     return 0;
 }
 
@@ -97,13 +112,16 @@ int CommListener::accept()
     // Low level socket code to accept a new client connection, and create
     // the associated commclient object.
     struct sockaddr_storage sst;
-    unsigned int addr_len = sizeof(sst);
+    SOCKLEN addr_len = sizeof(sst);
 
     debug(std::cout << "Accepting.." << std::endl << std::flush;);
     int asockfd = ::accept(m_listener.getSocket(),
                            (struct sockaddr *)&sst, &addr_len);
 
     if (asockfd < 0) {
+        log(ERROR,
+            String::compose("System error accepting network connection: %1",
+                            strerror(errno)));
         return -1;
     }
     debug(std::cout << "Accepted" << std::endl << std::flush;);
@@ -116,11 +134,15 @@ int CommListener::accept()
     }
     char buf[INET6_ADDRSTRLEN];
     const char * address = 0;
+#ifdef HAVE_INET_NTOP
     if (adr != 0) {
         address = ::inet_ntop(sst.ss_family, adr, buf, INET6_ADDRSTRLEN);
     } else {
         log(WARNING, "Unable to determine address type for connection");
     }
+#elif HAVE_GETNAMEINFO
+    // FIXME Get things using getnameinfo with NI_NUMERICHOST
+#endif // HAVE_INET_NTOP
     if (address == 0) {
         log(WARNING, "Unable to determine remote address for connection");
         logSysError(WARNING);
@@ -135,7 +157,7 @@ int CommListener::create(int asockfd, const char * address)
     std::string connection_id;
     if (newId(connection_id) < 0) {
         log(ERROR, "Unable to accept connection as no ID available");
-        close(asockfd);
+        closesocket(asockfd);
         return -1;
     }
 
