@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: CommServer.cpp,v 1.61 2007-11-20 13:04:33 alriddoch Exp $
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -27,8 +29,11 @@
 
 #include "common/log.h"
 #include "common/debug.h"
-#include "common/BaseWorld.h"
+#include "common/system.h"
 #include "common/compose.hpp"
+#include "common/BaseWorld.h"
+
+#include <skstream/sksocket.h>
 
 #include <iostream>
 
@@ -46,16 +51,17 @@ static const bool debug_flag = false;
 CommServer::CommServer(ServerRouting & svr) : m_congested(false), m_server(svr)
 {
 #ifdef HAVE_EPOLL_CREATE
-    // FIXME 64 is a random figure I pulled out of the air. I don't even know
-    // what this is used for.
+    // 64 seems like a suitable value for initial number of sockets to be
+    // handled, though there is very little documentation on what would be
+    // a good choice here.
     m_epollFd = epoll_create(64);
-    // FIXME This should only fail if there is a shortage of kernel memory,
-    // which should never happen.
     if (m_epollFd < 0) {
-        log(CRITICAL, "Out of memory calling epoll_create()");
+        log(CRITICAL, String::compose("epoll_create: %s", strerror(errno)));
         exit_flag = true;
     }
 #endif // HAVE_EPOLL_CREATE
+    // Initialise the time
+    gettimeofday(&m_timeVal, NULL);
 }
 
 CommServer::~CommServer()
@@ -79,9 +85,7 @@ bool CommServer::idle()
 {
     // Update the time, and get the core server object to process
     // stuff.
-    // FIXME These idle methods are now getting called way too often
-    // if the core server is busy. Cut it back a bit. Probably can avoid
-    // calling them at all if we are busy.
+    time_t old_seconds = m_timeVal.tv_sec;
     gettimeofday(&m_timeVal, NULL);
 
     bool busy = m_server.m_world.idle(m_timeVal.tv_sec, m_timeVal.tv_usec);
@@ -89,7 +93,7 @@ bool CommServer::idle()
     // We only call the idlers if the world has returned that it is not busy,
     // and the last call to select/poll with a sleep time provided did not
     // return any traffic.
-    if (!busy && !m_congested) {
+    if (!busy && !m_congested && old_seconds != m_timeVal.tv_sec) {
         IdleSet::const_iterator I = m_idlers.begin();
         IdleSet::const_iterator Iend = m_idlers.end();
         for (; I != Iend; ++I) {
@@ -120,16 +124,24 @@ void CommServer::poll()
     bool busy = idle();
 
 #ifdef HAVE_EPOLL_CREATE
-    static struct epoll_event events[16];
+    static const int max_events = 16;
 
-    int rval = ::epoll_wait(m_epollFd, events, 16, (busy ? 0 : 100));
+    static struct epoll_event events[max_events];
+
+    int rval = ::epoll_wait(m_epollFd, events, max_events, (busy ? 0 : 100));
 
     if (rval <  0) {
         if (errno != EINTR) {
-            log(ERROR, String::compose("epoll_wait: %1", strerror(errno)).c_str());
+            log(CYLOG_ERROR, String::compose("epoll_wait: %1", strerror(errno)));
         }
-    } else {
-        m_congested = (rval != 0) || m_congested && busy;
+        return;
+    }
+
+    m_congested = (rval != 0) || m_congested && busy;
+
+    if (rval == max_events) {
+        // If we see this alot, we should increase the maximum
+        log(NOTICE, "epoll_wait returned the maximum number of events.");
     }
 
     for (int i = 0; i < rval; ++i) {
@@ -161,7 +173,7 @@ void CommServer::poll()
 #else // HAVE_EPOLL_CREATE
 
     fd_set sock_fds;
-    int highest = 0;
+    SOCKET_TYPE highest = 0;
     struct timeval tv;
 
     tv.tv_sec = 0;
@@ -176,7 +188,7 @@ void CommServer::poll()
            pendingConnections = true;
            continue;
        }
-       int socket_fd = (*I)->getFd();
+       SOCKET_TYPE socket_fd = (*I)->getFd();
        FD_SET(socket_fd, &sock_fds);
        if (socket_fd > highest) {
            highest = socket_fd;
@@ -187,13 +199,13 @@ void CommServer::poll()
 
     if (rval < 0) {
         if (errno != EINTR) {
-            log(ERROR, "Error caused by select() in main loop");
-            logSysError(ERROR);
+            log(CYLOG_ERROR, "Error caused by select() in main loop");
+            logSysError(CYLOG_ERROR);
         }
         return;
     }
 
-    if ((rval == 0) && !pendingConnections) {
+    if (rval == 0 && !pendingConnections) {
         return;
     }
     
@@ -240,8 +252,8 @@ void CommServer::addSocket(CommSocket * cs)
     ee.data.ptr = cs;
     int ret = ::epoll_ctl(m_epollFd, EPOLL_CTL_ADD, cs->getFd(), &ee);
     if (ret != 0) {
-        log(ERROR, "Error calling epoll_ctl to add socket");
-        logSysError(ERROR);
+        log(CYLOG_ERROR, "Error calling epoll_ctl to add socket");
+        logSysError(CYLOG_ERROR);
     }
 #endif // HAVE_EPOLL_CREATE
     m_sockets.insert(cs);
@@ -258,8 +270,8 @@ void CommServer::removeSocket(CommSocket * cs)
     // FIXME This may not be necessary
     int ret = ::epoll_ctl(m_epollFd, EPOLL_CTL_DEL, cs->getFd(), &ee);
     if (ret != 0) {
-        log(ERROR, "Error calling epoll_ctl to remove socket");
-        logSysError(ERROR);
+        log(CYLOG_ERROR, "Error calling epoll_ctl to remove socket");
+        logSysError(CYLOG_ERROR);
     }
 #endif // HAVE_EPOLL_CREATE
     m_sockets.erase(cs);
