@@ -15,6 +15,12 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: CommUnixListener.cpp,v 1.28 2007-05-29 00:39:18 alriddoch Exp $
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "CommUnixListener.h"
 
 #include "CommLocalClient.h"
@@ -23,12 +29,17 @@
 #include "common/id.h"
 #include "common/log.h"
 #include "common/debug.h"
+#include "common/system.h"
 #include "common/globals.h"
 
 #include <iostream>
 
+#include <unistd.h>
+
 #include <cassert>
 
+#ifdef HAVE_SYS_UN_H
+
 static const bool debug_flag = false;
 
 /// \brief Constructor unix listen socket object.
@@ -42,7 +53,7 @@ CommUnixListener::CommUnixListener(CommServer & svr) : CommSocket(svr),
 CommUnixListener::~CommUnixListener()
 {
     if (m_bound) {
-        unlink(m_path.c_str());
+        ::unlink(m_path.c_str());
     }
 }
 
@@ -56,9 +67,10 @@ bool CommUnixListener::eof()
 {
     return false;
 }
+
 bool CommUnixListener::isOpen() const
 {
-    return true;
+    return m_unixListener.is_open();
 }
 
 int CommUnixListener::read()
@@ -71,8 +83,6 @@ void CommUnixListener::dispatch()
 {
 }
 
-#define UNIX_PATH_MAX 108
-
 /// \brief Create and bind the listen socket.
 ///
 /// If an initial attempt fails, then this function attempts to unlink the
@@ -115,7 +125,7 @@ int CommUnixListener::create(int asockfd)
     std::string connection_id;
     if (newId(connection_id) < 0) {
         log(ERROR, "Unable to accept connection as no ID available");
-        close(asockfd);
+        closesocket(asockfd);
         return -1;
     }
 
@@ -130,3 +140,5 @@ int CommUnixListener::create(int asockfd)
 
     return 0;
 }
+
+#endif // HAVE_SYS_UN_H
