@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Connection.cpp,v 1.168 2007-11-28 20:22:43 alriddoch Exp $
+
 #include "Connection.h"
 
 #include "ServerRouting.h"
@@ -24,13 +26,14 @@
 #include "Player.h"
 #include "ExternalMind.h"
 #include "Persistance.h"
+#include "ExternalProperty.h"
 
 #include "rulesets/Character.h"
 
 #include "common/id.h"
 #include "common/log.h"
-#include "common/const.h"
 #include "common/debug.h"
+#include "common/Update.h"
 #include "common/globals.h"
 #include "common/serialno.h"
 #include "common/inheritance.h"
@@ -48,6 +51,8 @@
 using Atlas::Message::Element;
 using Atlas::Objects::Root;
 using Atlas::Objects::Operation::Info;
+using Atlas::Objects::Operation::Move;
+using Atlas::Objects::Operation::Update;
 using Atlas::Objects::Entity::Anonymous;
 
 static const bool debug_flag = false;
@@ -61,7 +66,7 @@ Connection::Connection(CommClient & client,
                                               m_server(svr)
 {
     m_server.incClients();
-    logEvent(CONNECT, String::compose("%1 - - Connect from %2", id, addr).c_str());
+    logEvent(CONNECT, String::compose("%1 - - Connect from %2", id, addr));
 }
 
 Connection::~Connection()
@@ -73,25 +78,11 @@ Connection::~Connection()
 
     debug(std::cout << "destroy called" << std::endl << std::flush;);
     
-    logEvent(DISCONNECT, String::compose("%1 - - Disconnect", getId()).c_str());
+    logEvent(DISCONNECT, String::compose("%1 - - Disconnect", getId()));
 
     BaseDict::const_iterator Iend = m_objects.end();
     for (BaseDict::const_iterator I = m_objects.begin(); I != Iend; ++I) {
-        Account * ac = dynamic_cast<Account *>(I->second);
-        if (ac != NULL) {
-            m_server.m_lobby.delAccount(ac);
-            ac->m_connection = NULL;
-            logEvent(LOGOUT, String::compose("%1 %2 - Disconnect account %3", getId(), ac->getId(), ac->m_username).c_str());
-            continue;
-        }
-        Character * character = dynamic_cast<Character *>(I->second);
-        if (character != NULL) {
-            if (character->m_externalMind != NULL) {
-                delete character->m_externalMind;
-                character->m_externalMind = NULL;
-                logEvent(DROP_CHAR, String::compose("%1 - %2 Disconnect character %3(%4)", getId(), character->getId(), character->getName(), character->getType()).c_str());
-            }
-        }
+        removePlayer(I->second, "Disconnect");
     }
 
     m_server.decClients();
@@ -124,10 +115,57 @@ Account * Connection::addPlayer(const std::string& username,
     return player;
 }
 
+/// \brief Remove an object from this connection.
+///
+/// The object being removed may be a player, or another type of object such
+/// as an avatar. If it is an player or other account, a pointer is returned.
+Account * Connection::removePlayer(BaseEntity * obj, const std::string & event)
+{
+    Account * ac = dynamic_cast<Account *>(obj);
+    if (ac != 0) {
+        m_server.m_lobby.delAccount(ac);
+        ac->m_connection = 0;
+        logEvent(LOGOUT, String::compose("%1 %2 - %4 account %3", getId(),
+                                         ac->getId(), ac->m_username, event));
+        return ac;
+    }
+    Character * character = dynamic_cast<Character *>(obj);
+    if (character != 0) {
+        if (character->m_externalMind != 0) {
+            // Send a move op stopping the current movement
+            Anonymous move_arg;
+            move_arg->setId(character->getId());
+            // Include the EXTERNAL property which is changing to zero.
+            // It would be more correct at this point to send a separate
+            // update to have the property update itself, but this
+            // will be much less of an issue once Sight(Set) is created
+            // more correctly
+            move_arg->setAttr("external", 0);
+            ::addToEntity(Vector3D(0,0,0), move_arg->modifyVelocity());
+
+            Move move;
+            move->setFrom(character->getId());
+            move->setArgs1(move_arg);
+            character->externalOperation(move);
+
+            delete character->m_externalMind;
+            character->m_externalMind = 0;
+            logEvent(DROP_CHAR, String::compose("%1 - %2 %4 character (%3)",
+                                                getId(),
+                                                character->getId(),
+                                                character->getType(),
+                                                event));
+        }
+    }
+    return 0;
+}
+
 void Connection::addObject(BaseEntity * obj)
 {
     m_objects[obj->getIntId()] = obj;
-    obj->destroyed.connect(sigc::bind(sigc::mem_fun(this, &Connection::objectDeleted), obj->getIntId()));
+    obj->destroyed.connect(sigc::bind(sigc::mem_fun(this,
+                                                    &Connection::objectDeleted),
+                                      obj->getIntId()));
 }
 
 void Connection::removeObject(long id)
@@ -147,6 +185,27 @@ void Connection::disconnect()
     m_commClient.disconnect();
 }
 
+void Connection::connectAvatar(Character * chr)
+{
+    chr->m_externalMind = new ExternalMind(*this, chr->getId(),
+                                                  chr->getIntId());
+
+    if (chr->getProperty("external") == 0) {
+        ExternalProperty * ep = new ExternalProperty(chr->m_externalMind);
+        chr->setProperty("external", ep);
+    }
+
+    Anonymous update_arg;
+    update_arg->setId(chr->getId());
+    update_arg->setAttr("external", 1);
+
+    Update update;
+    update->setTo(chr->getId());
+    update->setArgs1(update_arg);
+
+    chr->sendWorld(update);
+}
+
 int Connection::verifyCredentials(const Account & account,
                                   const Root & creds) const
 {
@@ -171,7 +230,10 @@ void Connection::operation(const Operation & op, OpVector & res)
     debug(std::cout << "send on to " << from << std::endl << std::flush;);
     BaseDict::const_iterator I = m_objects.find(integerId(from));
     if (I == m_objects.end()) {
-        error(op, String::compose("Client \"%1\" op from \"%2\" is from non-existant object.", op->getParents().front(), from).c_str(), res);
+        error(op, String::compose("Client \"%1\" op from \"%2\" is from "
+                                  "non-existant object.",
+                                  op->getParents().front(), from),
+              res);
         return;
     }
     BaseEntity * b_ent = I->second;
@@ -181,11 +243,10 @@ void Connection::operation(const Operation & op, OpVector & res)
         return;
     }
     Character * character = dynamic_cast<Character *>(b_ent);
-    if ((character != NULL) && (character->m_externalMind == NULL)) {
+    if (character != NULL && character->m_externalMind == NULL) {
         debug(std::cout << "Subscribing existing character" << std::endl
                         << std::flush;);
-        character->m_externalMind = new ExternalMind(*this, character->getId(),
-                                                     character->getIntId());
+        connectAvatar(character);
         Info info;
         Anonymous info_arg;
         character->addToEntity(info_arg);
@@ -193,7 +254,9 @@ void Connection::operation(const Operation & op, OpVector & res)
 
         res.push_back(info);
 
-        logEvent(TAKE_CHAR, String::compose("%1 - %2 Taken character %3(%4)", getId(), ig_ent->getId(), ig_ent->getName(), ig_ent->getType()).c_str());
+        logEvent(TAKE_CHAR, String::compose("%1 - %2 Taken character (%3)",
+                                            getId(), ig_ent->getId(),
+                                            ig_ent->getType()));
     }
     ig_ent->externalOperation(op);
 }
@@ -231,7 +294,7 @@ void Connection::LoginOperation(const Operation & op, OpVector & res)
     // account, either from existing account ....
     Account * player = m_server.getAccountByName(username);
     // or if not, from the database
-    if (consts::enable_database && player == 0) {
+    if (database_flag && player == 0) {
         debug(std::cout << "No account called " << username
                         << " in server. Checking in database."
                         << std::endl << std::flush;);
@@ -269,7 +332,8 @@ void Connection::LoginOperation(const Operation & op, OpVector & res)
     debug(std::cout << "Good login" << std::endl << std::flush;);
     res.push_back(info);
 
-    logEvent(LOGIN, String::compose("%1 %2 - Login account %3", getId(), player->getId(), username).c_str());
+    logEvent(LOGIN, String::compose("%1 %2 - Login account %3",
+                                    getId(), player->getId(), username));
 }
 
 void Connection::CreateOperation(const Operation & op, OpVector & res)
@@ -311,9 +375,9 @@ void Connection::CreateOperation(const Operation & op, OpVector & res)
     }
     const std::string & password = passwd_attr.String();
 
-    if ((0 != m_server.getAccountByName(username)) ||
-        (consts::enable_database && Persistance::instance()->findAccount(username)) ||
-        (username.empty()) || (password.empty())) {
+    if (username.empty() || password.empty() ||
+        (0 != m_server.getAccountByName(username)) ||
+        (database_flag && Persistance::instance()->findAccount(username))) {
         // Account exists, or creation data is duff
         clientError(op, "Account creation is invalid", res);
         return;
@@ -323,7 +387,7 @@ void Connection::CreateOperation(const Operation & op, OpVector & res)
         clientError(op, "Account creation failed", res);
         return;
     }
-    if (consts::enable_database) {
+    if (database_flag) {
         Persistance::instance()->putAccount(*player);
     }
     Info info;
@@ -333,7 +397,8 @@ void Connection::CreateOperation(const Operation & op, OpVector & res)
     debug(std::cout << "Good create" << std::endl << std::flush;);
     res.push_back(info);
 
-    logEvent(LOGIN, String::compose("%1 %2 - Create account %3", getId(), player->getId(), username).c_str());
+    logEvent(LOGIN, String::compose("%1 %2 - Create account %3", getId(),
+                                    player->getId(), username));
 }
 
 void Connection::LogoutOperation(const Operation & op, OpVector & res)
@@ -346,7 +411,6 @@ void Connection::LogoutOperation(const Operation & op, OpVector & res)
         if (!op->isDefaultSerialno()) {
             info->setRefno(op->getSerialno());
         }
-        // FIXME Direct call of send. Need local refno handling.
         send(info);
         disconnect();
         return;
@@ -364,27 +428,47 @@ void Connection::LogoutOperation(const Operation & op, OpVector & res)
     }
     BaseDict::iterator I = m_objects.find(obj_id);
     if (I == m_objects.end()) {
-        error(op, "Got logout for unknown entity ID", res);
+        error(op, String::compose("Got logout for unknown entity ID(%1)",
+                                  obj_id),
+              res);
         return;
     }
-    // This is very similar code to stuff that does the same job in
-    // Connection destructor. Should be possible to share some, but
-    // the log messages should probably be different.
-    Account * ac = dynamic_cast<Account *>(I->second);
-    Character * character = dynamic_cast<Character *>(I->second);
+    Account * ac = removePlayer(I->second, "Logout");
     if (ac != 0) {
-        m_server.m_lobby.delAccount(ac);
-        ac->m_connection = 0;
         m_objects.erase(I);
-        // FIXME Remove all characters associated with this account.
-        logEvent(LOGOUT, String::compose("%1 %2 - Logout account %3", getId(), ac->getId(), ac->m_username).c_str());
-    }
-    if (character != 0) {
-        assert(character->m_externalMind != 0);
-        delete character->m_externalMind;
-        character->m_externalMind = 0;
-        logEvent(DROP_CHAR, String::compose("%1 - %2 Logout character %3(%4)", getId(), character->getId(), character->getName(), character->getType()).c_str());
-
+        EntityDict::const_iterator J = ac->getCharacters().begin();
+        EntityDict::const_iterator Jend = ac->getCharacters().end();
+        for (; J != Jend; ++J) {
+            Entity * chr = J->second;
+            Character * character = dynamic_cast<Character *>(chr);
+            if (character != 0) {
+                if (character->m_externalMind != 0) {
+                    ExternalMind * em = dynamic_cast<ExternalMind *>(character->m_externalMind);
+                    if (em == 0) {
+                        log(ERROR, String::compose("Character %1(%2) has "
+                                                   "external mind object which "
+                                                   "is not an ExternalMind",
+                                                   chr->getType(),
+                                                   chr->getId()));
+                    } else {
+                        if (&em->m_connection != this) {
+                            log(ERROR,
+                                String::compose("Connection(%1) has found a "
+                                                "character in its dictionery "
+                                                "which is connected to another "
+                                                "Connection(%2)", getId(),
+                                                em->m_connection.getId()));
+                            removeObject(J->second->getIntId());
+                        }
+                    }
+                } else {
+                    removeObject(J->second->getIntId());
+                }
+            } else {
+                // Non character entity
+                removeObject(J->second->getIntId());
+            }
+        }
     }
 
     Info info;
@@ -415,7 +499,9 @@ void Connection::GetOperation(const Operation & op, OpVector & res)
         debug(std::cout << "Get got for " << id << std::endl << std::flush;);
         Atlas::Objects::Root o = Inheritance::instance().getClass(id);
         if (!o.isValid()) {
-            error(op, String::compose("Unknown type definition for \"%1\" requested", id).c_str(), res);
+            error(op, String::compose("Unknown type definition for \"%1\" "
+                                      "requested", id),
+                  res);
             return;
         }
         info->setArgs1(o);
