@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Connection.h,v 1.57 2007-12-07 01:19:16 alriddoch Exp $
+
 #ifndef SERVER_CONNECTION_H
 #define SERVER_CONNECTION_H
 
@@ -22,9 +24,10 @@
 
 #include <sigc++/connection.h>
 
-class ServerRouting;
-class CommClient;
 class Account;
+class Character;
+class CommClient;
+class ServerRouting;
 
 /// \brief Class representing connections from a client at the Atlas level.
 ///
@@ -47,6 +50,7 @@ class Connection : public OOGThing {
     bool m_obsolete;
 
     Account * addPlayer(const std::string &, const std::string &);
+    Account * removePlayer(BaseEntity *, const std::string & event);
   protected:
     virtual int verifyCredentials(const Account &,
                                   const Atlas::Objects::Root &) const;
@@ -62,6 +66,8 @@ class Connection : public OOGThing {
     void removeObject(long id);
     void objectDeleted(long id);
 
+    void connectAvatar(Character * chr);
+
     void disconnect();
     void send(const Operation & op) const;
 
