@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: PersistantThingFactory.h,v 1.26 2007-12-07 01:19:16 alriddoch Exp $
+
 #ifndef SERVER_THING_FACTORY_H
 #define SERVER_THING_FACTORY_H
 
@@ -23,12 +25,12 @@
 #include <set>
 
 class Entity;
+class ScriptFactory;
+class TypeNode;
 
 template <class T>
 class Persistor;
 
-class ScriptFactory;
-
 /// \brief Interface class for connecting a newly created entity to its
 /// persistor
 class PersistorBase {
@@ -73,22 +75,44 @@ class FactoryBase {
     /// Set of factories for classes which inherit from the class handled
     /// by this factory.
     std::set<FactoryBase *> m_children;
+    /// Inheritance type of this class.
+    TypeNode * m_type;
 
     virtual ~FactoryBase();
 
-    virtual Entity * newThing(const std::string & id, long intId) = 0;
-    virtual Entity * newPersistantThing(const std::string & id, long intId, PersistorBase **) = 0;
+    /// \brief Create a new Entity and make it persistent.
+    ///
+    /// @param id a string giving the identifier of the Entity.
+    /// @param intId an integer giving the identifier of the Entity.
+    /// @param pb a pointer to the persistor object for the Entity.
+    virtual Entity * newPersistantThing(const std::string & id, long intId, PersistorBase ** pb) = 0;
+    /// \brief Add anything required to the entity after it has been created.
     virtual int populate(Entity &) = 0;
+    /// \brief Create a copy of this factory.
     virtual FactoryBase * duplicateFactory() = 0;
 };
 
+template <class T>
+class ThingFactory : public FactoryBase {
+  protected:
+    ThingFactory(ThingFactory<T> & o);
+  public:
+    ThingFactory();
+    virtual ~ThingFactory();
+
+    virtual T * newPersistantThing(const std::string & id, long intId,
+                                   PersistorBase ** p);
+    virtual int populate(Entity &);
+    virtual FactoryBase * duplicateFactory();
+};
+
 // How do we make sure the peristance hooks are put in place in a typesafe way
 // but after all the initial attribute have been set.
 
 /// \brief Class template for factories for creating instances of the give
 /// entity class
 template <class T>
-class PersistantThingFactory : public FactoryBase {
+class PersistantThingFactory : public ThingFactory<T> {
   protected:
     PersistantThingFactory(PersistantThingFactory<T> & p) : m_p(p.m_p),
                                                             m_master(false) { }
@@ -99,12 +123,12 @@ class PersistantThingFactory : public FactoryBase {
     PersistantThingFactory() : m_p(* new Persistor<T>()), m_master(true) { }
     virtual ~PersistantThingFactory();
  
-    virtual T * newThing(const std::string & id, long intId);
     virtual T * newPersistantThing(const std::string & id, long intId, PersistorBase ** p);
-    virtual int populate(Entity &);
     virtual FactoryBase * duplicateFactory();
 };
 
+/// \brief Class template for factories for entity classes which cannot or
+/// should not be instanced
 template <class T>
 class ForbiddenThingFactory : public FactoryBase {
   public:
@@ -113,7 +137,6 @@ class ForbiddenThingFactory : public FactoryBase {
 
     virtual ~ForbiddenThingFactory();
  
-    virtual T * newThing(const std::string & id, long intId);
     virtual T * newPersistantThing(const std::string & id, long intId, PersistorBase ** p);
     virtual int populate(Entity &);
     virtual FactoryBase * duplicateFactory();
