@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: PersistantThingFactory_impl.h,v 1.12 2007-07-08 23:45:12 alriddoch Exp $
+
 #ifndef SERVER_PERSISTANT_THING_FACTORY_IMPL_H
 #define SERVER_PERSISTANT_THING_FACTORY_IMPL_H
 
@@ -26,6 +28,41 @@ void PersistorConnection<T>::persist()
     m_p.persist(m_t);
 }
 
+template <class T>
+ThingFactory<T>::ThingFactory(ThingFactory<T> & o)
+{
+}
+
+template <class T>
+ThingFactory<T>::ThingFactory()
+{
+}
+
+template <class T>
+ThingFactory<T>::~ThingFactory()
+{
+}
+
+template <class T>
+T * ThingFactory<T>::newPersistantThing(const std::string & id, long intId, PersistorBase ** p)
+{
+    return new T(id, intId);
+}
+
+template <class T>
+int ThingFactory<T>::populate(Entity &)
+{
+    return 0;
+}
+
+template <class T>
+FactoryBase * ThingFactory<T>::duplicateFactory()
+{
+    FactoryBase * f = new ThingFactory<T>(*this);
+    f->m_parent = this;
+    return f;
+}
+
 template <class T>
 PersistantThingFactory<T>::~PersistantThingFactory()
 {
@@ -34,12 +71,6 @@ PersistantThingFactory<T>::~PersistantThingFactory()
     }
 }
 
-template <class T>
-T * PersistantThingFactory<T>::newThing(const std::string & id, long intId)
-{
-    return new T(id, intId);
-}
-
 template <class T>
 T * PersistantThingFactory<T>::newPersistantThing(const std::string & id, long intId, PersistorBase ** p)
 {
@@ -48,12 +79,6 @@ T * PersistantThingFactory<T>::newPersistantThing(const std::string & id, long i
     return t;
 }
 
-template <class T>
-int PersistantThingFactory<T>::populate(Entity &)
-{
-    return 0;
-}
-
 template <class T>
 FactoryBase * PersistantThingFactory<T>::duplicateFactory()
 {
@@ -68,12 +93,6 @@ ForbiddenThingFactory<T>::~ForbiddenThingFactory()
     delete &m_p;
 }
 
-template <class T>
-T * ForbiddenThingFactory<T>::newThing(const std::string &, long)
-{
-    return 0;
-}
-
 template <class T>
 T * ForbiddenThingFactory<T>::newPersistantThing(const std::string &, long, PersistorBase **)
 {
