@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Player.cpp,v 1.49 2007-09-22 15:34:05 alriddoch Exp $
+
 #include "Player.h"
 
 #include <Atlas/Objects/SmartPtr.h>
@@ -29,6 +31,13 @@ using Atlas::Message::MapType;
 using Atlas::Message::ListType;
 using Atlas::Objects::Entity::RootEntity;
 
+/// \brief Constructor
+///
+/// @param conn the Connection this account is being created by.
+/// @param username a string giving the username.
+/// @param passwd a string giving the password.
+/// @param id a string giving the identifier of the account.
+/// @param intId an integer giving the identifier of the account.
 Player::Player(Connection * conn, const std::string& username,
                const std::string& passwd, const std::string & id, long intId) :
                Account(conn, username, passwd, id, intId)
@@ -88,7 +97,10 @@ int Player::characterError(const Operation & op,
     // Parents must have been checked already before calling this method
     const std::string & type = ent->getParents().front(); 
     if (Player::playableTypes.find(type) == Player::playableTypes.end()) {
-        error(op, String::compose("You cannot create a character of type \"%1\".", type).c_str(), res, getId());
+        error(op,
+              String::compose("You cannot create a character of type \"%1\".",
+                              type),
+              res, getId());
         return true;
     }
     return false;
