@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Restoration.cpp,v 1.32 2007-07-29 03:33:35 alriddoch Exp $
+
 #include "Restoration.h"
 #include "Restorer.h"
 #include "Persistor.h"
@@ -22,9 +24,6 @@
 
 #include "rulesets/Creator.h"
 #include "rulesets/Plant.h"
-#include "rulesets/Food.h"
-#include "rulesets/Area.h"
-#include "rulesets/Line.h"
 #include "rulesets/Structure.h"
 #include "rulesets/Stackable.h"
 #include "rulesets/World.h"
@@ -48,9 +47,6 @@ Restoration::Restoration(ServerRouting & svr) : server(svr),
     m_restorers["character"] = &Restorer<Character>::restore;
     m_restorers["creator"] = &Restorer<Creator>::restore;
     m_restorers["plant"] = &Restorer<Plant>::restore;
-    m_restorers["food"] = &Restorer<Food>::restore;
-    m_restorers["line"] = &Restorer<Line>::restore;
-    m_restorers["area"] = &Restorer<Area>::restore;
     m_restorers["structure"] = &Restorer<Structure>::restore;
     m_restorers["stackable"] = &Restorer<Stackable>::restore;
 }
@@ -124,7 +120,8 @@ void Restoration::restoreChildren(Entity * loc)
             long intId = integerId(id);
 
             if (intId == -1) {
-                log(ERROR, String::compose("Invalid ID \"%1\" from database while restoring.", id).c_str());
+                log(ERROR, String::compose("Invalid ID \"%1\" from database "
+                                           "while restoring.", id));
                 continue;
             }
 
