@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Restorer_impl.h,v 1.29 2007-12-03 23:18:52 alriddoch Exp $
+
 #ifndef SERVER_RESTORER_IMPL_H
 #define SERVER_RESTORER_IMPL_H
 
@@ -33,8 +35,7 @@
 template <class T>
 void Restorer<T>::rEntity(DatabaseResult::const_iterator & dr)
 {
-    dr.readColumn("type", this->m_type);
-    dr.readColumn("name", this->m_name);
+    // dr.readColumn("type", this->m_type);
     dr.readColumn("px", this->m_location.m_pos.x());
     dr.readColumn("py", this->m_location.m_pos.y());
     dr.readColumn("pz", this->m_location.m_pos.z());
@@ -58,8 +59,6 @@ void Restorer<T>::rEntity(DatabaseResult::const_iterator & dr)
         hc.setValid();
         this->m_location.modifyBBox();
     }
-    dr.readColumn("status", this->m_status);
-    dr.readColumn("mass", this->m_mass);
     dr.readColumn("seq", this->m_seq);
     dr.readColumn("attributes", this->m_attributes);
 }
