@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: WorldRouter.cpp,v 1.221 2007-12-02 23:49:07 alriddoch Exp $
+
 #include "WorldRouter.h"
 
 #include "EntityFactory.h"
@@ -27,23 +29,29 @@
 #include "common/const.h"
 #include "common/globals.h"
 #include "common/random.h"
+#include "common/system.h"
 #include "common/serialno.h"
 #include "common/compose.hpp"
+#include "common/inheritance.h"
 
 #include "common/Setup.h"
 
 #include <Atlas/Objects/Operation.h>
-#include <Atlas/Objects/RootEntity.h>
+#include <Atlas/Objects/Anonymous.h>
 
 #include <sstream>
 #include <algorithm>
 
 using Atlas::Message::Element;
 using Atlas::Objects::Operation::Setup;
+using Atlas::Objects::Operation::Appearance;
 using Atlas::Objects::Entity::RootEntity;
+using Atlas::Objects::Entity::Anonymous;
 
 static const bool debug_flag = false;
 
+/// \brief Type to hold an operation and the Entity it is from for efficiency
+/// when broadcasting.
 struct OpQueEntry {
     Operation op;
     Entity & from;
@@ -93,20 +101,19 @@ void WorldRouter::updateTime(int sec, int usec)
 /// The Entity representing the world is implicity constructed.
 /// Currently the world entity is included in the perceptives list,
 /// but I am not clear why. Need to look into why.
-WorldRouter::WorldRouter() : BaseWorld(*new World(consts::rootWorldId, consts::rootWorldIntId))
+WorldRouter::WorldRouter() : BaseWorld(*new World(consts::rootWorldId,
+                                                  consts::rootWorldIntId))
 {
     struct timeval tv;
     gettimeofday(&tv, NULL);
     m_initTime = tv.tv_sec;
     updateTime(tv.tv_sec, tv.tv_usec);
     m_gameWorld.incRef();
-    m_gameWorld.m_world = this;
-    m_gameWorld.setType("world");
+    EntityFactory::init(*this);
+    m_gameWorld.setType(Inheritance::instance().getType("world"));
     m_eobjects[m_gameWorld.getIntId()] = &m_gameWorld;
     m_perceptives.insert(&m_gameWorld);
-    m_objectList.insert(&m_gameWorld);
     //WorldTime tmp_date("612-1-1 08:57:00");
-    EntityFactory::init(*this);
 }
 
 /// \brief Destructor for the world object.
@@ -151,7 +158,7 @@ void WorldRouter::addOperationToQueue(const Operation & op, Entity & ent)
     op->setFutureSeconds(0.);
     OpQueue::iterator I = m_operationQueue.begin();
     OpQueue::iterator Iend = m_operationQueue.end();
-    for (; (I != Iend) && ((*I).op->getSeconds() <= t) ; ++I);
+    for (; I != Iend && (*I).op->getSeconds() <= t; ++I);
     m_operationQueue.insert(I, OpQueEntry(op, ent));
 }
 
@@ -166,7 +173,7 @@ void WorldRouter::addOperationToQueue(const Operation & op, Entity & ent)
 Operation WorldRouter::getOperationFromQueue()
 {
     OpQueue::const_iterator I = m_operationQueue.begin();
-    if ((I == m_operationQueue.end()) || ((*I)->getSeconds() > m_realTime)) {
+    if (I == m_operationQueue.end() || (*I)->getSeconds() > m_realTime) {
         return NULL;
     }
     debug(std::cout << "pulled op off queue" << std::endl << std::flush;);
@@ -175,7 +182,7 @@ Operation WorldRouter::getOperationFromQueue()
     return op;
 }
 
-/// \brief Provide an adjusted heigh for the given entity.
+/// \brief Provide an adjusted height for the given entity.
 ///
 /// If the position has a parent which has an associated geometry
 /// which define its childrens position, e.g terrain or a floor,
@@ -183,7 +190,8 @@ Operation WorldRouter::getOperationFromQueue()
 /// This function recurses through the parents until it finds
 /// A parent which defines the height.
 /// @return the modified Z coord of the position.
-float WorldRouter::constrainHeight(Entity * parent, const Point3D & pos,
+float WorldRouter::constrainHeight(LocatedEntity * parent,
+                                   const Point3D & pos,
                                    const std::string & mode)
 {
     assert(parent != 0);
@@ -229,14 +237,13 @@ float WorldRouter::constrainHeight(Entity * parent, const Point3D & pos,
 /// Verify that the entity has a valid location, setting to
 /// the default spawn area if necessary. Handle inserting the
 /// entity into the loc/contains tree maintained by the Entity
-/// class. Handle omnipresent entities, and sending a Setup op
-/// to the entity.
+/// class. Send a Setup op to the entity.
 Entity * WorldRouter::addEntity(Entity * ent, bool setup)
 {
     debug(std::cout << "WorldRouter::addEntity(Entity *)" << std::endl
                     << std::flush;);
+    assert(ent->getIntId() != 0);
     m_eobjects[ent->getIntId()] = ent;
-    m_objectList.insert(ent);
     assert(ent->m_location.isValid());
 
     if (!ent->m_location.isValid()) {
@@ -260,7 +267,9 @@ Entity * WorldRouter::addEntity(Entity * ent, bool setup)
                 ent->setAttr("mode", "fixed");
             }
         } else {
-            log(ERROR, String::compose("Mode on entity is a %1 in WorldRouter::addEntity", Element::typeName(mode_attr.getType())).c_str());
+            log(ERROR, String::compose("Mode on entity is a %1 in "
+                                       "WorldRouter::addEntity",
+                                       Element::typeName(mode_attr.getType())));
         }
     }
     ent->m_location.m_pos.z() = constrainHeight(ent->m_location.m_loc,
@@ -269,22 +278,25 @@ Entity * WorldRouter::addEntity(Entity * ent, bool setup)
     ent->m_location.m_loc->m_contains.insert(ent);
     ent->m_location.m_loc->incRef();
     if (cont_change) {
-        ent->m_location.m_loc->m_update_flags |= a_cont;
-        ent->m_location.m_loc->updated.emit();
+        Entity * loc = dynamic_cast<Entity *>(ent->m_location.m_loc);
+        assert(loc != 0);
+        loc->m_update_flags |= a_cont;
+        loc->updated.emit();
     }
     debug(std::cout << "Entity loc " << ent->m_location << std::endl
                     << std::flush;);
-    ent->m_world = this;
-    if (consts::enable_omnipresence &&
-        (ent->getAttributes().find("omnipresent") !=
-         ent->getAttributes().end())) {
-        m_omnipresentList.insert(ent);
-    }
     if (setup) {
         Setup s;
         s->setTo(ent->getId());
         s->setFutureSeconds(-0.1);
         message(s, m_gameWorld);
+
+        Anonymous arg;
+        Appearance app;
+        arg->setId(ent->getId());
+        arg->setStamp(ent->getSeq());
+        app->setArgs1(arg);
+        message(app, *ent);
     }
     return ent;
 }
@@ -309,9 +321,9 @@ Entity * WorldRouter::addNewEntity(const std::string & typestr,
 
     Entity * ent = EntityFactory::instance()->newEntity(id, intId, typestr, attrs);
     if (ent == 0) {
-        std::string msg = std::string("Attempt to create an entity of type \"")
-                          + typestr + "\" but type is unknown or forbidden";
-        log(ERROR, msg.c_str());
+        log(ERROR, String::compose("Attempt to create an entity of type \"%1\" "
+                                   "but type is unknown or forbidden",
+                                   typestr));
         return 0;
     }
     return addEntity(ent);
@@ -327,7 +339,8 @@ Task * WorldRouter::newTask(const std::string & name, Character & owner)
 {
     Task * task = EntityFactory::instance()->newTask(name, owner);
     if (task == 0) {
-        log(ERROR, String::compose("Attempt to create a task of type \"%1\" but type is unknown or forbidden", name).c_str());
+        log(ERROR, String::compose("Attempt to create a task of type \"%1\" "
+                                   "but type is unknown or forbidden", name));
     }
     return task;
 }
@@ -338,6 +351,7 @@ Task * WorldRouter::newTask(const std::string & name, Character & owner)
 /// tool and operation class given.
 /// @param tool the type of tool activating the task
 /// @param op the type of operation acitivating the task
+/// @param target the entity that the task uses as its target
 /// @param owner the character who will own the task
 /// @return a pointer to the new task
 Task * WorldRouter::activateTask(const std::string & tool,
@@ -361,11 +375,8 @@ void WorldRouter::delEntity(Entity * ent)
         log(WARNING, "Attempt to delete game world");
         return;
     }
-    if (consts::enable_omnipresence) {
-        m_omnipresentList.erase(ent);
-    }
+    assert(ent->getIntId() != 0);
     m_perceptives.erase(ent);
-    m_objectList.erase(ent);
     m_eobjects.erase(ent->getIntId());
     ent->destroy();
     ent->decRef();
@@ -392,25 +403,19 @@ void WorldRouter::message(const Operation & op, Entity & ent)
 /// This should probably go, as there is essentially no sane reason
 /// for broadcasting a random op to all entities.
 /// @return a reference to the list of entities to be used for braodcast.
-const EntitySet & WorldRouter::broadcastList(const Operation & op) const
+bool WorldRouter::broadcastPerception(const Operation & op) const
 {
-    // FIXME Use numeric types
-    const std::list<std::string> & parents = op->getParents();
-    if (!parents.empty()) {
-        const std::string & parent = parents.front();
-        if ((parent == "sight") || (parent == "sound") ||
-            (parent == "appearance") || (parent == "disappearance")) {
-            return m_perceptives;
-        }
-        std::string msg = std::string("Broadcasting ") + parent + " op from "
-                                                       + op->getFrom();
-        log(WARNING, msg.c_str());
-    } else {
-        std::string msg = std::string("Broadcasting op with no parent from ")
-                                                       + op->getFrom();
-        log(ERROR, msg.c_str());
+    int op_class = op->getClassNo();
+    if (op_class == Atlas::Objects::Operation::SIGHT_NO ||
+        op_class == Atlas::Objects::Operation::SOUND_NO ||
+        op_class == Atlas::Objects::Operation::APPEARANCE_NO ||
+        op_class == Atlas::Objects::Operation::DISAPPEARANCE_NO) {
+        return true;
     }
-    return m_objectList;
+    log(WARNING, String::compose("Broadcasting %1 op from %2",
+                                 op->getParents().front(),
+                                 op->getFrom()));
+    return false;
 }
 
 /// \brief Deliver an operation to its target.
@@ -441,15 +446,22 @@ void WorldRouter::deliverTo(const Operation & op, Entity & ent)
 /// sight ranges for perception operations.
 /// @param op operation to be dispatched to the world. This is non-const
 /// so that broadcast ops can have their TO set correctly for each target.
+/// @param from entity the operation to be dispatched was send from. Note
+/// that it is possible that this entity has been destroyed, but it
+/// should still have a valid location, so can be used for range
+/// calculations.
 void WorldRouter::operation(const Operation & op, Entity & from)
 {
-    const std::string & to = op->getTo();
     debug(std::cout << "WorldRouter::operation {"
                     << op->getParents().front() << ":"
-                    << op->getFrom() << ":" << to << "}" << std::endl
-                    << std::flush;);
+                    << op->getFrom() << ":" << op->getTo() << "}"
+                    << std::endl << std::flush;);
+    assert(op->getFrom() == from.getId());
+    assert(!op->getParents().empty());
 
-    if (!to.empty()) {
+    if (!op->isDefaultTo()) {
+        const std::string & to = op->getTo();
+        assert(!to.empty());
         Entity * to_entity = 0;
 
         if (to == from.getId()) {
@@ -472,47 +484,42 @@ void WorldRouter::operation(const Operation & op, Entity & from)
         assert(to_entity != 0);
 
         deliverTo(op, *to_entity);
-        if (op->getParents().front() == "delete") { // FIXME numeric type
-            assert(op->getClassNo() == Atlas::Objects::Operation::DELETE_NO);
+        if (op->getClassNo() == Atlas::Objects::Operation::DELETE_NO) {
             delEntity(to_entity);
         }
-    } else {
+    } else if (broadcastPerception(op)) {
         // Where broadcasts go depends on type of op
-        const EntitySet & broadcast = broadcastList(op);
-        assert(op->getFrom() == from.getId());
-        if (!consts::enable_ranges) {
-            EntitySet::const_iterator I = broadcast.begin();
-            EntitySet::const_iterator Iend = broadcast.end();
-            for (; I != Iend; ++I) {
-                op->setTo((*I)->getId());
-                deliverTo(op, **I);
-            }
-        } else {
-            float fromSquSize = from.m_location.squareBoxSize();
-            EntitySet::const_iterator I = broadcast.begin();
-            EntitySet::const_iterator Iend = broadcast.end();
-            for (; I != Iend; ++I) {
-                // Calculate square distance to target
-                float dist = squareDistance(from.m_location, (*I)->m_location);
-                float view_factor = fromSquSize / dist;
-                if (view_factor < consts::square_sight_factor) {
-                    debug(std::cout << "Op from " << from.getId()
-                                    << " cannot be seen by " << (*I)->getId()
-                                    << std::endl << std::flush;);
-                    continue;
-                }
-                op->setTo((*I)->getId());
-                deliverTo(op, **I);
+        float fromSquSize = from.m_location.squareBoxSize();
+        EntitySet::const_iterator I = m_perceptives.begin();
+        EntitySet::const_iterator Iend = m_perceptives.end();
+        for (; I != Iend; ++I) {
+            // Calculate square distance to target
+            float dist = squareDistance(from.m_location, (*I)->m_location);
+            float view_factor = fromSquSize / dist;
+            if (view_factor < consts::square_sight_factor) {
+                debug(std::cout << "Op from " << from.getId()
+                                << " cannot be seen by " << (*I)->getId()
+                                << std::endl << std::flush;);
+                continue;
             }
+            op->setTo((*I)->getId());
+            deliverTo(op, **I);
+        }
+    } else {
+        EntityDict::const_iterator I = m_eobjects.begin();
+        EntityDict::const_iterator Iend = m_eobjects.end();
+        for (; I != Iend; ++I) {
+            op->setTo(I->second->getId());
+            deliverTo(op, *I->second);
         }
     }
 }
 
-/// Add entity provided to this list of perceptive entities.
+/// Add entity provided to the list of perceptive entities.
 /// Look up the entity with the id provided, and add a pointer
 /// to the entity to the set of perceptive entities. This method is
 /// called when key events occur that indicate that the entity in
-/// question can receive broadcase perception operations.
+/// question can receive broadcast perception operations.
 void WorldRouter::addPerceptive(Entity * perceptive)
 {
     debug(std::cout << "WorldRouter::addPerceptive" << std::endl << std::flush;);
@@ -537,8 +544,7 @@ bool WorldRouter::idle(int sec, int usec)
     unsigned int op_count = 0;
     OpQueue::iterator I = m_operationQueue.begin();
     OpQueue::iterator Iend = m_operationQueue.end();
-    while ((++op_count < 10) && (I != Iend) &&
-           ((*I)->getSeconds() <= m_realTime)) {
+    while (++op_count < 10 && I != Iend && (*I)->getSeconds() <= m_realTime) {
         assert(I != m_operationQueue.end());
         OpQueEntry & oqe = *I;
         Dispatching.emit(oqe.op);
@@ -546,12 +552,10 @@ bool WorldRouter::idle(int sec, int usec)
             operation(oqe.op, oqe.from);
         }
         catch (...) {
-            std::string msg = std::string("Exception caught in world.idle()")
-                            + " thrown while processing "
-                            // + oqe->getParents().front()
-                            + " operation sent to " + oqe->getTo()
-                            + " from " + oqe->getFrom() + ".";
-            log(ERROR, msg.c_str());
+            log(ERROR, String::compose("Exception caught in world.idle() "
+                                       "thrown while processing operation "
+                                       "sent to \"%1\" from \"%2\"",
+                                       oqe->getTo(), oqe->getFrom()));
         }
         m_operationQueue.erase(I);
         I = m_operationQueue.begin();
@@ -559,7 +563,7 @@ bool WorldRouter::idle(int sec, int usec)
 
     I = m_immediateQueue.begin();
     Iend = m_immediateQueue.end();
-    while ((++op_count < 10) && (I != Iend)) {
+    while (++op_count < 10 && I != Iend) {
         assert(I != m_immediateQueue.end());
         OpQueEntry & oqe = *I;
         Dispatching.emit(oqe.op);
@@ -567,12 +571,10 @@ bool WorldRouter::idle(int sec, int usec)
             operation(oqe.op, oqe.from);
         }
         catch (...) {
-            std::string msg = std::string("Exception caught in world.idle()")
-                            + " thrown while processing "
-                            // + oqe->getParents().front()
-                            + " operation sent to " + oqe->getTo()
-                            + " from " + oqe->getFrom() + ".";
-            log(ERROR, msg.c_str());
+            log(ERROR, String::compose("Exception caught in world.idle() "
+                                       "thrown while processing operation "
+                                       "sent to \"%1\" from \"%2\"",
+                                       oqe->getTo(), oqe->getFrom()));
         }
         m_immediateQueue.erase(I);
         I = m_immediateQueue.begin();
@@ -592,10 +594,13 @@ bool WorldRouter::idle(int sec, int usec)
 /// instance with this name was not found.
 Entity * WorldRouter::findByName(const std::string & name)
 {
+    Element name_attr;
     EntityDict::const_iterator Iend = m_eobjects.end();
     for (EntityDict::const_iterator I = m_eobjects.begin(); I != Iend; ++I) {
-        if (I->second->getName() == name) {
-            return I->second;
+        if (I->second->getAttr("name", name_attr)) {
+            if (name_attr == name) {
+                return I->second;
+            }
         }
     }
     return NULL;
@@ -611,7 +616,7 @@ Entity * WorldRouter::findByType(const std::string & type)
 {
     EntityDict::const_iterator Iend = m_eobjects.end();
     for(EntityDict::const_iterator I = m_eobjects.begin(); I != Iend; ++I) {
-        if (I->second->getType() == type) {
+        if (I->second->getType()->name() == type) {
             return I->second;
         }
     }
