@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: WorldRouter.h,v 1.80 2007-12-02 23:49:07 alriddoch Exp $
+
 #ifndef SERVER_WORLD_ROUTER_H
 #define SERVER_WORLD_ROUTER_H
 
@@ -55,8 +57,7 @@ typedef std::list<OpQueEntry> OpQueue;
 ///
 /// This class has one instance which manages the game world.
 /// It maintains a list of all ih-game (IG) objects in the server.
-/// It explicitly also maintains lists of perceptive and omnipresent
-/// entities.
+/// It explicitly also maintains lists of perceptive entities.
 class WorldRouter : public BaseWorld {
   private:
     /// An ordered queue of operations to be dispatched in the future
@@ -65,16 +66,12 @@ class WorldRouter : public BaseWorld {
     OpQueue m_immediateQueue;
     /// The system time when the server was started.
     time_t m_initTime;
-    /// List of all IG entities.
-    EntitySet m_objectList;
     /// List of perceptive entities.
     EntitySet m_perceptives;
-    /// List of omnipresent entities. Obsolete.
-    EntitySet m_omnipresentList;
 
     void addOperationToQueue(const Operation &, Entity &);
     Operation getOperationFromQueue();
-    const EntitySet & broadcastList(const Operation &) const;
+    bool broadcastPerception(const Operation &) const;
     void updateTime(int sec, int usec);
     void deliverTo(const Operation &, Entity &);
     void delEntity(Entity * obj);
@@ -95,7 +92,9 @@ class WorldRouter : public BaseWorld {
     virtual void message(const Operation &, Entity &);
     virtual Entity * findByName(const std::string & name);
     virtual Entity * findByType(const std::string & type);
-    virtual float constrainHeight(Entity *, const Point3D &, const std::string &);
+    virtual float constrainHeight(LocatedEntity * parent,
+                                  const Point3D & pos,
+                                  const std::string & mode);
 };
 
 #endif // SERVER_WORLD_ROUTER_H
