@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: server.cpp,v 1.157 2007-12-06 23:50:14 alriddoch Exp $
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -35,6 +37,7 @@
 
 #include "rulesets/Python_API.h"
 #include "rulesets/MindFactory.h"
+#include "rulesets/Entity.h"
 
 #include "common/id.h"
 #include "common/log.h"
@@ -61,8 +64,21 @@ int main(int argc, char ** argv)
 
     interactive_signals();
 
-    if (loadConfig(argc, argv, true) < 0) {
-        // Fatal error loading config file
+    int config_status = loadConfig(argc, argv, USAGE_SERVER);
+    if (config_status < 0) {
+        if (config_status == CONFIG_VERSION) {
+            std::cout << argv[0] << " (cyphesis) " << consts::version
+                      << " (Cyphesis build " << consts::buildId << ")"
+                      << std::endl << std::flush;
+
+            return 0;
+        } else if (config_status == CONFIG_HELP) {
+            showUsage(argv[0], USAGE_SERVER);
+            return 0;
+        } else if (config_status != CONFIG_ERROR) {
+            log(ERROR, "Unknown error reading configuration.");
+        }
+        // Fatal error loading config file.
         return EXIT_CONFIG_ERROR;
     }
 
@@ -75,62 +91,66 @@ int main(int argc, char ** argv)
         }
     }
 
+    readConfigItem(instance, "usedatabase", database_flag);
+
     // If we are a daemon logging to syslog, we need to set it up.
     initLogger();
 
     // Initialise the persistance subsystem. If we have been built with
     // database support, this will open the various databases used to
     // store server data.
-    if (consts::enable_database) {
-        int dbstatus = Persistance::init();
+    if (database_flag) {
+        Persistance * p = Persistance::instance();
+        int dbstatus = p->init();
         if (dbstatus < 0) {
-            log(CRITICAL, _("Critical error opening databases. Init failed."));
-            if (dbstatus == -2) {
-                log(INFO, "Database connection established, but unable to create required tables.");
-                log(INFO, "Please ensure that any obsolete database tables have been removed.");
+            database_flag = false;
+            log(ERROR, "Error opening database. Database disabled.");
+            if (dbstatus == DATABASE_TABERR) {
+                log(INFO, "Database connection established, "
+                          "but unable to create required tables.");
+                log(INFO, "Please ensure that any obsolete database "
+                          "tables have been removed.");
             } else {
                 log(INFO, "Unable to connect to the RDBMS.");
-                log(INFO, "Please ensure that the RDBMS is running, the cyphesis database exists and is accessible to the user running cyphesis.");
-            }
-            return EXIT_DATABASE_ERROR;
-        }
-
-        // If the restricted flag is set in the config file, then we
-        // don't allow connecting users to create accounts. Accounts must
-        // be created manually by the server administrator.
-        if (global_conf->findItem("cyphesis", "restricted")) {
-            restricted_flag = global_conf->getItem("cyphesis","restricted");
-            if (restricted_flag) {
-                log(INFO, "Setting restricted mode.");
+                log(INFO, "Please ensure that the RDBMS is running, "
+                          "the cyphesis database exists and is accessible "
+                          "to the user running cyphesis.");
             }
+            log(INFO, String::compose("To disable this message please run:\n\n"
+                                      "    cyconfig --%1:usedatabase=false\n\n"
+                                      "to permanently disable database usage.",
+                                      instance));
         }
     }
 
-    if (global_conf->findItem("cyphesis", "inittime")) {
-        timeoffset = global_conf->getItem("cyphesis","inittime");
+    // If the restricted flag is set in the config file, then we
+    // don't allow connecting users to create accounts. Accounts must
+    // be created manually by the server administrator.
+    if (readConfigItem(instance, "restricted", restricted_flag) == 0) {
+        if (restricted_flag) {
+            log(INFO, "Setting restricted mode.");
+        }
     }
 
+    readConfigItem(instance, "inittime", timeoffset);
+
     bool useMetaserver = false;
-    if (global_conf->findItem("cyphesis", "usemetaserver")) {
-        useMetaserver = global_conf->getItem("cyphesis","usemetaserver");
-    }
+    readConfigItem(instance, "usemetaserver", useMetaserver);
 
     std::string mserver("metaserver.worldforge.org");
-    if (global_conf->findItem("cyphesis", "metaserver")) {
-        mserver = global_conf->getItem("cyphesis", "metaserver").as_string();
-    }
+    readConfigItem(instance, "metaserver", mserver);
 
     std::string server_name;
-    if (global_conf->findItem("cyphesis", "servername")) {
-        server_name = global_conf->getItem("cyphesis","servername").as_string();
-    } else {
-        server_name = get_hostname();
+    if (readConfigItem(instance, "servername", server_name) != 0) {
+        if (instance == "cyphesis") {
+            server_name = get_hostname();
+        } else {
+            server_name = instance;
+        }
     }
 
-    int nice=0;
-    if (global_conf->findItem("cyphesis", "nice")) {
-        nice = global_conf->getItem("cyphesis", "nice");
-    }
+    int nice = 1;
+    readConfigItem(instance, "nice", nice);
     
     // Start up the python subsystem.
     init_python_api();
@@ -157,7 +177,7 @@ int main(int argc, char ** argv)
         return EXIT_DATABASE_ERROR;
     }
 
-    ServerRouting server(world, rulesets.front(), server_name,
+    ServerRouting server(world, ruleset, server_name,
                          server_id, int_id,
                          lobby_id, lobby_int_id);
 
@@ -169,16 +189,26 @@ int main(int argc, char ** argv)
     // not creating a new world using the contents of the database as a
     // template
 
-    if (consts::enable_database) {
+    if (database_flag) {
         log(INFO, _("Restoring world from database..."));
 
         Restoration restore(server);
         if (restore.read() == 1) {
             debug(std::cout << "Bootstrapping world" << std::endl << std::flush;);
             EntityFactory::instance()->initWorld();
+            assert(!world.m_gameWorld.m_location.m_pos.isValid());
+            assert(!world.m_gameWorld.m_location.m_orientation.isValid());
+        } else {
+            // The world should not have POS or ORIENTATION, but will have
+            // picked one up when restored from the database.
+            world.m_gameWorld.m_location.m_pos = Point3D(0,0,0);
+            world.m_gameWorld.m_location.m_pos.setValid(false);
+            world.m_gameWorld.m_location.m_orientation = Quaternion();
+            // world.m_gameWorld.m_location.m_orientation.setValid(false);
+            
         }
 
-        log(INFO, _(" world restored"));
+        log(INFO, _("Restored world."));
 
         CommPSQLSocket * dbsocket = new CommPSQLSocket(commServer,
                                         Persistance::instance()->m_connection);
@@ -194,9 +224,34 @@ int main(int argc, char ** argv)
     }
 
     CommListener * listener = new CommListener(commServer);
-    if (listener->setup(client_port_num) != 0) {
-        log(ERROR, "Could not create client listen socket. Init failed.");
-        return EXIT_SOCKET_ERROR;
+    if (client_port_num < 0) {
+        client_port_num = dynamic_port_start;
+        for (; client_port_num <= dynamic_port_end; client_port_num++) {
+            if (listener->setup(client_port_num) == 0) {
+                break;
+            }
+        }
+        if (client_port_num > dynamic_port_end) {
+            log(ERROR, "Could not find free client listen socket. "
+                       "Init failed.");
+            log(INFO, String::compose("To allocate 8 more ports please run:"
+                                      "\n\n    cyconfig "
+                                      "--cyphesis:dynamic_port_end=%1\n\n",
+                                      dynamic_port_end + 8));
+            return EXIT_PORT_ERROR;
+        }
+        log(INFO, String::compose("Auto configuring new instance \"%1\" "
+                                  "to use port %2.",
+                                  instance, client_port_num));
+        global_conf->setItem(instance, "tcpport", client_port_num,
+                             varconf::USER);
+        global_conf->setItem(CYPHESIS, "dynamic_port_start",
+                             client_port_num + 1, varconf::USER);
+    } else {
+        if (listener->setup(client_port_num) != 0) {
+            log(ERROR, "Could not create client listen socket. Init failed.");
+            return EXIT_SOCKET_ERROR;
+        }
     }
     commServer.addSocket(listener);
 
@@ -208,16 +263,17 @@ int main(int argc, char ** argv)
         commServer.addSocket(peerListener);
     }
 
+#ifdef HAVE_SYS_UN_H
     CommUnixListener * localListener = new CommUnixListener(commServer);
     if (localListener->setup(client_socket_name) != 0) {
-        std::stringstream str;
-        str << "Could not create local listen socket with address \"";
-        str << localListener->getPath() << "\".";
-        log(ERROR, str.str().c_str());
+        log(ERROR, String::compose("Could not create local listen socket "
+                                   "with address \"%1\"",
+                                   localListener->getPath()));
         delete localListener;
     } else {
         commServer.addSocket(localListener);
     }
+#endif
 
     if (useMetaserver) {
         CommMetaClient * cmc = new CommMetaClient(commServer);
@@ -243,6 +299,11 @@ int main(int argc, char ** argv)
 
 #endif // defined(HAVE_LIBHOWL)
 
+    // Configuration is now complete, and verified as somewhat sane, so
+    // we save the updated user config.
+
+    updateUserConfiguration();
+
     log(INFO, "Running");
     logEvent(START, "- - - Standalone server startup");
 
@@ -279,7 +340,7 @@ int main(int argc, char ** argv)
       // cause the destruction of the server and world objects, and the entire
       // world contents
 
-    Persistance::shutdown();
+    Persistance::instance()->shutdown();
 
     EntityFactory::instance()->flushFactories();
     EntityFactory::del();
