@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Charactertest.cpp,v 1.14 2007-12-03 23:29:20 alriddoch Exp $
+
 #include "IGEntityExerciser.h"
 #include "allOperations.h"
 
@@ -33,6 +35,9 @@ int main(int argc, char ** argv)
     init_python_api();
 
     Character e("1", 1);
+    TypeNode type;
+    type.name() = "character";
+    e.setType(&type);
 
     IGEntityExerciser<Character> ee(e);
 
@@ -47,14 +52,9 @@ int main(int argc, char ** argv)
     ee.runOperations();
 
     std::set<std::string> attrNames;
-    attrNames.insert("status");
     attrNames.insert("id");
-    attrNames.insert("name");
-    attrNames.insert("mass");
     attrNames.insert("bbox");
     attrNames.insert("contains");
-    attrNames.insert("drunkness");
-    attrNames.insert("sex");
 
     // Make sure we have all the default attributes
     assert(ee.checkAttributes(attrNames));
