@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Creatortest.cpp,v 1.26 2007-12-03 23:29:20 alriddoch Exp $
+
 #include "IGEntityExerciser.h"
 #include "allOperations.h"
 #include "TestWorld.h"
@@ -34,10 +36,9 @@ int main(int argc, char ** argv)
     init_python_api();
 
     Creator e("1", 1);
-    Entity world("2", 2);
-    TestWorld tw(world);
-
-    e.m_world = &tw;
+    TypeNode type;
+    type.name() = "creator";
+    e.setType(&type);
 
     IGEntityExerciser<Creator> ee(e);
 
@@ -52,14 +53,9 @@ int main(int argc, char ** argv)
     ee.runOperations();
 
     std::set<std::string> attrNames;
-    attrNames.insert("status");
     attrNames.insert("id");
-    attrNames.insert("name");
-    attrNames.insert("mass");
     attrNames.insert("bbox");
     attrNames.insert("contains");
-    attrNames.insert("drunkness");
-    attrNames.insert("sex");
 
     // Make sure we have all the default attributes
     assert(ee.checkAttributes(attrNames));
