@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: EntityExerciser.h,v 1.26 2007-07-04 21:29:11 alriddoch Exp $
+
 #ifndef TESTS_ENTITY_EXERCISER_H
 #define TESTS_ENTITY_EXERCISER_H
 
@@ -22,14 +24,9 @@
 
 #include "common/Add.h"
 #include "common/Burn.h"
-#include "common/Chop.h"
 #include "common/Connect.h"
-#include "common/Cut.h"
-#include "common/Delve.h"
-#include "common/Dig.h"
 #include "common/Eat.h"
 #include "common/Monitor.h"
-#include "common/Mow.h"
 #include "common/Nourish.h"
 #include "common/Setup.h"
 #include "common/Tick.h"
@@ -37,6 +34,7 @@
 #include "common/Update.h"
 
 #include <Atlas/Objects/Operation.h>
+#include <Atlas/Objects/Anonymous.h>
 
 #include <cassert>
 
@@ -57,6 +55,7 @@ class EntityExerciser {
     void addAllOperations(std::set<std::string> & ops);
 
     void runOperations();
+    void runConversions();
     void flushOperations(OpVector & ops);
 };
 
@@ -111,20 +110,6 @@ inline void EntityExerciser<EntityType>::runOperations()
         m_ent.LogoutOperation(op, ov);
         flushOperations(ov);
     }
-    {
-        Atlas::Objects::Operation::Action op;
-        dispatchOp(op);
-        OpVector ov;
-        m_ent.ActionOperation(op, ov);
-        flushOperations(ov);
-    }
-    {
-        Atlas::Objects::Operation::Chop op;
-        dispatchOp(op);
-        OpVector ov;
-        m_ent.ChopOperation(op, ov);
-        flushOperations(ov);
-    }
     {
         Atlas::Objects::Operation::Combine op;
         dispatchOp(op);
@@ -139,13 +124,6 @@ inline void EntityExerciser<EntityType>::runOperations()
         m_ent.CreateOperation(op, ov);
         flushOperations(ov);
     }
-    {
-        Atlas::Objects::Operation::Cut op;
-        dispatchOp(op);
-        OpVector ov;
-        m_ent.CutOperation(op, ov);
-        flushOperations(ov);
-    }
     {
         Atlas::Objects::Operation::Delete op;
         dispatchOp(op);
@@ -168,13 +146,6 @@ inline void EntityExerciser<EntityType>::runOperations()
         m_ent.EatOperation(op, ov);
         flushOperations(ov);
     }
-    {
-        Atlas::Objects::Operation::Burn op;
-        dispatchOp(op);
-        OpVector ov;
-        m_ent.BurnOperation(op, ov);
-        flushOperations(ov);
-    }
     {
         Atlas::Objects::Operation::Get op;
         dispatchOp(op);
@@ -311,6 +282,26 @@ inline void EntityExerciser<EntityType>::runOperations()
     }
 }
 
+template <class EntityType>
+inline void EntityExerciser<EntityType>::runConversions()
+{
+    {
+        Atlas::Message::MapType data;
+
+        assert(data.empty());
+        m_ent.addToMessage(data);
+        assert(!data.empty());
+    }
+
+    {
+        Atlas::Objects::Entity::Anonymous data;
+
+        assert(data->isDefaultId());
+        m_ent.addToEntity(data);
+        assert(!data->isDefaultId());
+    }
+}
+
 template <class EntityType>
 inline void EntityExerciser<EntityType>::flushOperations(OpVector & ops)
 {
