@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Entitytest.cpp,v 1.16 2007-12-03 23:29:20 alriddoch Exp $
+
 #include "IGEntityExerciser.h"
 #include "allOperations.h"
 
@@ -38,14 +40,8 @@ int main()
     std::set<std::string> opNames;
     ee.addAllOperations(opNames);
 
-    // Throw an op of every type at the entity again now it is subscribed
-    ee.runOperations();
-
     std::set<std::string> attrNames;
-    attrNames.insert("status");
     attrNames.insert("id");
-    attrNames.insert("name");
-    attrNames.insert("mass");
     attrNames.insert("bbox");
     attrNames.insert("contains");
 
@@ -96,5 +92,9 @@ int main()
     // Read the contents of the Atlas Message back in
     e.merge(entityAsAtlas);
 
+    // Throw an op of every type at the entity again now it is subscribed,
+    // and full of data.
+    ee.runOperations();
+
     return 0;
 }
