@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Thingtest.cpp,v 1.13 2007-12-03 23:29:20 alriddoch Exp $
+
 #include "IGEntityExerciser.h"
 #include "allOperations.h"
 
@@ -28,6 +30,9 @@ using Atlas::Message::ListType;
 int main()
 {
     Thing e("1", 1);
+    TypeNode type;
+    type.name() = "thing";
+    e.setType(&type);
 
     IGEntityExerciser<Thing> ee(e);
 
@@ -42,10 +47,7 @@ int main()
     ee.runOperations();
 
     std::set<std::string> attrNames;
-    attrNames.insert("status");
     attrNames.insert("id");
-    attrNames.insert("name");
-    attrNames.insert("mass");
     attrNames.insert("bbox");
     attrNames.insert("contains");
 
