@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: Worldtest.cpp,v 1.28 2007-12-03 23:29:20 alriddoch Exp $
+
 #include "IGEntityExerciser.h"
 #include "allOperations.h"
 #include "TestWorld.h"
@@ -28,9 +30,10 @@ using Atlas::Message::ListType;
 
 int main()
 {
-    World e("1", 1);
-    TestWorld tw(e);
-    e.m_world = &tw;
+    World e("0", 0);
+    TypeNode type;
+    type.name() = "world";
+    e.setType(&type);
 
     IGEntityExerciser<World> ee(e);
 
@@ -45,10 +48,7 @@ int main()
     ee.runOperations();
 
     std::set<std::string> attrNames;
-    attrNames.insert("status");
     attrNames.insert("id");
-    attrNames.insert("name");
-    attrNames.insert("mass");
     attrNames.insert("bbox");
     attrNames.insert("contains");
     attrNames.insert("terrain");
