@@ -15,6 +15,8 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+// $Id: consttest.cpp,v 1.8 2006-12-27 11:19:20 alriddoch Exp $
+
 #include "common/const.h"
 
 #include <string>
@@ -48,31 +50,16 @@ int main()
     float square_base_velocity;
     square_base_velocity = consts::square_base_velocity;
     assert(square_base_velocity > 0);
-    float sight_range;
-    sight_range = consts::sight_range;
-    assert(sight_range > 0);
-    float square_sight_range;
-    square_sight_range = consts::square_sight_range;
-    assert(square_sight_range > 0);
     float sight_factor;
     sight_factor = consts::sight_factor;
     assert(sight_factor > 0);
     float square_sight_factor;
     square_sight_factor = consts::square_sight_factor;
     assert(square_sight_factor > 0);
-    float hearing_range;
-    hearing_range = consts::hearing_range;
-    assert(hearing_range > 0);
     float minSqrBoxSize;
     minSqrBoxSize = consts::minSqrBoxSize;
     assert(minSqrBoxSize > 0);
     
-    bool enable_ranges;
-    enable_ranges = consts::enable_ranges;
-    assert(enable_ranges == false || enable_ranges == true);
-    bool enable_omnipresence;
-    enable_omnipresence = consts::enable_omnipresence;
-    assert(enable_omnipresence == false || enable_omnipresence == true);
     bool enable_persistence;
     enable_persistence = consts::enable_persistence;
     assert(enable_persistence == false || enable_persistence == true);
@@ -98,6 +85,9 @@ int main()
     versions = consts::version;
     assert(versions.size() > 0);
 
+#if 0
+// For the time being, as these are now in the server/buildid.cpp
+// we are leaving the out
     const char * buildTime;
     buildTime = consts::buildTime;
     assert(buildTime != 0);
@@ -111,6 +101,7 @@ int main()
     std::string buildDates;
     buildDates = consts::buildDate;
     assert(buildDates.size() > 0);
+#endif
 
     return 0;
 }
