@@ -15,9 +15,7 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+// $Id: AdminClient.cpp,v 1.18 2007-06-12 18:56:21 alriddoch Exp $
 
 #include "AdminClient.h"
 
@@ -53,6 +51,10 @@ using Atlas::Objects::Operation::Error;
 
 static const bool debug_flag = false;
 
+/// \brief Output formatted representation of Atlas message data.
+///
+/// @param item Atlas value to be output
+/// @param recurse flag indicating whether to recurse into container messages
 void AdminClient::output(const Element & item, bool recurse)
 {
     std::cout << " ";
@@ -99,13 +101,26 @@ void AdminClient::output(const Element & item, bool recurse)
     }
 }
 
+/// \brief Function call from the base class when an object arrives from the
+/// server
+///
+/// @param obj Object that has arrived from the server
 void AdminClient::objectArrived(const Root & obj)
 {
     RootOperation op = Atlas::Objects::smart_dynamic_cast<RootOperation>(obj);
     if (!op.isValid()) {
-        // FIXME report the parents and objtype
         std::cerr << "ERROR: Non op object received from server"
                   << std::endl << std::flush;;
+        if (!obj->isDefaultParents() && !obj->getParents().empty()) {
+            std::cerr << "NOTICE: Unexpected object has parent "
+                      << obj->getParents().front()
+                      << std::endl << std::flush;
+        }
+        if (!obj->isDefaultObjtype()) {
+            std::cerr << "NOTICE: Unexpected object has objtype "
+                      << obj->getObjtype()
+                      << std::endl << std::flush;
+        }
         return;
     }
     debug(std::cout << "A " << op->getParents().front() << " op from client!" << std::endl << std::flush;);
@@ -118,6 +133,9 @@ void AdminClient::objectArrived(const Root & obj)
     }
 }
 
+/// \brief Called when an Info operation arrives
+///
+/// @param op Operation to be processed
 void AdminClient::infoArrived(const RootOperation & op)
 {
     reply_flag = true;
@@ -136,6 +154,9 @@ void AdminClient::infoArrived(const RootOperation & op)
     }
 }
 
+/// \brief Called when an Error operation arrives
+///
+/// @param op Operation to be processed
 void AdminClient::errorArrived(const RootOperation & op)
 {
     reply_flag = true;
@@ -151,6 +172,7 @@ void AdminClient::errorArrived(const RootOperation & op)
     }
 }
 
+/// \brief AdminClient constructor
 AdminClient::AdminClient() : error_flag(false), reply_flag(false),
                              login_flag(false), encoder(0), codec(0),
                              ios(0), exit(false)
@@ -170,7 +192,9 @@ AdminClient::~AdminClient()
     }
 }
 
-
+/// \brief Main client application loop
+///
+/// Check for incoming data until the client is ready to exit
 void AdminClient::loop()
 {
     while (!exit) {
@@ -178,8 +202,8 @@ void AdminClient::loop()
     };
 }
 
+/// \brief Poll the codec to see if data is available
 void AdminClient::poll()
-// poll the codec if select says there is something there.
 {
     fd_set infds;
     struct timeval tv;
@@ -193,18 +217,21 @@ void AdminClient::poll()
 
     int retval = select(cli_fd+1, &infds, NULL, NULL, &tv);
 
-    if (retval) {
-        if (FD_ISSET(cli_fd, &infds)) {
-            if (ios->peek() == -1) {
-                std::cerr << "Server disconnected" << std::endl << std::flush;
-                exit = true;
-            } else {
-                codec->poll();
-            }
+    if (retval < 1) {
+        return;
+    }
+
+    if (FD_ISSET(cli_fd, &infds)) {
+        if (ios->peek() == -1) {
+            std::cerr << "Server disconnected" << std::endl << std::flush;
+            exit = true;
+        } else {
+            codec->poll();
         }
     }
 }
 
+/// \brief Read login credentials from standard input
 void AdminClient::getLogin()
 {
     // This needs to be re-written to hide input, so the password can be
@@ -215,8 +242,10 @@ void AdminClient::getLogin()
     std::cin >> password;
 }
 
-int AdminClient::checkRule(const std::string & id)
+/// \brief Check if a rule exists in the rule table
+///
 /// @return 0 if a rule exists, 1 if it does not
+int AdminClient::checkRule(const std::string & id)
 {
     error_flag = false;
     reply_flag = false;
@@ -244,6 +273,11 @@ int AdminClient::checkRule(const std::string & id)
     return 1;
 }
 
+/// \brief Upload a rule description to the server
+///
+/// @param id Identifier of the rule to be uploaded
+/// @param set Name of the ruleset rule is from
+/// @param rule Atlas description of the rule
 int AdminClient::uploadRule(const std::string & id, const std::string & set,
                             const MapType & rule)
 {
@@ -252,6 +286,9 @@ int AdminClient::uploadRule(const std::string & id, const std::string & set,
     login_flag = false;
 
     if (m_uploadedRules.find(id) != m_uploadedRules.end()) {
+        std::cout << "Overriden rule " << id << " ignored."
+                  << std::endl << std::flush;
+
         return -1;
     }
 
@@ -320,7 +357,6 @@ int AdminClient::uploadRule(const std::string & id, const std::string & set,
         debug(std::cerr << "Rule \"" << id << "\" to be uploaded has parent \""
                         << parent << "\" which does not exist on server yet."
                         << std::endl << std::flush;);
-        // FIXME Make sure there is not a rule with the same ID already waiting
         RuleWaitList::const_iterator J = m_waitingRules.lower_bound(parent);
         RuleWaitList::const_iterator Jend = m_waitingRules.upper_bound(parent);
         for (; J != Jend; ++J) {
@@ -392,6 +428,9 @@ int AdminClient::uploadRule(const std::string & id, const std::string & set,
     return count;
 }
 
+/// \brief Connect to a remote server using a network socket
+///
+/// @param host Hostname where the server is running.
 int AdminClient::connect(const std::string & host)
 {
     tcp_socket_stream * stream = new tcp_socket_stream;
@@ -405,6 +444,9 @@ int AdminClient::connect(const std::string & host)
     return negotiate();
 }
 
+/// \brief Connect to a local server using a unix socket
+///
+/// @param filename Filename of unix socket where the server is running.
 int AdminClient::connect_unix(const std::string & filename)
 {
     unix_socket_stream * stream = new unix_socket_stream;
@@ -418,6 +460,7 @@ int AdminClient::connect_unix(const std::string & filename)
     return negotiate();
 }
 
+/// \brief Setup Atlas negotiation on a new server connection.
 int AdminClient::negotiate()
 {
     // Do client negotiation with the server
@@ -447,6 +490,10 @@ int AdminClient::negotiate()
 
 }
 
+/// \brief Keep polling the server connection until a response arrives.
+///
+/// An Info operation is typically expected but this function will also
+/// return if an Error operation arrives.
 void AdminClient::waitForInfo()
 {
     for (int i = 0; i < 10 && !reply_flag; ++i) {
@@ -454,6 +501,9 @@ void AdminClient::waitForInfo()
     }
 }
 
+/// \brief Send a Login operation to the remote server
+///
+/// This function uses credentials that have been set earlier.
 int AdminClient::login()
 {
     Account account;
@@ -481,6 +531,12 @@ int AdminClient::login()
     return -1;
 }
 
+/// \brief Report information about rules which didn't upload
+///
+/// When this client is used to upload rules to the server, sometimes it is
+/// not possible to upload some until their location in the inheritance
+/// tree has been found. This function reports any rules for which a place
+/// was never found, typically because its parent did not exist in the tree.
 void AdminClient::report()
 {
     if (m_waitingRules.empty()) {
